@@ -69,6 +69,19 @@ public class Method implements AttrContainer, Member {
     return method;
   }
 
+  /** A copy constructor, except you can override the declaring class.
+   * This can be used to improve binary compatibility by emitting method
+   * references where the declared class is the type of the receiver.
+   */
+  public Method (Method base, ClassType clas)
+  {
+    arg_types = base.arg_types;
+    return_type = base.return_type;
+    name = base.name;
+    access_flags = base.access_flags;
+    classfile = clas;
+  }
+
   Method (ClassType clfile, int flags)
   {
     if (clfile.last_method == null)
@@ -140,6 +153,7 @@ public class Method implements AttrContainer, Member {
       classfile.constants = new ConstantPool();
     prepareCode(0);
     code.sourceDbgExt = classfile.sourceDbgExt;
+    code.noteParamTypes();
     code.pushScope();
   }
 
@@ -230,21 +244,28 @@ public class Method implements AttrContainer, Member {
     Attribute.writeAll(this, dstr);
   }
 
+  public static String makeSignature (Type arg_types[], Type return_type)
+  {
+    /* #ifdef JAVA5 */
+    StringBuilder buf = new StringBuilder(100);
+    /* #else */
+    // StringBuffer buf = new StringBuffer(100);
+    /* #endif */
+    int args_count = arg_types.length; 
+    buf.append('(');
+    for (int i = 0; i < args_count; i++)
+      buf.append (arg_types[i].getSignature());
+    buf.append(')');
+    buf.append(return_type.getSignature());
+    return buf.toString();
+  }
+
   String signature;
 
   public String getSignature ()
   {
     if (signature == null)
-      {
-	StringBuffer buf = new StringBuffer(100);
-	int args_count = arg_types.length; 
-	buf.append('(');
-	for (int i = 0; i < args_count; i++)
-	  buf.append (arg_types[i].getSignature());
-	buf.append(')');
-	buf.append(return_type.getSignature());
-	signature = buf.toString();
-      }
+      signature = makeSignature(arg_types, return_type);
     return signature;
   }
 
@@ -255,9 +276,9 @@ public class Method implements AttrContainer, Member {
       throw new ClassFormatError("bad method signature");
     int pos = 1;
     /* #ifdef JAVA5 */
-    // java.util.Stack<Type> types = new java.util.Stack<Type>();
+    java.util.Stack<Type> types = new java.util.Stack<Type>();
     /* #else */
-    java.util.Stack types = new java.util.Stack();
+    // java.util.Stack types = new java.util.Stack();
     /* #endif */
     for (;;)
       {
