@@ -5,6 +5,8 @@ package gnu.bytecode;
 import java.util.*;
 import gnu.kawa.util.AbstractWeakHashTable;
 
+/** An abstract type as used by both gnu.bytecode and gnu.expr. */
+
 public abstract class Type
 /* #ifdef JAVA5 */
  implements java.lang.reflect.Type
@@ -34,6 +36,13 @@ public abstract class Type
     return this;
   }
 
+  /** If this is a type alias, get the aliased type.
+   */
+  public Type getRealType ()
+  {
+    return this;
+  }
+
   static ClassToTypeMap mapClassToType;
 
   /** Maps Java type name (e.g. "java.lang.String[]") to corresponding Type. */
@@ -219,16 +228,16 @@ public abstract class Type
 	type = signatureToPrimitive(c);
 	if (type != null)
 	  out.print(type.getName());
-        return;
       }
-    if (c == '[')
+    else if (c == '[')
       {
         printSignature(sig, off+1, len-1, out);
         out.print("[]");
-        return;
       }
-    if (c == 'L' && len > 2 && sig.indexOf(';', off) == len-1+off)
+    else if (c == 'L' && len > 2 && sig.indexOf(';', off) == len-1+off)
       out.print(sig.substring(off+1,len-1+off).replace('/', '.'));
+    else
+      out.append(sig, off, len-off);
   }
 
   /** Return the length of the signature starting at a given string position.
@@ -396,7 +405,7 @@ public abstract class Type
    * -1: if this is a pure subtype of other;
    * -2: if they have values in common but neither is a subtype of the other;
    * -3: if the types have no values in common.
-   * "Same member" is rather loose;  by "A is a subtype of B"
+   * "Same values" is rather loose;  by "A is a subtype of B"
    * we mean that all instance of A can be "widened" to B.
    * More formally, A.compare(B) returns:
    *  1: all B values can be converted to A without a coercion failure
@@ -446,6 +455,18 @@ public abstract class Type
     return obj;
   }
 
+  /** Convert from stackType (usually PrimType) to this type.
+   * However, we might only convert part-way, to some object type.
+   * If converting to this type might fail at run-time, only convert
+   * to Object (as by emitCoerceToObject); a caller can use
+   * {@code stackType.emitConvertFromObject} to convert the rest,
+   * but that might throw an exception. (This is a bit of a kludge.)
+   */
+  public void emitConvertFromPrimitive (Type stackType, CodeAttr code)
+  {
+    stackType.emitCoerceToObject(code);
+  }
+
   /** Compile code to convert a object of this type on the stack to Object. */
   public void emitCoerceToObject (CodeAttr code)
   {
