@@ -7,8 +7,8 @@ import gnu.lists.*;
 import gnu.mapping.*;
 import gnu.bytecode.CodeAttr;
 import gnu.bytecode.ClassType;
-import gnu.kawa.lispexpr.LispLanguage;
-import gnu.kawa.lispexpr.ReadTable;
+import gnu.text.*;
+import gnu.kawa.lispexpr.*;
 import gnu.kawa.reflect.FieldLocation;
 
 /** Abstract class for Lisp-like languages with separate namespaces. */
@@ -148,6 +148,8 @@ public abstract class Lisp2 extends LispLanguage
       {
         if (val instanceof Procedure || val instanceof kawa.lang.Syntax)
           defun(name, val);
+        else if(val instanceof LangObjType)
+          defun(name, ((LangObjType) val).getConstructor());
         else
           define(name.getName(), val);
       }
@@ -157,6 +159,7 @@ public abstract class Lisp2 extends LispLanguage
   {
     ReadTable tab = new Lisp2ReadTable();
     tab.initialize();
+    tab.setInitialColonIsKeyword(true);
     return tab;
   }
 }
