@@ -3,10 +3,10 @@
 ;;; limitations in the Common Lisp support making that difficult.
 
 (define (car x)
-  (if (eq? x '()) x (slot-ref (as <pair> x) 'car)))
+  (if (eq? x '()) x ((as <pair> x):getCar)))
 
 (define (cdr x)
-  (if (eq? x '()) x (slot-ref (as <pair> x) 'cdr)))
+  (if (eq? x '()) x ((as <pair> x):getCdr)))
 
 (define (setcar (p <pair>) x)
   (set-car! p x))
@@ -16,12 +16,12 @@
 
 ;; SYMBOLS
 
-(define (boundp symbol) :: |<clisp:boolean>|
+(define (boundp symbol) :: |clisp:boolean|
   ((primitive-static-method <gnu.commonlisp.lang.Symbols> "isBound"
                             <boolean> (<object>))
    symbol))
 
-(define (symbolp x) :: |<clisp:boolean>|
+(define (symbolp x) :: |clisp:boolean|
   (invoke-static <gnu.commonlisp.lang.Symbols> 'isSymbol x))
 
 (define (symbol-name symbol)
@@ -108,17 +108,16 @@
 		 symbol object))
 
 (define (apply func #!rest (args :: <Object[]>))
-  (invoke-static <gnu.kawa.functions.Apply> 'doApply
-		 (as <function>
-		     (if (symbol? func) (symbol-function func) func))
-		 args))
+  ((as <function>
+       (if (symbol? func) (symbol-function func) func)):applyN
+       (gnu.kawa.functions.Apply:getArguments args 0 apply)))
 
 ;;; ARRAYS
 
 (define (length (x :: <gnu.lists.Sequence>))
   (invoke x 'size))
 
-(define (arrayp x) <clisp:boolean>
+(define (arrayp x) |clisp:boolean|
   (instance? x <gnu.lists.SimpleVector>))
 
 (define (aref
@@ -138,11 +137,11 @@
 
 ;;; STRINGS
 
-(define (stringp x) <clisp:boolean>
+(define (stringp x) |clisp:boolean|
   (instance? x <string>))
 
 (define (make-string (count :: <int>) ch)
-  (make <string> count (invoke-static <gnu.commonlisp.lang.CommonLisp> 'asChar ch)))
+  (make <gnu.lists.FString> count (invoke-static <gnu.commonlisp.lang.CommonLisp> 'asChar ch)))
 
 (define (substring (str :: <string>) from #!optional (to '()))
   (if (eq? to '())
@@ -151,10 +150,10 @@
       (set! to (- (string-length str) to)))
   (if (< from 0)
       (set! from (- (string-length str) from)))
-  (invoke str 'copy from to))
+  (make <gnu.lists.FString> str (as <int> from) (as <int> (- to from))))
 
 (define (char-to-string ch)
-  (make <string> 1 (invoke-static <gnu.commonlisp.lang.CommonLisp> 'asChar ch)))
+  (make <gnu.lists.FString> 1 (invoke-static <gnu.commonlisp.lang.CommonLisp> 'asChar ch)))
 
 (define (functionp x) |<clisp:boolean>|
   (instance? x <function>))
