@@ -123,6 +123,7 @@ public class ApplyExp extends Expression
     LambdaExp func_lambda = null;
     String func_name = null;
     Declaration owner = null;
+    Object quotedValue = null;
     if (exp_func instanceof LambdaExp)
       {
 	func_lambda = (LambdaExp) exp_func;
@@ -151,26 +152,31 @@ public class ApplyExp extends Expression
 	    if (value != null && value instanceof LambdaExp) 
 	      func_lambda = (LambdaExp) value;
 	    if (value != null && value instanceof QuoteExp) 
-	      {
-		Object quotedValue = ((QuoteExp) value).getValue();
-		if (checkInlineable && quotedValue instanceof Inlineable)
-                  {
-                    ((Inlineable) quotedValue).compile(exp, comp, target);
-                    return;
-		  }
-	      }
+              quotedValue = ((QuoteExp) value).getValue();
 	  }
       }
     else if (exp_func instanceof QuoteExp)
       {
-        Object proc = ((QuoteExp) exp_func).getValue();
-	if (proc instanceof Inlineable)
-	  {
-            if (checkInlineable)
-              {
-                ((Inlineable) proc).compile(exp, comp, target);
-                return;
-              }
+        quotedValue = ((QuoteExp) exp_func).getValue();
+      }
+    if (checkInlineable && quotedValue instanceof Procedure)
+      {
+        Procedure proc = (Procedure) quotedValue;
+        if (target instanceof IgnoreTarget && proc.isSideEffectFree())
+          {
+            for (int i = 0; i < args_length;  i++)
+              exp.args[i].compile(comp, target);
+            return;
+          }
+        try
+          {
+            if (inlineCompile(proc, exp, comp, target))
+              return;
+          }
+        catch (Throwable ex)
+          {
+            comp.getMessages().error('e', "caught exception in inline-compiler for "+quotedValue+" - "+ex, ex);
+            return;
           }
       }
 
@@ -518,13 +524,26 @@ public class ApplyExp extends Expression
     this.type = type;
   }
 
-  public final gnu.bytecode.Type getType()
+  public boolean side_effects ()
+  {
+    Object value = derefFunc(func).valueIfConstant();
+    if (value instanceof Procedure
+        && ((Procedure) value).isSideEffectFree())
+      {
+        Expression[] a = args;
+        int alen = a.length;
+        for (int i = 0;  i < alen;  i++)
+          {
+            if (a[i].side_effects())
+              return true;
+          }
+        return false;
+      }
+    return true;
+  }
+
+  static Expression derefFunc(Expression afunc)
   {
-    if (type != null)
-      return type;
-    Expression afunc = func;
-    // In case of cycles.
-    type = Type.pointer_type;
     if (afunc instanceof ReferenceExp)
       {
 	Declaration func_decl = ((ReferenceExp) afunc).binding;
@@ -532,11 +551,23 @@ public class ApplyExp extends Expression
 	if (func_decl != null && ! func_decl.getFlag(Declaration.IS_UNKNOWN))
 	  afunc = func_decl.getValue();
       }
+    return afunc;
+  }
+
+  public final gnu.bytecode.Type getType()
+  {
+    if (type != null)
+      return type;
+    Expression afunc = derefFunc(func);
+    // In case of cycles.
+    type = Type.pointer_type;
     if (afunc instanceof QuoteExp)
       {
-	Object proc = ((QuoteExp) afunc).getValue();
-	if (proc instanceof Inlineable)
-	  type = ((Inlineable) proc).getReturnType(args);
+        Object value = ((QuoteExp) afunc).getValue();
+        Inlineable compiler;
+        if (value instanceof Procedure
+            && (compiler = asInlineable((Procedure) value)) != null)
+          type = compiler.getReturnType(args);
       }
     else if (afunc instanceof LambdaExp)
       {
@@ -545,6 +576,23 @@ public class ApplyExp extends Expression
     return type;
   }
 
+  public static Inlineable asInlineable (Procedure proc)
+  {
+    if (proc instanceof Inlineable)
+      return (Inlineable) proc;
+    return (Inlineable) Procedure.compilerKey.get(proc);
+  }
+
+  static boolean inlineCompile (Procedure proc, ApplyExp exp, Compilation comp, Target target)
+    throws Throwable
+  {
+    Inlineable compiler = asInlineable(proc);
+    if (compiler == null)
+      return false;
+    compiler.compile(exp, comp, target);
+    return true;
+  }
+
   public final Expression inlineIfConstant(Procedure proc, ExpWalker walker)
   {
     return inlineIfConstant(proc, walker.getMessages());
@@ -579,7 +627,7 @@ public class ApplyExp extends Expression
       }
     try
       {
-	return new QuoteExp(proc.applyN(vals));
+	return new QuoteExp(proc.applyN(vals), type);
       }
     catch (Throwable ex)
       {
