@@ -4,7 +4,7 @@
 package gnu.expr;
 import gnu.bytecode.*;
 import gnu.mapping.*;
-import java.util.Vector;
+import java.util.*;
 
 public class ClassExp extends LambdaExp
 {
@@ -16,6 +16,7 @@ public class ClassExp extends LambdaExp
   public static final int IS_ABSTRACT = LambdaExp.NEXT_AVAIL_FLAG;
   public static final int INTERFACE_SPECIFIED = 2 * LambdaExp.NEXT_AVAIL_FLAG;
   public static final int CLASS_SPECIFIED = 4 * LambdaExp.NEXT_AVAIL_FLAG;
+  public static final int HAS_SUBCLASS = 8 * LambdaExp.NEXT_AVAIL_FLAG;
 
   /** True if there is at least one explicit "<init>" ("*init*"} method. */
   boolean explicitInit;
@@ -50,19 +51,12 @@ public class ClassExp extends LambdaExp
 
   public ClassExp ()
   {
-    type = null;
-    // Make sure we actually generate a class.
-    setCanRead(true);
   }
 
   public ClassExp (boolean simple)
   {
     this.simple = simple;
-    if (simple)
-      instanceType = type = new ClassType();
-    else
-      instanceType = type = new PairClassType(); // For now.
-    setCanRead(true);
+    instanceType = type = new ClassType();
   }
 
   protected boolean mustCompile () { return true; }
@@ -78,7 +72,6 @@ public class ClassExp extends LambdaExp
   public void compilePushClass (Compilation comp, Target target)
   {
     ClassType new_class = type;
-    // Type.make(Class.forname)
 
     gnu.bytecode.CodeAttr code = comp.getCode();
     comp.loadClassRef(new_class);
@@ -161,14 +154,16 @@ public class ClassExp extends LambdaExp
       }
     if (superType != null && (flags & INTERFACE_SPECIFIED) != 0)
       comp.error('e', "cannot be interface since has superclass");
-    if (! simple && superType == null && (flags & CLASS_SPECIFIED) == 0)
+    if (! simple && superType == null && (flags & CLASS_SPECIFIED) == 0
+        && (getFlag(HAS_SUBCLASS)
+            || (nameDecl != null && nameDecl.isPublic())))
       {
-        PairClassType ptype = (PairClassType) type;
-        instanceType = new ClassType();
+        PairClassType ptype = new PairClassType();//(PairClassType) type;
+        type = ptype;
         ptype.setInterface(true);
         ptype.instanceType = instanceType;
         ClassType[] interfaces = { type };
-        // Can we better.  FIXME.
+        // Can do better.  FIXME.
         instanceType.setSuper(Type.pointer_type);
         instanceType.setInterfaces(interfaces);
       }
@@ -273,6 +268,8 @@ public class ClassExp extends LambdaExp
     if (partsDeclared)
       return;
     partsDeclared = true;
+    Hashtable<String,Declaration> seenFields
+      = new Hashtable<String,Declaration>();
     for (Declaration decl = firstDecl();
 	 decl != null;  decl = decl.nextDecl())
       {
@@ -299,6 +296,10 @@ public class ClassExp extends LambdaExp
 		decl.field
 		  = instanceType.addField(fname, decl.getType(), flags);
 		decl.setSimple(false);
+                Declaration old = seenFields.get(fname);
+                if (old != null)
+                  duplicateDeclarationError(old, decl, comp);
+                seenFields.put(fname, decl);
 	      }
 	  }
       }
@@ -313,6 +314,8 @@ public class ClassExp extends LambdaExp
             explicitInit = true;
             if (child.isAbstract())
               comp.error('e', "*init* method cannot be abstract", child);
+            if (type instanceof PairClassType)
+              comp.error('e', "'*init*' methods only supported for simple classes");
           }
         // Setting child.outer isn't normally needed.  The exception is
         // if we're called from object.rewriteClassDef and there is some
@@ -517,7 +520,7 @@ public class ClassExp extends LambdaExp
                     if (calledInit != instanceType && calledInit != superClass)
                       comp.error('e', "call to <init> for not this or super class");
                   }
-                else if (calledInit != superClass)
+                else if (superClass != null)
                   {
                     // Call default super constructor if there isn't an explicit
                     // call to a super constructor.
@@ -616,7 +619,7 @@ public class ClassExp extends LambdaExp
 		    String msg = vec.size() == 0
 		      ? "missing implementation for "
 		      : "ambiguous implementation for ";
-		    comp.error('e', msg+meth+" mname:"+mname);
+		    comp.error('e', msg+meth);
 		  }
 		else
 		  {
