@@ -10,6 +10,7 @@ import kawa.Shell;
 import gnu.text.*;
 import java.util.zip.*;
 import java.util.Stack;
+import gnu.kawa.functions.Convert;
 
 /** State for a single expression or module.
  * For each top-level thing (expression or file) we compile or evaluate
@@ -135,10 +136,10 @@ public class Compilation implements SourceLocator
 
   public static int defaultClassFileVersion =
     /* #ifdef JAVA6 */
-    // ClassType.JDK_1_6_VERSION
+    ClassType.JDK_1_6_VERSION
     /* #else */
     /* #ifdef JAVA5 */
-    ClassType.JDK_1_5_VERSION
+    // ClassType.JDK_1_5_VERSION
     /* #else */
     // ClassType.JDK_1_1_VERSION
     /* #endif */
@@ -163,10 +164,14 @@ public class Compilation implements SourceLocator
   public boolean usingTailCalls()
   { return defaultCallConvention >= CALL_WITH_TAILCALLS; }
 
+  public static final int MODULE_NONSTATIC = -1;
+  public static final int MODULE_STATIC_DEFAULT = 0;
+  public static final int MODULE_STATIC = 1;
+  public static final int MODULE_STATIC_RUN = 2;
   /** If moduleStatic > 0, (module-static #t) is implied by default.
    * If moduleStatic == 2, <clinit> calls run.
    * If moduleStatic < 0, (module-static #f) is implied by default. */
-  public static int moduleStatic = 0;
+  public static int moduleStatic = MODULE_STATIC_DEFAULT;
 
   ClassType[] classes;
   int numClasses;
@@ -178,6 +183,9 @@ public class Compilation implements SourceLocator
   /** True if the compiled result will be immediately loaded. */ 
   public boolean immediate;
 
+  /** Compilation was explicitly requested, rather than being a dependency. */
+  public boolean explicit;
+
   /** The current method. */
   public Method method;
 
@@ -364,13 +372,11 @@ public class Compilation implements SourceLocator
 
   LitTable litTable;
 
-  public static boolean generateAppletDefault = false;
   /** True if we should generate an Applet. */
-  public boolean generateApplet = generateAppletDefault;
+  public boolean generateApplet;
 
-  public static boolean generateServletDefault = false;
   /** True if we should generate an Servlet. */
-  public boolean generateServlet = generateServletDefault;
+  public boolean generateServlet;
 
   public final ClassType getModuleType()
   {
@@ -946,8 +952,7 @@ public class Compilation implements SourceLocator
 	  new File(parent).mkdirs();
 	clas.writeToFile(out_name);
       }
-    cleanupAfterCompilation();
-    litTable = null;
+    minfo.cleanupAfterCompilation();
   }
 
   public void cleanupAfterCompilation ()
@@ -961,6 +966,8 @@ public class Compilation implements SourceLocator
       minfo.exp.body = null;
     mainLambda.body = null;
     mainLambda = null;
+    if (! immediate)
+      litTable = null;
   }
 
   public void compileToArchive (ModuleExp mexp, String fname)
@@ -1054,10 +1061,7 @@ public class Compilation implements SourceLocator
 
   public void addMainClass (ModuleExp module)
   {
-    mustCompile = true;
-
     mainClass = module.classFor(this);
-
     ClassType type = mainClass;
     ClassType[] interfaces = module.getInterfaces();
     if (interfaces != null)
@@ -1864,10 +1868,24 @@ public class Compilation implements SourceLocator
           }
         if (wantedState >= RESOLVED && getState() < RESOLVED)
           {
+            // Doing addMainClass is a bit flakey in the case that
+            // ModuleExp.alwaysCompile is false.  We don't want to
+            // call addMainClass *unless* we're compiling, but when
+            // dealing with eval, mutually recursive modules, etc
+            // it doesn't quite work.
             addMainClass(mexp);
             language.resolve(this);
             setState(messages.seenErrors() ? ERROR_SEEN : RESOLVED);
           }
+
+        // Avoid writing class needlessly.
+        if (! explicit && ! immediate
+            && minfo.checkCurrent(ModuleManager.getInstance(), System.currentTimeMillis()))
+          {
+            minfo.cleanupAfterCompilation();
+            setState(CLASS_WRITTEN);
+          }
+
         if (wantedState >= WALKED && getState() < WALKED)
           {
             walkModule(mexp);
@@ -2053,7 +2071,7 @@ public class Compilation implements SourceLocator
             dumpInitializers(init);
           }
 
-	if (! immediate && module.staticInitRun())
+	if (module.staticInitRun())
 	  {
 	    code.emitGetStatic(moduleInstanceMainField);
 	    code.emitInvoke(typeModuleBody.getDeclaredMethod("run", 0));
@@ -2171,7 +2189,8 @@ public class Compilation implements SourceLocator
 				  .getDeclaredMethod("setDefaultFormat", 1));
 	  }
 	code.emitLoad(code.getArg(0));
-	code.emitInvokeStatic(typeModuleBody.getDeclaredMethod("processArgs", 1));
+	code.emitInvokeStatic(ClassType.make("gnu.expr.ApplicationMainSupport")
+                              .getDeclaredMethod("processArgs", 1));
 	if (moduleInstanceMainField != null)
 	  code.emitGetStatic(moduleInstanceMainField);
 	else
@@ -2238,7 +2257,7 @@ public class Compilation implements SourceLocator
         for (int i = manager.numModules;  --i >= 0; )
           {
             ModuleInfo mi = manager.modules[i];
-            String miClassName = mi.className;
+            String miClassName = mi.getClassName();
             if (miClassName == null
                 || ! miClassName.startsWith(mainPrefix))
               continue;
@@ -2352,6 +2371,8 @@ public class Compilation implements SourceLocator
    */
   public void mustCompileHere ()
   {
+    if (! mustCompile && ! ModuleExp.compilerAvailable)
+      error('e', "this expression must be compiled, but compiler is unavailable");
     mustCompile = true;
   }
 
@@ -2404,7 +2425,7 @@ public class Compilation implements SourceLocator
     ModuleExp module = new ModuleExp();
     if (filename != null)
       module.setFile(filename);
-    if (Compilation.generateAppletDefault)
+    if (generateApplet || generateServlet)
       module.setFlag(ModuleExp.SUPERTYPE_SPECIFIED);
     if (immediate)
       {
@@ -2425,10 +2446,12 @@ public class Compilation implements SourceLocator
   public final void pushScope (ScopeExp scope)
   {
     if (! mustCompile
-        // We set mustCompile if we see a LambdaExp - not because we must
-        // but because it is usually desirable.
         && (scope.mustCompile()
-            || (scope instanceof LambdaExp && ! (scope instanceof ModuleExp))))
+            || (ModuleExp.compilerAvailable
+                // We set mustCompile if we see a LambdaExp - not because
+                // we must but because it is usually desirable.
+                && scope instanceof LambdaExp
+                && ! (scope instanceof ModuleExp))))
       mustCompileHere();
     scope.outer = current_scope;
     current_scope = scope;
@@ -2682,6 +2705,32 @@ public class Compilation implements SourceLocator
     return loopRepeat(args);
   }
 
+  /**
+   * Convenience method to make an Expression that coerces a value.
+   * @param value to be coerced
+   * @param type to coerce value to
+   * @return expression that coerces value to type
+   */
+  public static ApplyExp makeCoercion(Expression value, Expression type)
+  {
+    Expression[] exps = new Expression[2];
+    exps[0] = type;
+    exps[1] = value;
+    QuoteExp c = new QuoteExp(Convert.getInstance());
+    return new ApplyExp(c, exps);
+  }
+
+  /**
+   * Convenience method to make an Expression that coerces a value.
+   * @param value to be coerced
+   * @param type to coerce value to
+   * @return expression that coerces value to type
+   */
+  public static Expression makeCoercion(Expression value, Type type)
+  {
+    return makeCoercion(value, new QuoteExp(type));
+  }
+
   /** If non-null, a helper method generated by getForNameHelper. */
   Method forNameHelper;
 
