@@ -1,4 +1,4 @@
-// Copyright (c) 2003  Per M.A. Bothner.
+// Copyright (c) 2003, 2009  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -56,10 +56,24 @@ public class Declaration
 
   public ScopeExp context;
 
+  /** The type of the value of this Declaration.
+   * It is null if the type is un-specified and not yet inferred.
+   * Will get set implicitly by getType, to avoid inconsistencies.
+   */
   protected Type type;
   protected Expression typeExp;
-  public final Expression getTypeExp() { return typeExp; }
-  public final Type getType() { return type; }
+  public final Expression getTypeExp()
+  {
+    if (typeExp == null)
+      setType(Type.objectType);
+    return typeExp;
+  }
+  public final Type getType()
+  {
+    if (type == null)
+      setType(Type.objectType);
+    return type;
+  }
   public final void setType(Type type)
   {
     this.type = type;
@@ -70,20 +84,12 @@ public class Declaration
   public final void setTypeExp (Expression typeExp)
   {
     this.typeExp = typeExp;
-    Type t;
+    Type t = null;
     if (typeExp instanceof TypeValue)
       t = ((TypeValue) typeExp).getImplementationType();
-    else if (typeExp instanceof QuoteExp)
-      {
-        Object typeValue = ((QuoteExp) typeExp).getValue();
-        if (typeValue instanceof Type)
-          t = (Type) typeValue;
-        else if (typeValue instanceof Class)
-          t = Type.make((Class) typeValue);
-        else
-          t = Type.pointer_type;
-      }
     else
+      t = Language.getDefaultLanguage().getTypeFor(typeExp, false);
+    if (t == null)
       t = Type.pointer_type;
     this.type = t;
     if (var != null) var.setType(t);
@@ -126,7 +132,7 @@ public class Declaration
   public final void setSyntax ()
   {
     setSimple(false);
-    setFlag(Declaration.IS_CONSTANT | Declaration.IS_SYNTAX);
+    setFlag(IS_CONSTANT|IS_SYNTAX|EARLY_INIT);
   }
 
   /** Return the ScopeExp that contains (declares) this Declaration. */
@@ -173,7 +179,8 @@ public class Declaration
       }
     CodeAttr code = comp.getCode();
     Type rtype = getType();
-    if ((flags & ReferenceExp.CREATE_FIELD_REFERENCE) != 0)
+    if (! isIndirectBinding()
+        && (flags & ReferenceExp.DONT_DEREFERENCE) != 0)
       {
         if (field == null)
           throw new Error("internal error: cannot take location of "+this);
@@ -235,6 +242,11 @@ public class Declaration
             comp.compileConstant(val, target);
             return;
           }
+        else if (value != QuoteExp.undefined_exp && ignorable())
+          {
+            value.compile(comp, target);
+            return;
+          }
         else
           {
             Variable var = getVariable();
@@ -346,15 +358,30 @@ public class Declaration
    */
   public final Expression getValue()
   {
-    if (value == QuoteExp.undefined_exp
-        && field != null
-        && ((field.getModifiers() & Access.STATIC+Access.FINAL)
-            == Access.STATIC+Access.FINAL)
-        && ! isIndirectBinding())
+    if (value == QuoteExp.undefined_exp)
+      {
+        if (field != null
+            && ((field.getModifiers() & Access.STATIC+Access.FINAL)
+                == Access.STATIC+Access.FINAL)
+            && ! isIndirectBinding())
+          {
+            try
+              {
+                value = new QuoteExp(field.getReflectField().get(null));
+              }
+            catch (Throwable ex)
+              {
+              }
+          }
+      }
+    else if (value instanceof QuoteExp && getFlag(TYPE_SPECIFIED)
+             && value.getType() != type)
       {
         try
           {
-            value = new QuoteExp(field.getReflectField().get(null));
+            Object val = ((QuoteExp) value).getValue();
+            Type t = getType();
+            value = new QuoteExp(t.coerceFromObject(val), t);
           }
         catch (Throwable ex)
           {
@@ -382,6 +409,16 @@ public class Declaration
     return (v instanceof QuoteExp) && v != QuoteExp.undefined_exp;
   }
 
+  boolean shouldEarlyInit ()
+  {
+    return getFlag(EARLY_INIT) || isCompiletimeConstant ();
+  }
+
+  public boolean isCompiletimeConstant ()
+  {
+    return getFlag(IS_CONSTANT) && hasConstantValue();
+  }
+
   /** This prefix is prepended to field names for unknown names. */
   static final String UNKNOWN_PREFIX = "loc$";
 
@@ -461,25 +498,28 @@ public class Declaration
   public static final int IS_DYNAMIC = 0x10000000;
 
   /** Initialize in {@code <init>}/{@code <clinit>}
-   * rather than in {@code run}/{@code $run$}>. */
+   * rather than in {@code run}/{@code $run$}. */
   public static final int EARLY_INIT = 0x20000000;
   /** A reference to a module instance. */
   public static final int MODULE_REFERENCE = 0x40000000;
 
-  protected int flags = IS_SIMPLE;
+  public static final long VOLATILE_ACCESS = 0x80000000;
+  public static final long TRANSIENT_ACCESS = 0x100000000l;
 
-  public final boolean getFlag (int flag)
+  protected long flags = IS_SIMPLE;
+
+  public final boolean getFlag (long flag)
   {
     return (flags & flag) != 0;
   }
 
-  public final void setFlag (boolean setting, int flag)
+  public final void setFlag (boolean setting, long flag)
   {
     if (setting) flags |= flag;
     else flags &= ~flag;
   }
 
-  public final void setFlag (int flag)
+  public final void setFlag (long flag)
   {
     flags |= flag;
   }
@@ -496,15 +536,24 @@ public class Declaration
 
   public short getAccessFlags (short defaultFlags)
   {
-    if (getFlag(Declaration.PRIVATE_ACCESS))
-      return Access.PRIVATE;
-    if (getFlag(Declaration.PROTECTED_ACCESS))
-      return Access.PROTECTED;
-    if (getFlag(Declaration.PACKAGE_ACCESS))
-      return 0;
-    if (getFlag(Declaration.PUBLIC_ACCESS))
-      return Access.PUBLIC;
-    return defaultFlags;
+    short flags;
+    if (getFlag(PRIVATE_ACCESS|PROTECTED_ACCESS|PACKAGE_ACCESS|PUBLIC_ACCESS))
+      {
+        flags = 0;
+        if (getFlag(PRIVATE_ACCESS))
+          flags |= Access.PRIVATE;
+        if (getFlag(PROTECTED_ACCESS))
+          flags |= Access.PROTECTED;
+        if (getFlag(PUBLIC_ACCESS))
+          flags |= Access.PUBLIC;
+      }
+    else
+      flags = defaultFlags;
+    if (getFlag(VOLATILE_ACCESS))
+      flags |= Access.VOLATILE;
+    if (getFlag(TRANSIENT_ACCESS))
+      flags |= Access.TRANSIENT;
+    return flags;
   }
 
   public final boolean isAlias() { return (flags & IS_ALIAS) != 0; }
@@ -536,6 +585,13 @@ public class Declaration
     setFlag(indirectBinding, INDIRECT_BINDING);
   }
 
+  public void maybeIndirectBinding (Compilation comp)
+  {
+    if (isLexical() 
+        && ! (context instanceof ModuleExp) || context == comp.mainLambda)
+      setIndirectBinding(true);
+  }
+
   /* Note:  You probably want to use !ignorable(). */
   public final boolean getCanRead() { return (flags & CAN_READ) != 0; }
   public final void setCanRead(boolean read)
@@ -609,7 +665,10 @@ public class Declaration
 
   public boolean isStatic()
   {
-    if (getFlag(STATIC_SPECIFIED))
+    if (field != null)
+      return field.getStaticFlag();
+    if (getFlag(STATIC_SPECIFIED)
+        || isCompiletimeConstant())
       return true;
     if (getFlag(NONSTATIC_SPECIFIED))
       return false;
@@ -662,12 +721,12 @@ public class Declaration
 
   public Declaration (Object name)
   {
-    this(name, Type.pointer_type);
+    setName(name);
   }
 
-  public Declaration (Object s, Type type)
+  public Declaration (Object name, Type type)
   {
-    setName(s);
+    setName(name);
     setType(type);
   }
 
@@ -819,7 +878,7 @@ public class Declaration
       }
     */
     sbuf.append("/fl:");
-    sbuf.append(Integer.toHexString(flags));
+    sbuf.append(Long.toHexString(flags));
     if (ignorable())
       sbuf.append("(ignorable)");
     Expression tx = typeExp;
@@ -877,7 +936,8 @@ public class Declaration
     int fflags = 0;
     boolean isConstant = getFlag(IS_CONSTANT);
     boolean typeSpecified = getFlag(TYPE_SPECIFIED);
-    if (isPublic() && ! isConstant && ! typeSpecified)
+    if (comp.immediate && context instanceof ModuleExp
+        && ! isConstant && ! typeSpecified)
       setIndirectBinding(true);
     // In immediate mode we may need to access the field from a future
     // command in a different "runtime package" (see JVM spec) because it
@@ -885,7 +945,6 @@ public class Declaration
     if (isPublic() || external_access || comp.immediate)
       fflags |= Access.PUBLIC;
     if (isStatic()
-	|| (isConstant && value instanceof QuoteExp)
         // "Dynamic" variables use ThreadLocation, based on the current
         // Environment, so we don't need more than one static field.
         || (getFlag(Declaration.IS_UNKNOWN
@@ -894,56 +953,59 @@ public class Declaration
 	|| (value instanceof ClassExp
 	    && ! ((LambdaExp) value).getNeedsClosureEnv()))
       fflags |= Access.STATIC;
-    if ((isIndirectBinding() || isConstant)
+    if ((isIndirectBinding()
+         || (isConstant
+             && (shouldEarlyInit()
+                 || (context instanceof ModuleExp && ((ModuleExp) context).staticInitRun()))))
         && (context instanceof ClassExp || context instanceof ModuleExp))
       fflags |= Access.FINAL;
     Type ftype = getType().getImplementationType();
     if (isIndirectBinding() && ! ftype.isSubtype(Compilation.typeLocation))
-      if (getFlag(EARLY_INIT) && isAlias())
-        ftype = ClassType.make("gnu.kawa.reflect.FieldLocation");
-      else
-        ftype = Compilation.typeLocation;
-    String fname = getName();
-    int nlength;
-    if (fname==null)
+      ftype = Compilation.typeLocation;
+    if (! ignorable())
       {
-        fname = "$unnamed$0";
-        nlength = fname.length() - 2; // Without the "$0".
-      }
-    else
-      {
-        fname = Compilation.mangleNameIfNeeded(fname);
-        if (getFlag(IS_UNKNOWN))
-          fname = UNKNOWN_PREFIX + fname;
-        if (external_access && ! getFlag(Declaration.MODULE_REFERENCE))
-          fname = PRIVATE_PREFIX + fname;
-        nlength = fname.length();
-      }
-    int counter = 0;
-    while (frameType.getDeclaredField(fname) != null)
-      fname = fname.substring(0, nlength) + '$' + (++ counter);
-
-    field = frameType.addField (fname, ftype, fflags);
-    if (value instanceof QuoteExp)
-      {
-	Object val = ((QuoteExp) value).getValue();
-	if (val.getClass().getName().equals(ftype.getName()))
-	  {
-	    Literal literal = comp.litTable.findLiteral(val);
-	    if (literal.field == null)
-	      literal.assign(field, comp.litTable);
-	  }
-	else if (ftype instanceof PrimType
-		 || "java.lang.String".equals(ftype.getName()))
-	  {
-	    if (val instanceof gnu.text.Char)
-	      val = gnu.math.IntNum.make(((gnu.text.Char) val).intValue());
-	    field.setConstantValue(val, frameType);
-	    return;
-	  }
+        String fname = getName();
+        int nlength;
+        if (fname==null)
+          {
+            fname = "$unnamed$0";
+            nlength = fname.length() - 2; // Without the "$0".
+          }
+        else
+          {
+            fname = Compilation.mangleNameIfNeeded(fname);
+            if (getFlag(IS_UNKNOWN))
+              fname = UNKNOWN_PREFIX + fname;
+            if (external_access && ! getFlag(Declaration.MODULE_REFERENCE))
+              fname = PRIVATE_PREFIX + fname;
+            nlength = fname.length();
+          }
+        int counter = 0;
+        while (frameType.getDeclaredField(fname) != null)
+          fname = fname.substring(0, nlength) + '$' + (++ counter);
+        field = frameType.addField (fname, ftype, fflags);
+        if (value instanceof QuoteExp)
+          {
+            Object val = ((QuoteExp) value).getValue();
+            if (field.getStaticFlag()
+                  && val.getClass().getName().equals(ftype.getName()))
+              {
+                Literal literal = comp.litTable.findLiteral(val);
+                if (literal.field == null)
+                  literal.assign(field, comp.litTable);
+              }
+            else if (ftype instanceof PrimType
+                     || "java.lang.String".equals(ftype.getName()))
+              {
+                if (val instanceof gnu.text.Char)
+                  val = gnu.math.IntNum.make(((gnu.text.Char) val).intValue());
+                field.setConstantValue(val, frameType);
+                return;
+              }
+          }
       }
     // The EARLY_INIT case is handled in SetExp.compile.
-    if (! getFlag(EARLY_INIT)
+    if (! shouldEarlyInit()
 	&& (isIndirectBinding()
 	    || (value != null && ! (value instanceof ClassExp))))
       {
