@@ -377,18 +377,22 @@ public abstract class Language
   public void loadClass(String name)
     throws java.lang.ClassNotFoundException
   {
+    Class clas;
     try
       {
-	Class clas = Class.forName(name);
-	Object inst = clas.newInstance ();
-	ClassMemberLocation.defineAll(inst, this, Environment.getCurrent());
-	if (inst instanceof ModuleBody)
-	  ((ModuleBody)inst).run();
+        clas = Class.forName(name);
       }
     catch (java.lang.ClassNotFoundException ex)
       {
 	throw ex;
       }
+    try
+      {
+	Object inst = clas.newInstance ();
+	ClassMemberLocation.defineAll(inst, this, Environment.getCurrent());
+	if (inst instanceof ModuleBody)
+	  ((ModuleBody)inst).run();
+      }
     catch (Exception ex)
       {
 	throw new WrappedException("cannot load "+name, ex);
@@ -448,6 +452,10 @@ public abstract class Language
    * any module import. */
   public static final int PARSE_PROLOG = 8;
   public static final int PARSE_FOR_EVAL = PARSE_IMMEDIATE|PARSE_CURRENT_NAMES;
+  public static final int PARSE_FOR_APPLET = 16;
+  public static final int PARSE_FOR_SERVLET = 32;
+  /** Compilation explicitly requested, not just because of an import. */
+  public static final int PARSE_EXPLICIT = 64;
 
   public static boolean requirePedantic;
 
@@ -494,7 +502,13 @@ public abstract class Language
     Compilation tr = getCompilation(lexer, messages, lexical);
     if (requirePedantic)
       tr.pedantic = true;
-    tr.immediate = immediate;;
+    tr.immediate = immediate;
+    if ((options & PARSE_FOR_APPLET) != 0)
+      tr.generateApplet = true;
+    if ((options & PARSE_FOR_SERVLET) != 0)
+      tr.generateServlet = true;
+    if ((options & PARSE_EXPLICIT) != 0)
+      tr.explicit = true;
     if ((options & PARSE_PROLOG) != 0)
       tr.setState(Compilation.PROLOG_PARSING);
     tr.pushNewModule(lexer);
@@ -653,6 +667,35 @@ public abstract class Language
     return null;
   }
 
+  public static/* for now */ Type unionType (Type t1, Type t2)
+  {
+    if (t1 == Type.toStringType)
+      t1 = Type.javalangStringType;
+    if (t2 == Type.toStringType)
+      t2 = Type.javalangStringType;
+    if (t1 == t2)
+      return t1;
+    if (t1 instanceof PrimType && t2 instanceof PrimType)
+      {
+        char sig1 = t1.getSignature().charAt(0);
+        char sig2 = t2.getSignature().charAt(0);
+        if (sig1 == sig2)
+          return t1;
+        if ((sig1 == 'B' || sig1 == 'S' || sig1 == 'I') && (sig2 == 'I' || sig2 == 'J'))
+          return t2;
+        if ((sig2 == 'B' || sig2 == 'S' || sig2 == 'I') && (sig1 == 'I' || sig1 == 'J'))
+          return t1;
+        if (sig1 == 'F' && sig2 == 'D')
+          return t2;
+        if (sig2 == 'F' && sig1 == 'D')
+          return t1;
+        return Type.objectType;
+      }
+    // FIXME handle class types better:
+    // Type.lowestCommonSuperType(t1, t2);
+    return Type.objectType;
+  }
+
   public Declaration declFromField (ModuleExp mod, Object fvalue, Field fld)
   {
     String fname = fld.getName();
@@ -663,9 +706,10 @@ public abstract class Language
     // get name from value of field.
     boolean isImportedInstance;
     boolean externalAccess = false;
+    boolean isFinal = (fld.getModifiers() & Access.FINAL) != 0;
     if ((isImportedInstance = fname.endsWith("$instance")))
       fdname = fname;
-    else if (fvalue instanceof Named) // && ! isAlias
+    else if (isFinal && fvalue instanceof Named) // && ! isAlias
       fdname = ((Named) fvalue).getSymbol();
     else
       {
@@ -691,7 +735,6 @@ public abstract class Language
       : getTypeFor(ftype.getReflectClass());
     Declaration fdecl = mod.addDeclaration(fdname, dtype);
     boolean isStatic = (fld.getModifiers() & Access.STATIC) != 0;
-    boolean isFinal = (fld.getModifiers() & Access.FINAL) != 0;
     if (isAlias)
       fdecl.setIndirectBinding(true);
     else if (isFinal && ftype instanceof ClassType)
@@ -767,11 +810,6 @@ public abstract class Language
     return getTypeFor(clas).coerceToObject(obj);
   }
 
-  public Object coerceToObject(int val)
-  {
-    return gnu.math.IntNum.make(val);
-  }
-
   public static synchronized void setDefaults (Language lang)
   {
     Language.setDefaultLanguage(lang);
