@@ -19,6 +19,12 @@ public class LetExp extends ScopeExp
 
   protected boolean mustCompile () { return false; }
 
+  protected Object evalVariable (int i, CallContext ctx) throws Throwable
+  {
+    Expression init = inits[i];
+    return init.eval(ctx);
+  }
+
   public void apply (CallContext ctx) throws Throwable
   {
     setIndexes();
@@ -38,6 +44,7 @@ public class LetExp extends ScopeExp
         System.arraycopy(evalFrames, 0, newFrames, 0, evalFrames.length);
         ctx.evalFrames = evalFrames = newFrames;
       }
+    Object[] oldFrame = evalFrames[level]; // usually null
     evalFrames[level] = evalFrame;
 
     try
@@ -46,10 +53,9 @@ public class LetExp extends ScopeExp
         for (Declaration decl = firstDecl(); decl != null;
              decl = decl.nextDecl(), i++)
           {
-            Expression init = inits[i];
-            if (init == QuoteExp.undefined_exp)
+            if (inits[i] == QuoteExp.undefined_exp)
               continue;
-            Object value = init.eval(ctx);
+            Object value = evalVariable(i, ctx);
             Type type = decl.type;
             if (type != null && type != Type.pointer_type)
               value = type.coerceFromObject(value);
@@ -65,7 +71,7 @@ public class LetExp extends ScopeExp
       }
     finally
       {
-        evalFrames[level] = null;
+        evalFrames[level] = oldFrame;
       }
   }
 
@@ -169,7 +175,7 @@ public class LetExp extends ScopeExp
 	else
 	  {
 	    Type varType = decl.getType();
-	    varTarget = CheckedTarget.getInstance(varType);
+            varTarget = CheckedTarget.getInstance(decl);
 	    if (init == QuoteExp.undefined_exp)
 	      {
 		// Typically created by letrec.
