@@ -95,11 +95,34 @@ public abstract class ModuleBody extends Procedure0
     mainPrintValues = value;
   }
 
-  /** This is invoked by main when ModuleBody is compiled with --main. */
-  public static void processArgs (String[] args)
+  /** Number of times exitDecrement calls before we exit. */
+  private static int exitCounter;
+  /** See exitDecrement. */
+  public static synchronized void exitIncrement()
   {
-    int iArg = kawa.repl.processArgs(args, 0, args.length);
-    kawa.repl.setArgs(args, iArg);
+    if (exitCounter == 0)
+      exitCounter++;
+    exitCounter++;
+  }
+
+  /** Work around an AWT bug, where AWT threads are non-daemon.
+   * Thus if you start up AWT, the JVM will wait for the AWT to finish,
+   * even if there are no other non-daemon threads.
+   * So call exitIncrement() each time a Freme is created,
+   * and call exitDecrement() a Frame is closed. */
+  public static synchronized void exitDecrement()
+  {
+    int counter = exitCounter;
+    if (counter > 0)
+      {
+	counter--;
+	if (counter == 0)
+	  {
+	    System.exit(0);
+	  }
+	else
+	  exitCounter = counter;
+      }
   }
 
   /** This is invoked by main when ModuleBody is compiled with --main. */
@@ -109,8 +132,6 @@ public abstract class ModuleBody extends Procedure0
     try
       {
 	CallContext ctx = CallContext.getInstance();
-        // FIXME - not appropriate for XQuery.
-	ClassMemberLocation.defineAll(this, Language.getDefaultLanguage(), ctx.getEnvironment());
 	if (getMainPrintValues())
 	  {
 	    OutPort out = OutPort.outDefault();
@@ -126,7 +147,7 @@ public abstract class ModuleBody extends Procedure0
 	  }
 	// Redundant if registerShutdownHook succeeded (e.g on JDK 1.3).
 	gnu.mapping.OutPort.runCleanups();
-	kawa.repl.exitDecrement();
+	exitDecrement();
       }
     catch (Throwable ex)
       {
