@@ -41,6 +41,9 @@ public class ModuleExp extends LambdaExp
    * if there is a race condition on the counter.) */
   public static int interactiveCounter;
 
+  /** Compile to a class for immediate evaluation.
+   * Return null on error, if so errors go to comp.getMessages().
+   */
   public static Class evalToClass (Compilation comp, URL url)
     throws SyntaxException
   {
@@ -160,8 +163,14 @@ public class ModuleExp extends LambdaExp
       }
   }
 
+  // TODO: This should be false #ifdef Android.
+  // The complication is that it needs to be true while building
+  // Kawa itself, or generally compiling code *for* Android.
+  // I.e. we need to be able to distinguish compile-time and run-time.
+  public static boolean compilerAvailable = true;
+
   /** Flag to force compilation, even when not required. */
-  public static boolean alwaysCompile = true;
+  public static boolean alwaysCompile = compilerAvailable;
 
   public final static boolean evalModule (Environment env, CallContext ctx,
                                        Compilation comp, URL url,
@@ -173,17 +182,21 @@ public class ModuleExp extends LambdaExp
     Object inst = evalModule1(env, comp, url, msg);
     if (inst == null)
       return false;
-    evalModule2(env, ctx, language, mexp, inst, msg);
+    evalModule2(env, ctx, language, mexp, inst);
     return true;
   }
 
+  /** Parse and compile a module.
+   * @return null on error; otherwise a "cookie" that can be passed
+   * to evalModule2 or CompiledModule.
+   */
   public final static Object evalModule1 (Environment env,
                                           Compilation comp, URL url,
                                           OutPort msg)
     throws SyntaxException
   {
-    comp.getLanguage().resolve(comp);
     ModuleExp mexp = comp.getModule();
+    mexp.info = comp.minfo;
     Environment orig_env = Environment.getCurrent();
     Compilation orig_comp = Compilation.getCurrent();
     SourceMessages messages = comp.getMessages();
@@ -199,11 +212,8 @@ public class ModuleExp extends LambdaExp
         if (alwaysCompile)
           comp.mustCompile = true;
 
-        if (comp.mustCompile)
-          comp.addMainClass(mexp);
-
-        comp.walkModule(mexp);
-        comp.setState(Compilation.WALKED);
+        comp.process(Compilation.RESOLVED);
+        comp.minfo.loadByStages(Compilation.WALKED);
 
         if (msg != null ? messages.checkErrors(msg, 20) : messages.seenErrors())
           return null;
@@ -256,7 +266,7 @@ public class ModuleExp extends LambdaExp
 
   public final static void evalModule2 (Environment env, CallContext ctx,
                                         Language language, ModuleExp mexp,
-                                        Object inst, OutPort msg)
+                                        Object inst)
     throws Throwable
   {
     Environment orig_env = Environment.getCurrent();
@@ -274,30 +284,28 @@ public class ModuleExp extends LambdaExp
 	else
 	  {
             if (inst instanceof Class)
+              inst = ModuleContext.getContext().findInstance((Class) inst);
+
+            if (inst instanceof Runnable)
               {
-                Class clas = (Class) inst;
-                try
+                if (inst instanceof ModuleBody)
                   {
-                    inst = clas.getDeclaredField("$instance").get(null);
+                    ModuleBody mb = (ModuleBody) inst;
+                    if (! mb.runDone)
+                      {
+                        mb.runDone = true;
+                        mb.run(ctx);
+                      }
                   }
-                catch (NoSuchFieldException ex)
-                  {
-                    inst = clas.newInstance();
-                  }
-                /* #ifdef use:java.lang.Throwable.getCause */
-                catch (ExceptionInInitializerError ex)
-                  {
-                    throw ex.getCause();
-                  }
-                /* #endif */
+                else
+                  ((Runnable) inst).run();
               }
 
-            //	    try
-	      {
-                if (inst instanceof ModuleBody)
-                  ((ModuleBody) inst).run(ctx);
-
-		// Import declarations defined in module into the Environment.
+            if (mexp == null)
+              gnu.kawa.reflect.ClassMemberLocation.defineAll(inst, language, env);
+            else
+              {
+                // Import declarations defined in module into the Environment.
 		for (Declaration decl = mexp.firstDecl();
 		     decl != null;  decl = decl.nextDecl())
 		  {
@@ -313,9 +321,7 @@ public class ModuleExp extends LambdaExp
                     // the environment, and just require lexical lookup.
                     // However, various parts of the code makes use of
                     // the environment.
-		    if ((decl.field.getModifiers() & Access.FINAL) != 0
-                        && ! (dvalue instanceof ReferenceExp
-                              && ((ReferenceExp) dvalue).getBinding().needsContext()))
+		    if ((decl.field.getModifiers() & Access.FINAL) != 0)
 		      {
 			Object value;
 			if (dvalue instanceof QuoteExp
@@ -380,7 +386,7 @@ public class ModuleExp extends LambdaExp
     // In immediate mode there is no point in a non-static module:
     // a static module is simpler and more efficient.
     return (getFlag(STATIC_SPECIFIED)
-	    || ((gnu.expr.Compilation.moduleStatic > 0
+	    || ((gnu.expr.Compilation.moduleStatic >= Compilation.MODULE_STATIC_DEFAULT
                  || getFlag(IMMEDIATE))
 		&& ! getFlag(SUPERTYPE_SPECIFIED)
 		&& ! getFlag(NONSTATIC_SPECIFIED)));
@@ -391,7 +397,7 @@ public class ModuleExp extends LambdaExp
   {
     return (isStatic()
             && (getFlag(STATIC_RUN_SPECIFIED)
-                || Compilation.moduleStatic == 2));
+                || Compilation.moduleStatic == Compilation.MODULE_STATIC_RUN));
   }
 
   public void allocChildClasses (Compilation comp)
@@ -431,8 +437,7 @@ public class ModuleExp extends LambdaExp
             // Kludge - needed for macros - see Savannah bug #13601.
             && ! decl.isNamespaceDecl()
             && ! (decl.getFlag(Declaration.IS_CONSTANT)
-                  && decl.getFlag(Declaration.CAN_READ|Declaration.CAN_CALL))
-	    && ! (value instanceof ClassExp))
+                  && decl.getFlag(Declaration.CAN_READ|Declaration.CAN_CALL)))
 	  continue;
 	if (decl.getFlag(Declaration.IS_UNKNOWN))
 	  continue;
@@ -444,10 +449,9 @@ public class ModuleExp extends LambdaExp
 	  }
 	else
 	  {
-            if (! (decl.getFlag(Declaration.IS_CONSTANT) || decl.isAlias())
-		|| value == QuoteExp.undefined_exp)
-	      value = null;
-	    decl.makeField(comp, value);
+	    decl.makeField(comp,
+                           decl.shouldEarlyInit() || decl.isAlias() ? value
+                           : null);
 	  }
       }
   }
