@@ -5,6 +5,8 @@ package gnu.expr;
 import gnu.bytecode.*;
 import gnu.mapping.*;
 import gnu.kawa.lispexpr.LangObjType;
+import gnu.lists.ConsumerWriter;
+import java.io.Writer;
 
 /** A primitive Procedure implemented by a plain Java method. */
 
@@ -22,6 +24,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
    * 'V' means expect a target (this) argument, even if method is static;
    * '\0' means don't expect a target. */
   char mode;
+  boolean sideEffectFree;
 
   /** If non-null, the LambdaExp that this PrimProcedure implements. */
   LambdaExp source;
@@ -39,6 +42,16 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
 
   public Method getMethod () { return method; }
 
+  public boolean isSideEffectFree ()
+  {
+    return sideEffectFree;
+  }
+
+  public void setSideEffectFree ()
+  {
+    sideEffectFree = true;
+  }
+
   /** Return true iff the last parameter is a "rest" argument. */
   public boolean takesVarArgs()
   {
@@ -436,7 +449,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
 
   /** Compile arguments and push unto stack.
    * @param args arguments to evaluate and push.
-   * @param normally 0, but 1 in the case of a constructor,
+   * @param startArg Normally 0, but 1 in the case of a constructor,
    *   or the case of "static" method of a non-static class.
    * @param thisType If we are calling a non-static function,
    *   then args[0] is the receiver and thisType is its expected class.
@@ -545,7 +558,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
   public void compile (ApplyExp exp, Compilation comp, Target target)
   {
     gnu.bytecode.CodeAttr code = comp.getCode();
-    ClassType mclass = method == null ? null :  method.getDeclaringClass();
+    ClassType mclass = method == null ? null : method.getDeclaringClass();
     Expression[] args = exp.getArgs();
     if (isConstructor())
       {
@@ -575,6 +588,21 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
         thisType = null;
         startArg = 1;
       }
+    // Handle: (invoke-special ThisClass (this) '*init* arg ....)
+    // (This test is perhaps not quote as robust as it should be.)
+    else if (opcode() == 183 && mode == 'P' && "<init>".equals(method.getName()))
+      {
+        // Specifically handle passing a static-link.
+        ClassType mclass = method == null ? null :  method.getDeclaringClass();
+        if (mclass.hasOuterLink())
+          {
+            code.emitPushThis();
+            // Push the incoming static-link.
+            code.emitLoad(code.getCurrentScope().getVariable(1));
+            thisType = null;
+            startArg = 1;
+          }
+      }
     else if (takesTarget() && method.getStaticFlag())
       startArg = 1;
 
@@ -752,6 +780,92 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
 			decl, atypes, language);
   }
 
+  public static void disassemble$X (Procedure pproc, CallContext ctx)
+    throws Exception
+  {
+    gnu.lists.Consumer cons = ctx.consumer;
+    disassemble(pproc, cons instanceof Writer ? (Writer) cons: new ConsumerWriter(cons));
+  }
+
+  public static void disassemble (Procedure proc, Writer out)
+    throws Exception
+  {
+    disassemble(proc, new ClassTypeWriter(null, out, 0));
+  }
+
+  public static void disassemble (Procedure proc, ClassTypeWriter cwriter)
+    throws Exception
+  {
+    if (proc instanceof GenericProc)
+      {
+        GenericProc gproc = (GenericProc) proc;
+        int n = gproc.getMethodCount();
+        cwriter.print("Generic procedure with ");
+        cwriter.print(n);
+        cwriter.println(n == 1 ? " method." : "methods.");
+        for (int i = 0;  i < n;  i++)
+          {
+            Procedure mproc = gproc.getMethod(i);
+            if (mproc != null)
+              {
+                cwriter.println();
+                disassemble(mproc, cwriter);
+              }
+          }
+        return;
+      }
+    String pname = null;
+    Class cl = proc.getClass();
+    if (proc instanceof ModuleMethod)
+      cl = ((ModuleMethod) proc).module.getClass();
+    else if (proc instanceof PrimProcedure)
+      {
+        Method pmethod = ((PrimProcedure) proc).method;
+        if (pmethod != null)
+          {
+            cl = pmethod.getDeclaringClass().getReflectClass();
+            pname = pmethod.getName();
+          }
+      }
+    ClassLoader loader = cl.getClassLoader();
+    String cname = cl.getName();
+    String rname = cname.replace('.', '/') + ".class";
+    ClassType ctype = new ClassType();
+    java.io.InputStream rin = loader.getResourceAsStream(rname);
+    if (rin == null)
+      throw new RuntimeException("missing resource "+rname); // FIXME exception
+    ClassFileInput cinput = new ClassFileInput(ctype, rin);
+    cwriter.setClass(ctype);
+    java.net.URL resource = loader.getResource(rname);
+    cwriter.print("In class ");
+    cwriter.print(cname);
+    if (resource != null)
+      {
+        cwriter.print(" at ");
+        cwriter.print(resource);
+      }
+    cwriter.println();
+    if (pname == null)
+      {
+        pname = proc.getName();
+        if (pname == null)
+          {
+            cwriter.println("Anonymous function - unknown method.");
+            return;
+          }
+        pname = Compilation.mangleName(pname);
+      }
+    for (Method method = ctype.getMethods();
+         method != null; method = method.getNext())
+      {
+        String mname = method.getName();
+        if (mname.equals(pname)) {
+          cwriter.printMethod(method);
+        }
+      }
+    cwriter.flush();
+  }
+
   public static Class getProcedureClass (Object pproc)
   {
     Class procClass;
