@@ -16,6 +16,7 @@ public class QuoteExp extends Expression
   public final Object valueIfConstant() { return value; }
 
   protected Type type;
+  public final gnu.bytecode.Type getRawType() { return type; }
   public final gnu.bytecode.Type getType()
   {
     if (type != null)
@@ -109,8 +110,9 @@ public class QuoteExp extends Expression
     String msg = WrongArguments.checkArgCount(proc, nargs);
     if (msg != null)
       return walker.noteError(msg);
-    if (proc instanceof CanInline)
-      return ((CanInline) proc).inline(exp, walker, argsInlined);
+    Expression inlined = walker.maybeInline(exp, argsInlined, proc);
+    if (inlined != null)
+      return inlined;
     if (! argsInlined)
       exp.args = walker.walkExps(exp.args, exp.args.length);
     if (exp.getFlag(ApplyExp.INLINE_IF_CONSTANT))
@@ -122,7 +124,7 @@ public class QuoteExp extends Expression
     Compilation comp = walker.getCompilation();
     if (comp.inlineOk(proc))
       {
-	if (proc instanceof Inlineable)
+	if (ApplyExp.asInlineable(proc) != null)
           {
             if (exp.getFunction() == this)
               return exp;
