@@ -247,6 +247,15 @@ public abstract class ScopeExp extends Expression
     return n;
   }
 
+  public int countNonDynamicDecls ()
+  {
+    int n = 0;
+    for (Declaration decl = firstDecl(); decl != null; decl = decl.nextDecl())
+      if (! decl.getFlag(Declaration.IS_DYNAMIC))
+        n++;
+    return n;
+  }
+
   public static int nesting (ScopeExp sc)
   {
     int n = 0;
@@ -258,6 +267,16 @@ public abstract class ScopeExp extends Expression
     return n;
   }
 
+  /** True if given scope is nesed in this scope, perhaps indirectly. */
+  public boolean nestedIn (ScopeExp outer)
+  {
+    for (ScopeExp sc = this; sc != null; sc = sc.outer)
+      {
+        if (sc ==  outer)
+          return true;
+      }
+    return false;
+  }
   /** Size of evalFrame to allocate in interpreter. */
   protected int frameSize;
 
