@@ -158,24 +158,20 @@ public class SetExp extends AccessExp
 
     Declaration decl = binding;
     Expression declValue = decl.getValue();
-    if (decl.getFlag(Declaration.EARLY_INIT)
-	&& isDefining() && ! decl.ignorable())
-      {
-	BindingInitializer.create(decl, new_value, comp);
-      }
-    else if (declValue instanceof LambdaExp
+    if (declValue instanceof LambdaExp
 	&& decl.context instanceof ModuleExp
-	&& (! decl.isPrivate() || declValue instanceof ClassExp)
+        && ! decl.ignorable()
 	&& ((LambdaExp) declValue).getName() != null // FIXME
 	&& declValue == new_value)
       {
 	((LambdaExp) new_value).compileSetField(comp);
       }
-    else if (decl.context instanceof ModuleExp
-	     && (decl.getFlag(Declaration.IS_CONSTANT) || decl.isAlias())
-             && isDefining()
-             && declValue != null)
-      { // This is handled in ModuleExp's allocFields method.  But:
+    else if ((decl.shouldEarlyInit() || decl.isAlias())
+        && decl.context instanceof ModuleExp
+	&& isDefining() && ! decl.ignorable())
+      {
+        if (decl.shouldEarlyInit())
+          BindingInitializer.create(decl, new_value, comp);
         if (needValue)
           {
             decl.load(this, 0, comp, Target.pushObject);
