@@ -5,6 +5,7 @@
 (define-alias <frame> <gnu.jemacs.buffer.EFrame>)
 (define-alias <keymap> <gnu.jemacs.buffer.EKeymap>)
 (define-alias <toolkit> <gnu.jemacs.buffer.EToolkit>)
+(require <gnu.commonlisp.lisp.PrimOps>)
 
 (define (set-size (win :: <java.awt.Component>) (w :: <int>) (h :: <int>))
   (invoke win w h))
@@ -22,9 +23,8 @@
 
 ;;; MINIBUFFERS
 
-(define (read-dialog prompt)
-  (symbol->string
-   (invoke (as <frame> (window-frame)) 'ask prompt)))
+(define (read-dialog prompt) :: <string>
+  (invoke (as <frame> (window-frame)) 'ask prompt))
 
 (define read-from-minibuffer read-dialog)
 
@@ -91,7 +91,7 @@
 (define current-menubar #!null)
 
 (define (set-menubar-dirty-flag #!optional (frame :: <frame> (selected-frame)))
-  (invoke frame 'setMenu current-menubar))
+  (frame:setMenuBar current-menubar))
 
 
 (define (set-menubar menubar)
@@ -178,7 +178,7 @@
   (let ((name (invoke buffer 'getFileName)))
     (if (eq? name #!null)
         #f
-        (symbol->string name))))
+        name)))
 
 (define (set-visited-file-name filename #!optional (buffer (current-buffer)))
   ((primitive-virtual-method <buffer> "setFileName" <void>
@@ -441,7 +441,9 @@
 
 (define (insert-char ch #!optional (count '())
 		     (buffer :: <buffer> (current-buffer)))
-  (invoke buffer 'insert ch (if (eq? count '()) 1 count) #!null))
+  (invoke buffer 'insertChar
+	  (gnu.jemacs.lang.ELisp:asChar ch)
+	  (if (eq? count '()) 1 count) #!null))
 
 (define (insert #!rest (args :: <Object[]>))
   (let ((buffer :: <buffer> (current-buffer)))
@@ -575,10 +577,10 @@
     buffer))    
 
 (define (scheme-interaction-window)
-  (interaction-window 'scheme "Scheme interaction"))
+  (interaction-window "scheme" "Scheme interaction"))
 
 (define (elisp-interaction-window)
-  (interaction-window 'elisp "Lisp interaction"))
+  (interaction-window "elisp" "Lisp interaction"))
 
 (define (interaction-window (lang-name :: <java.lang.String>) buffer-name)
   (let ((buffer (get-buffer-create buffer-name)))
@@ -600,3 +602,22 @@
 (define (make-variable-buffer-local symbol)
   (invoke-static <gnu.jemacs.buffer.Buffer> 'makeBufferLocal symbol #t)
   symbol)
+(define (default-value symbol)
+  (let ((save <buffer>:current))
+    (<buffer>:setCurrent #!null)
+    (try-finally
+     (symbol-value symbol)
+     (<buffer>:setCurrent save))))
+(define (default-boundp symbol)
+  (let ((save <buffer>:current))
+    (<buffer>:setCurrent #!null)
+    (try-finally
+     (boundp symbol)
+     (<buffer>:setCurrent save))))
+(define (set-default symbol value)
+  (let ((save <buffer>:current))
+    (<buffer>:setCurrent #!null)
+    (try-finally
+     (set symbol value)
+     (<buffer>:setCurrent save))))
+  
\ No newline at end of file
