@@ -23,30 +23,39 @@ public class lambda extends Lambda
 
     Pair pair;
     int i = 0;
-    if (body instanceof Pair
-	&& (pair = (Pair) body).car instanceof FString)
+    if (body instanceof Pair)
       {
-	// Process documentation string.  FIXME.
-	body = pair.cdr;
+        pair = (Pair) body;
+	if (
+            /* #ifdef use:java.lang.CharSequence */
+            pair.getCar() instanceof CharSequence
+            /* #else */
+            // pair.getCar() instanceof String || pair.getCar() instanceof CharSeq
+            /* #endif */
+            )
+          {
+            // Process documentation string.  FIXME.
+            body = pair.getCdr();
+          }
       }
     Object interactive = null;
     if (body instanceof Pair
-	&& (pair = (Pair) body).car instanceof Pair)
+	&& (pair = (Pair) body).getCar() instanceof Pair)
       {
-	Pair first_application = (Pair) pair.car;
-	Object first_function = first_application.car;
+	Pair first_application = (Pair) pair.getCar();
+	Object first_function = first_application.getCar();
 	if (first_function instanceof Symbol
 	    && ((Symbol) first_function).getName() == "interactive")
 	  {
-	    interactive = first_application.cdr;
+	    interactive = first_application.getCdr();
 	    if (interactive != LList.Empty
 		&& ! (interactive instanceof Pair
-		      && ((Pair) interactive).cdr == LList.Empty))
+		      && ((Pair) interactive).getCdr() == LList.Empty))
 	      {
 		tr.syntaxError ("missing 'interactive' specification");
 		interactive = null;
 	      }
-	    body = pair.cdr;
+	    body = pair.getCdr();
 	  }
       }
     if (body instanceof PairWithPosition)
@@ -68,6 +77,7 @@ public class lambda extends Lambda
 	     arg = arg.nextDecl(), i++)
 	  {
 	    Declaration decl = let.addDeclaration(arg.getSymbol());
+            decl.setCanWrite(true);
 	    decl.setFluid(true);
 	    decl.setIndirectBinding(true);
 	    inits[i] = new ReferenceExp(arg);
@@ -88,8 +98,14 @@ public class lambda extends Lambda
           interactive = QuoteExp.nullExp;
         else
           {
-            Object arg = ((Pair) interactive).car;
-            if (arg instanceof FString)
+            Object arg = ((Pair) interactive).getCar();
+            if (
+                /* #ifdef use:java.lang.CharSequence */
+                arg instanceof CharSequence
+                /* #else */
+                // arg instanceof String || arg instanceof CharSeq
+                /* #endif */
+                )
               interactive = new QuoteExp(arg.toString());
             else
               {
