@@ -26,8 +26,10 @@ public class AppendValues extends MethodProc implements CanInline, Inlineable
       }
   }
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     if (args.length == 1)
       return args[0];
@@ -48,19 +50,6 @@ public class AppendValues extends MethodProc implements CanInline, Inlineable
 	for (int i = 0;  i < nargs;  i++)
 	  args[i].compileWithPosition(comp, target);
       }
-    else if (target instanceof SeriesTarget && nargs > 0)
-      {
-	CodeAttr code = comp.getCode();
-	SeriesTarget serTarget = (SeriesTarget) target;
-	Label done = serTarget.done;
-        serTarget.done = null;
-	for (int i = 0;  i < nargs;  i++)
-	  {
-	    if (i + 1 == nargs) // Last one
-              serTarget.done = done;
-	    args[i].compileWithPosition(comp, serTarget);
-	  }
-      }
     else
       {
 	ConsumerTarget.compileUsingConsumer(exp, comp, target);
