@@ -6,6 +6,7 @@ import gnu.math.*;
 import java.math.*;
 import gnu.bytecode.*;
 import gnu.kawa.lispexpr.LangPrimType;
+import gnu.kawa.lispexpr.LangObjType;
 
 public class Arithmetic
 {
@@ -104,12 +105,12 @@ public class Arithmetic
   static ClassType typeInteger = ClassType.make("java.lang.Integer");
   static ClassType typeLong = ClassType.make("java.lang.Long");
   */
-  static ClassType typeDFloNum = ClassType.make("gnu.math.DFloNum");
-  static ClassType typeRatNum = ClassType.make("gnu.math.RatNum");
-  static ClassType typeRealNum = ClassType.make("gnu.math.RealNum");
+  static LangObjType typeDFloNum = LangObjType.dflonumType;
+  static LangObjType typeRatNum = LangObjType.rationalType;
+  static LangObjType typeRealNum = LangObjType.realType;
   static ClassType typeNumber = ClassType.make("java.lang.Number");
   static ClassType typeNumeric = ClassType.make("gnu.math.Numeric");
-  static ClassType typeIntNum = ClassType.make("gnu.math.IntNum");
+  static LangObjType typeIntNum = LangObjType.integerType;
 
   public static int classifyType (Type type)
   {
@@ -219,8 +220,6 @@ public class Arithmetic
     return new BigDecimal(value.toString());
   }
 
-  public static final IntNum ten_exp_9 = IntNum.make(1000000000);
-
   public static RatNum asRatNum (Object value)
   {
     if (value instanceof RatNum)
@@ -228,51 +227,15 @@ public class Arithmetic
     if (value instanceof BigInteger)
       return IntNum.valueOf(value.toString(), 10);
     if (value instanceof BigDecimal)
-      {
-	BigDecimal d = (BigDecimal) value;
-	RatNum v = IntNum.valueOf(d.unscaledValue().toString(), 10);
-	int scale = d.scale();
-	for (; scale >= 9; scale -= 9)
-	  v = RatNum.divide(v, ten_exp_9);
-	for (; scale <= -9; scale += 9)
-	  v = RatNum.times(v, ten_exp_9);
-	IntNum scaleVal;
-	switch (scale > 0 ? scale : -scale)
-	  {
-	  case 1: scaleVal = IntNum.make(10);  break;
-	  case 2: scaleVal = IntNum.make(100);  break;
-	  case 3: scaleVal = IntNum.make(1000);  break;
-	  case 4: scaleVal = IntNum.make(10000);  break;
-	  case 5: scaleVal = IntNum.make(100000);  break;
-	  case 6: scaleVal = IntNum.make(1000000);  break;
-	  case 7: scaleVal = IntNum.make(10000000);  break;
-	  case 8: scaleVal = IntNum.make(100000000);  break;
-	  default:
-	    return v;
-	  }
-	if (scale > 0)
-	  return RatNum.divide(v, scaleVal);
-	else
-	  return RatNum.times(v, scaleVal);
-      }
+      return RatNum.valueOf((BigDecimal) value);
     else
       return IntNum.make(((Number) value).longValue());
   }
 
   public static Numeric asNumeric (Object value)
   {
-    if (! (value instanceof Numeric))
-      {
-	if (value instanceof BigInteger || value instanceof Long
-	    || value instanceof Short || value instanceof Byte
-	    || value instanceof Integer)
-	  return asIntNum(value);
-	if (value instanceof BigDecimal)
-	  return asRatNum(value);
-	if (value instanceof Float || value instanceof Double)
-	  return new DFloNum(asDouble(value));
-      }
-    return (Numeric) value;
+    Numeric n = Numeric.asNumericOrNull(value);
+    return n != null ? n : (Numeric) value;
   }
 
   /** Convert a number to a String.
@@ -374,4 +337,34 @@ public class Arithmetic
         return (Number) value;
       }
   }
+
+  public static boolean isExact (Number num)
+  {
+    if (num instanceof Numeric)
+      return ((Numeric) num).isExact();
+    return !(num instanceof Double || num instanceof Float
+             || num instanceof BigDecimal);
+  }
+
+  public static Number toExact (Number num)
+  {
+    if (num instanceof Numeric)
+      return ((Numeric) num).toExact();
+    if (num instanceof Double || num instanceof Float
+        || num instanceof BigDecimal)
+      return DFloNum.toExact(num.doubleValue());
+    else
+      return num;
+  }
+
+  public static Number toInexact (Number num)
+  {
+    if (num instanceof Numeric)
+      return ((Numeric) num).toInexact();
+    if (num instanceof Double || num instanceof Float
+        || num instanceof BigDecimal)
+      return num;
+    else
+      return Double.valueOf(num.doubleValue());
+  }
 }
