@@ -23,6 +23,17 @@ public abstract class LispLanguage extends Language
   static public final String quasiquote_sym = "quasiquote";
   /** Used for Kawa infix ':' operator. */
   static public final Symbol lookup_sym = Namespace.EmptyNamespace.getSymbol("$lookup$");
+  // FUTURE: Used for: [ e1 e2 ... ]
+  // for future sequence/list constructors.
+  static public final Symbol bracket_list_sym = Namespace.EmptyNamespace.getSymbol("$bracket-list$");
+  // FUTURE: Used for: name[ e1 e2 ... ]
+  // Needed for array types - e.g. Object[]
+  // and (possible future) parameterized types - e.g. java.util.List[integer]
+  static public final Symbol bracket_apply_sym = Namespace.EmptyNamespace.getSymbol("$bracket-apply$");
+
+  public static StaticFieldLocation getNamedPartLocation =
+    new StaticFieldLocation("gnu.kawa.functions.GetNamedPart", "getNamedPart");
+  static { getNamedPartLocation.setProcedure(); }
 
   /** The default <code>ReadTable</code> for this language. */
   public ReadTable defaultReadTable = createReadTable();
@@ -68,7 +79,20 @@ public abstract class LispLanguage extends Language
               }
             tr.scanForm(sexp, mexp);
             if ((options & PARSE_ONE_LINE) != 0)
-              break;
+              {
+                if (tr.getMessages().seenErrors())
+                  {
+                    // Skip to end of line.
+                    for (;;)
+                      {
+                        int ch = reader.peek();
+                        if (ch < 0 || ch == '\r' || ch == '\n')
+                          break;
+                        reader.skip();
+                      }
+                  }
+                break;
+              }
             if ((options & PARSE_PROLOG) != 0
                 && tr.getState() >= Compilation.PROLOG_PARSED)
               {
