@@ -4,6 +4,7 @@ import gnu.mapping.*;
 import gnu.lists.*;
 import gnu.math.*;
 import gnu.expr.*;
+import gnu.kawa.util.GeneralHashTable;
 
 /** A Lexer for reading S-expressions in generic Lisp-like syntax.
  * This class may have outlived its usefulness: It's mostly just a
@@ -24,6 +25,8 @@ public class LispReader extends Lexer
     super(port, messages);
   }
 
+  GeneralHashTable<Integer,Object> sharedStructureTable;
+
   /** Read a #|...|#-style comment (which may contain other nested comments).
     * Assumes the initial "#|" has already been read.
     */
@@ -60,7 +63,7 @@ public class LispReader extends Lexer
 
   /** Get specification of how symbols should be case-folded.
     * @return Either 'P' (means preserve case), 'U' (upcase),
-    * 'D' (downcase, or 'I' (invert case).
+    * 'D' (downcase), or 'I' (invert case).
     */
   static char getReadCase()
   {
@@ -99,43 +102,158 @@ public class LispReader extends Lexer
     // Step 1:
     int startPos = tokenBufferLength;
 
-    if (entry == null)
+    seenEscapes = false;
+    int kind = entry.getKind();
+    switch (kind)
       {
+      case ReadTable.ILLEGAL:
 	// Step 2:
 	String err = ("invalid character #\\"+((char) ch));  // FIXME
 	if (interactive) fatal(err);
 	else error(err);
 	return Values.empty;
-      }
-    int kind = entry.getKind();
-    seenEscapes = false;
-    switch (kind)
-      {
       case ReadTable.WHITESPACE:
 	// Step 3:
 	return Values.empty;
       case ReadTable.TERMINATING_MACRO:
       case ReadTable.NON_TERMINATING_MACRO:
-	Object value = entry.read(this, ch, -1);
-	return value;
+	return entry.read(this, ch, -1);
       case ReadTable.CONSTITUENT:
-        if (ch == rtable.postfixLookupOperator)
-          { // Force an initial ':' to be treated as a CONSTITUENT.
-            tokenBufferAppend(ch);
-            ch = read();
-          }
       case ReadTable.SINGLE_ESCAPE: // Step 5:
       case ReadTable.MULTIPLE_ESCAPE: // Step 6:
       default:  // 
 	break;
       }
+    return readAndHandleToken(ch, startPos, rtable);
+  }
 
+  protected Object readAndHandleToken(int ch, int startPos, ReadTable rtable)
+    throws java.io.IOException, SyntaxException
+  {
     readToken(ch, getReadCase(), rtable);
     int endPos = tokenBufferLength;
-    if (seenEscapes)
-      return returnSymbol(startPos, endPos, rtable);
-    else
-      return handleToken(startPos, endPos, rtable);
+    if (! seenEscapes)
+      {
+        Object value = parseNumber(tokenBuffer, startPos, endPos - startPos,
+                                   '\0', 0, SCM_NUMBERS);
+        if (value != null && ! (value instanceof String))
+          return value;
+        /* Common Lisp only?  FIXME
+        if (isPotentialNumber(tokenBuffer, startPos, endPos))
+          {
+            error(value == null ? "not a valid number"
+                  : "not a valid number: " + value);
+            return IntNum.zero();
+          }
+        */
+      }
+
+    char readCase = getReadCase();
+    if (readCase == 'I')
+      {
+	int upperCount = 0;
+	int lowerCount = 0;
+	for (int i = startPos;  i < endPos;  i++)
+	  {
+	    char ci = tokenBuffer[i];
+	    if (ci == TOKEN_ESCAPE_CHAR)
+	      i++;
+	    else if (Character.isLowerCase(ci))
+	      lowerCount++;
+	    else if (Character.isUpperCase(ci))
+	      upperCount++;
+	  }
+	if (lowerCount == 0)
+	  readCase = 'D';
+	else if (upperCount == 0)
+	  readCase = 'U';
+	else
+	  readCase = 'P';
+      }
+
+    boolean handleUri =
+      (endPos >= startPos + 2
+       && tokenBuffer[endPos-1] == '}'
+       && tokenBuffer[endPos-2] != TOKEN_ESCAPE_CHAR
+       && peek() == ':');
+    int packageMarker = -1;
+    int lbrace = -1, rbrace = -1, braceNesting = 0;
+    int j = startPos;
+    boolean uriBad = false;
+    for (int i = startPos;  i < endPos;  i++)
+      {
+	char ci = tokenBuffer[i];
+	if (ci == TOKEN_ESCAPE_CHAR)
+	  {
+	    if (++ i < endPos)
+	      tokenBuffer[j++] = tokenBuffer[i];
+	    continue;
+	  }
+        if (handleUri)
+          {
+            if (ci == '{')
+              {
+                if (lbrace < 0)
+                  lbrace = j;
+                else if (braceNesting == 0)
+                  uriBad = true;
+                braceNesting++;
+              }
+            else if (ci == '}')
+              {
+                braceNesting--;
+                if (braceNesting < 0)
+                   uriBad = true;
+                else if (braceNesting == 0)
+                  {
+                    if (rbrace < 0)
+                      rbrace = j;
+                    else
+                      uriBad = true;
+                  }
+              }
+          }
+        if (braceNesting > 0)
+          ;
+	else if (ci == ':')
+	  packageMarker = packageMarker >= 0 ? -1 : j;
+	else if (readCase == 'U')
+	  ci = Character.toUpperCase(ci);
+	else if (readCase == 'D')
+	  ci = Character.toLowerCase(ci);
+	tokenBuffer[j++] = ci;
+      }
+    endPos = j;
+
+    int len = endPos - startPos;
+
+    if (lbrace >= 0 && rbrace > lbrace)
+      {
+        String prefix = lbrace > 0 ? new String(tokenBuffer, startPos, lbrace-startPos) : null;
+        lbrace++;
+        String uri = new String(tokenBuffer, lbrace, rbrace-lbrace);
+        ch = read(); // skip ':' - previously peeked.
+        ch = read();
+        Object rightOperand = readValues(ch, rtable.lookup(ch), rtable);
+        if (! (rightOperand instanceof SimpleSymbol))
+          error("expected identifier in symbol after '{URI}:'");
+        // FIXME should allow "compound keyword" - for attribute names
+        return Symbol.valueOf(rightOperand.toString(), uri, prefix);
+      }
+
+    if (rtable.initialColonIsKeyword && packageMarker == startPos && len > 1)
+      {
+	startPos++;
+	String str = new String(tokenBuffer, startPos, endPos-startPos);
+	return Keyword.make(str.intern());
+    }
+    if (rtable.finalColonIsKeyword && packageMarker == endPos - 1
+        && (len > 1 || seenEscapes))
+      {
+	String str = new String(tokenBuffer, startPos, len - 1);
+	return Keyword.make(str.intern());
+      }
+    return rtable.makeSymbol(new String(tokenBuffer, startPos, len));
   }
 
   public static final char TOKEN_ESCAPE_CHAR = '\uffff';
@@ -149,6 +267,7 @@ public class LispReader extends Lexer
       throws java.io.IOException, SyntaxException
   {
     boolean inEscapes = false;
+    int braceNesting = 0;
     for (;; ch = read())
       {
 	if (ch < 0)
@@ -159,7 +278,8 @@ public class LispReader extends Lexer
 	      break;
 	  }
 	ReadTableEntry entry = rtable.lookup(ch);
-	if (entry == null)
+	int kind = entry.getKind();
+	if (kind == ReadTable.ILLEGAL)
 	  {
 	    if (inEscapes)
 	      {
@@ -167,19 +287,34 @@ public class LispReader extends Lexer
 		tokenBufferAppend(ch);
 		continue;
 	      }
+            if (ch == '}' && --braceNesting >= 0)
+              {
+                tokenBufferAppend(ch);
+		continue;
+              }
 	    unread(ch);
 	    break;
 	  }
-	int kind = entry.getKind();
-        if (ch == rtable.postfixLookupOperator && ! inEscapes
-            && validPostfixLookupStart(rtable))
-          kind = ReadTable.TERMINATING_MACRO;
+        if (ch == rtable.postfixLookupOperator && ! inEscapes)
+          {
+            int next = port.peek();
+            if (next == rtable.postfixLookupOperator)
+              { // Looking at '::'
+                unread(ch);
+                break;
+              }
+            if (validPostfixLookupStart(next, rtable))
+              kind = ReadTable.TERMINATING_MACRO;
+          }
                   
 	if (kind == ReadTable.SINGLE_ESCAPE)
 	  {
 	    ch = read();
 	    if (ch < 0)
 	      eofError("unexpected EOF after single escape");
+            if (rtable.hexEscapeAfterBackslash
+                && (ch == 'x' || ch == 'X'))
+              ch = readHexEscape();
 	    tokenBufferAppend(TOKEN_ESCAPE_CHAR);
 	    tokenBufferAppend(ch);
 	    seenEscapes = true;
@@ -188,6 +323,7 @@ public class LispReader extends Lexer
 	if (kind == ReadTable.MULTIPLE_ESCAPE)
 	  {
 	    inEscapes = ! inEscapes;
+	    seenEscapes = true;
 	    continue;
 	  }
 	if (inEscapes)
@@ -202,15 +338,10 @@ public class LispReader extends Lexer
 	    switch (kind)
 	      {
 	      case ReadTable.CONSTITUENT:
-		// ... fall through ...
+                if (ch == '{' && entry == ReadTableEntry.brace)
+                  braceNesting++;
+                /* ... fall thotugh ... */
 	      case ReadTable.NON_TERMINATING_MACRO:
-		if (readCase == 'U'
-		    || (readCase == 'I' && Character.isLowerCase((char) ch)))
-		  ch = Character.toUpperCase((char) ch);
-		else if (readCase == 'D'
-			 || (readCase == 'I'
-			     && Character.isUpperCase((char) ch)))
-		  ch = Character.toLowerCase ((char) ch);
 		tokenBufferAppend(ch);
 		continue;
 	      case ReadTable.MULTIPLE_ESCAPE:
@@ -258,17 +389,14 @@ public class LispReader extends Lexer
       }
   }
 
-  protected boolean validPostfixLookupStart (ReadTable rtable)
+  protected boolean validPostfixLookupStart (int ch, ReadTable rtable)
       throws java.io.IOException
   {
-    int ch = port.peek();
-    ReadTableEntry entry;
-    if (ch < 0 || ch == ':' || (entry = rtable.lookup(ch)) == null
-        || ch == rtable.postfixLookupOperator)
+    if (ch < 0 || ch == ':' || ch == rtable.postfixLookupOperator)
       return false;
     if (ch == ',')
       return true;
-    int kind = entry.getKind();
+    int kind = rtable.lookup(ch).getKind();
     return kind == ReadTable.CONSTITUENT
       || kind == ReadTable.NON_TERMINATING_MACRO
       || kind == ReadTable.MULTIPLE_ESCAPE
@@ -287,7 +415,8 @@ public class LispReader extends Lexer
           break;
         // A kludge to map PreOpWord to ($lookup$ Pre 'Word).
         port.read();
-        if (! validPostfixLookupStart(rtable))
+        int ch2 = port.peek();
+        if (! validPostfixLookupStart(ch2, rtable))
           {
             unread();
             break;
@@ -459,7 +588,8 @@ public class LispReader extends Lexer
 
     boolean negative = ch == '-';
     boolean numeratorNegative = negative;
-    if (ch == '-' || ch == '+')
+    boolean sign_seen = ch == '-' || ch == '+';
+    if (sign_seen)
       {
 	if (pos >= end)
 	  return "no digits following sign";
@@ -582,8 +712,30 @@ public class LispReader extends Lexer
 	ch = buffer[pos++];
       }
 
+    char infnan = '\0';
+
     if (digits_start < 0)
-      return "no digits";
+      {
+        if (sign_seen
+            && pos + 4 < end && buffer[pos+3] == '.' && buffer[pos+4] == '0')
+          {
+            if (buffer[pos] == 'i'
+                && buffer[pos+1] == 'n'
+                && buffer[pos+2] == 'f')
+              {
+                infnan = 'i';
+              }
+            else if (buffer[pos] == 'n'
+                && buffer[pos+1] == 'a'
+                && buffer[pos+2] == 'n')
+              {
+                infnan = 'n';
+              }
+          }
+        if (infnan == '\0')
+          return "no digits";
+        pos += 5;
+      }
 
     if (hash_seen || underscore_seen)
       {
@@ -593,7 +745,14 @@ public class LispReader extends Lexer
     boolean inexact = (exactness == 'i' || exactness == 'I'
 		       || (exactness == ' ' && hash_seen));
     RealNum number = null;
-    if (exp_seen >= 0 || decimal_point >= 0)
+    char exp_char = '\0';
+    if (infnan != '\0')
+      {
+        inexact = true;
+	double d = infnan == 'i' ? Double.POSITIVE_INFINITY : Double.NaN;
+	number = new DFloNum(negative ? - d : d);
+      }
+    else if (exp_seen >= 0 || decimal_point >= 0)
       {
 	if (digits_start > decimal_point && decimal_point >= 0)
 	  digits_start = decimal_point;
@@ -602,8 +761,8 @@ public class LispReader extends Lexer
 	String str = new String(buffer, digits_start, pos - digits_start);
         if (exp_seen >= 0)
           {
-            char exp = buffer[exp_seen];
-            if (exp != 'e' && exp != 'E')
+            exp_char = Character.toLowerCase(buffer[exp_seen]);
+            if (exp_char != 'e')
               {
                 int prefix = exp_seen - digits_start;
                 str = str.substring(0, prefix)+'e'+str.substring(prefix+1);
@@ -712,6 +871,19 @@ public class LispReader extends Lexer
         return "excess junk after number";
 	
       }
+    else if (number instanceof DFloNum && exp_char > 0 && exp_char != 'e')
+      {
+        double d = number.doubleValue();
+        switch (exp_char)
+          {
+          case 'f':  case 's':
+            return Float.valueOf((float) d);
+          case 'd':
+            return Double.valueOf(d);
+          case 'l':
+            return java.math.BigDecimal.valueOf(d);
+          }
+      }
     return number;
   }
 
@@ -730,86 +902,6 @@ public class LispReader extends Lexer
 			    radix, negative);
   }
 
-  protected Object returnSymbol(int startPos, int endPos, ReadTable rtable)
-  {
-    char readCase = getReadCase();
-    if (readCase == 'I')
-      {
-	int upperCount = 0;
-	int lowerCount = 0;
-	for (int i = startPos;  i < endPos;  i++)
-	  {
-	    char ch = tokenBuffer[i];
-	    if (ch == TOKEN_ESCAPE_CHAR)
-	      i++;
-	    else if (Character.isLowerCase(ch))
-	      lowerCount++;
-	    else if (Character.isUpperCase(ch))
-	      upperCount++;
-	  }
-	if (lowerCount == 0)
-	  readCase = 'D';
-	else if (upperCount == 0)
-	  readCase = 'U';
-	else
-	  readCase = 'P';
-      }
-
-    int packageMarker = -1;
-    int j = startPos;
-    for (int i = startPos;  i < endPos;  i++)
-      {
-	char ch = tokenBuffer[i];
-	if (ch == TOKEN_ESCAPE_CHAR)
-	  {
-	    if (++ i < endPos)
-	      tokenBuffer[j++] = tokenBuffer[i];
-	    continue;
-	  }
-	if (ch == ':')
-	  packageMarker = packageMarker >= 0 ? -1 : j;
-	else if (readCase == 'U')
-	  ch = Character.toUpperCase(ch);
-	else if (readCase == 'D')
-	  ch = Character.toLowerCase(ch);
-	tokenBuffer[j++] = ch;
-      }
-    endPos = j;
-
-    int len = endPos - startPos;
-
-    if (rtable.initialColonIsKeyword && packageMarker == startPos && len > 1)
-      {
-	startPos++;
-	String str = new String(tokenBuffer, startPos, endPos-startPos);
-	return Keyword.make(str.intern());
-    }
-    if (rtable.finalColonIsKeyword && packageMarker == endPos - 1 && len > 1)
-      {
-	String str = new String(tokenBuffer, startPos, len - 1);
-	return Keyword.make(str.intern());
-      }
-    return rtable.makeSymbol(new String(tokenBuffer, startPos, len));
-  }
-
-  /** Classify and return a token in tokenBuffer from startPos to endPos. */
-  public Object handleToken(int startPos, int endPos,  ReadTable rtable)
-  {
-    Object value = parseNumber(tokenBuffer, startPos, endPos - startPos,
-			       '\0', 0, SCM_NUMBERS);
-    if (value != null && ! (value instanceof String))
-      return value;
-    /* Common Lisp only?  FIXME
-    if (isPotentialNumber(tokenBuffer, startPos, endPos))
-      {
-	error(value == null ? "not a valid number"
-	      : "not a valid number: " + value);
-	return IntNum.zero();
-      }
-    */
-    return returnSymbol(startPos, endPos, rtable);
-  }
-
   /** Reads a C-style String escape sequence.
    * Assume '\\' has already been read.
    * Return the converted character, or -1 on EOF, or -2 to ignore. */
@@ -948,31 +1040,38 @@ public class LispReader extends Lexer
 	  }
 	break;
       case 'x':
-	c = 0;
-	/* A hex escape, as in ANSI C.  */
-	for (;;)
-	  {
-	    int d = read();
-	    int v = Character.digit((char) d, 16);
-	    if (v >= 0)
-	      c = (c << 4) + v;
-	    else
-	      {
-                if (d != ';')
-                  {
-                    // FIXME: if strict-R6RS: ERROR
-                    if (d >= 0)
-                      unread(d);
-                  }
-		break;
-	      }
-	  }
-	break;
+      case 'X':
+	return readHexEscape();
       default:  break;
       }
     return c;
   }
 
+  public int readHexEscape ()
+    throws java.io.IOException, SyntaxException 
+  {
+    int c = 0;
+    /* A hex escape, as in ANSI C.  */
+    for (;;)
+      {
+        int d = read();
+        int v = Character.digit((char) d, 16);
+        if (v >= 0)
+          c = (c << 4) + v;
+        else
+          {
+            if (d != ';')
+              {
+                // FIXME: if strict-R6RS: ERROR
+                if (d >= 0)
+                  unread(d);
+              }
+            break;
+          }
+      }
+    return c;
+  }
+
   public final Object readObject (int c)
       throws java.io.IOException, SyntaxException
   {
@@ -993,7 +1092,7 @@ public class LispReader extends Lexer
     return LList.Empty;
   }
 
-  protected Object makePair (Object car, int line, int column)
+  protected Pair makePair (Object car, int line, int column)
   {
     String pname = port.getName();
     if (pname != null && line >= 0)
@@ -1005,7 +1104,7 @@ public class LispReader extends Lexer
 
   protected void setCdr (Object pair, Object cdr)
   {
-    ((PairWithPosition) pair).setCdrBackdoor(cdr);
+    ((Pair) pair).setCdrBackdoor(cdr);
   }
 
   /** Read a number from a LispReader
