@@ -3,11 +3,10 @@
 
 package gnu.kawa.lispexpr;
 import gnu.text.*;
-import gnu.mapping.InPort;
-import gnu.mapping.Values;
-import gnu.mapping.Procedure;
+import gnu.mapping.*;
 import gnu.bytecode.Type;
 import gnu.lists.*;
+import gnu.kawa.util.GeneralHashTable;
 /* #ifdef use:java.util.regex */
 import java.util.regex.*;
 /* #endif */
@@ -89,10 +88,10 @@ public class ReaderDispatchMisc extends ReadTableEntry
 	reader.tokenBufferAppend('#');
 	reader.tokenBufferAppend(ch);
 	return LispReader.readNumberWithRadix(2, reader, 0);
-        /* #ifdef use:java.util.regex */
+      /* #ifdef use:java.util.regex */
       case '/':
-        return readRegex(in, ch, count);
-        /* #endif */
+      return readRegex(in, ch, count);
+      /* #endif */
       case '|':
 	port = reader.getPort();
 	if (port instanceof InPort)
@@ -134,9 +133,9 @@ public class ReaderDispatchMisc extends ReadTableEntry
             && ((length
                  = LList.listLength(list = reader.readObject(), false))
                 > 0)
-            && ((Pair) list).getCar() instanceof String)
+            && ((Pair) list).getCar() instanceof Symbol)
           {
-            name = (String) ((Pair) list).getCar();
+            name = ((Pair) list).getCar().toString();
             Object proc = ReadTable.getCurrent().getReaderCtor(name);
             if (proc == null)
               in.error("unknown reader constructor "+name);
@@ -172,12 +171,42 @@ public class ReaderDispatchMisc extends ReadTableEntry
         else
           in.error("a non-empty list starting with a symbol must follow #,");
 	return Boolean.FALSE;
+      case '=':
+        Object object = reader.readObject();
+        if (in instanceof LispReader)
+          {
+            LispReader lin = (LispReader) in;
+            GeneralHashTable<Integer,Object> map = lin.sharedStructureTable;
+            if (map == null)
+              {
+                map = new GeneralHashTable<Integer,Object>();
+                lin.sharedStructureTable = map;
+              }
+            map.put(Integer.valueOf(count), object);
+          }
+        return object;
+      case '#':
+        if (in instanceof LispReader)
+          {
+            LispReader lin = (LispReader) in;
+            GeneralHashTable<Integer,Object> map = lin.sharedStructureTable;
+            if (map != null)
+              {
+                Integer key = Integer.valueOf(count);
+                object = map.get(key, in);
+                if (object != in)
+                  return object;
+              }
+          }
+        in.error("an unrecognized #n# back-reference was read");
+	return Values.empty;
       default:
 	in.error("An invalid #-construct was read.");
 	return Values.empty;
       }
   }
 
+  /* #ifdef use:java.util.regex */
   public static Pattern readRegex (Lexer in, int ch, int count)
     throws java.io.IOException, SyntaxException
   {
@@ -187,16 +216,16 @@ public class ReaderDispatchMisc extends ReadTableEntry
     int flags = 0;
     if (port instanceof InPort)
       {
-	saveReadState = ((InPort) port).readState;
-	((InPort) port).readState = '/';
+        saveReadState = ((InPort) port).readState;
+        ((InPort) port).readState = '/';
       }
     try
       {
-	for (;;)
-	  {
-	    int next;
+        for (;;)
+          {
+            int next;
 
-	    int c = port.read();
+            int c = port.read();
             if (c < 0)
               in.eofError("unexpected EOF in regex literal");
 	    if (c == ch)
@@ -229,12 +258,13 @@ public class ReaderDispatchMisc extends ReadTableEntry
               flags |= Pattern.DOTALL;
             else if (c == 'm' || c == 'M')
               flags |= Pattern.MULTILINE;
-            // Think this through more before adidng this feature:
-            // Perhaps we should use the 'x' handling from
-            // gnu.xquery.util.StringUtils.makePattern (which is
-            // smart enogh to handle space in character classes).
-            // Perhaps we should handle Scheme comments?
-            /*else if (c == 'x' || c == 'X')
+            /* Think this through more before adding this feature:
+            Perhaps we should use the 'x' handling from
+            gnu.xquery.util.StringUtils.makePattern (which is
+            smart enogh to handle space in character classes).
+            Perhaps we should handle Scheme comments?
+
+            else if (c == 'x' || c == 'X')
               flags |= Pattern.COMMENTS;
             */
             else if (Character.isLetter(c))
@@ -249,9 +279,10 @@ public class ReaderDispatchMisc extends ReadTableEntry
       }
     finally
       {
-	in.tokenBufferLength = startPos;
-	if (port instanceof InPort)
-	  ((InPort) port).readState = saveReadState;
+        in.tokenBufferLength = startPos;
+        if (port instanceof InPort)
+          ((InPort) port).readState = saveReadState;
       }
   }
+  /* #endif */
 }
