@@ -4,6 +4,7 @@
 package gnu.kawa.lispexpr;
 import gnu.text.Lexer;
 import gnu.text.SyntaxException;
+import gnu.lists.PairWithPosition;
 
 public class ReaderQuote extends ReadTableEntry
 {
@@ -31,6 +32,9 @@ public class ReaderQuote extends ReadTableEntry
     throws java.io.IOException, SyntaxException
   {
     LispReader reader = (LispReader) in;
+    String file = reader.getName();
+    int line1 = reader.getLineNumber() + 1;
+    int column1 = reader.getColumnNumber() + 1;
     Object magic = magicSymbol;
     if (next != '\0')
       {
@@ -40,7 +44,13 @@ public class ReaderQuote extends ReadTableEntry
 	else if (ch >= 0)
 	  reader.unread(ch);
       }
+    int line2 = reader.getLineNumber() + 1;
+    int column2 = reader.getColumnNumber() + 1;
     Object operand = reader.readObject();
-    return reader.makePair(magic, reader.makePair(operand, reader.makeNil()));
+    return PairWithPosition.make(magic,
+                                 PairWithPosition.make(operand,
+                                                       reader.makeNil(),
+                                                       file, line2, column2),
+                                 file, line1, column1);
   }
 }
