@@ -13,7 +13,13 @@ public class ReaderString extends ReadTableEntry
     return readString(in, ch, count);
   }
   
-  public static String readString (Lexer in, int ch, int count)
+  public static
+  /* #ifdef use:java.lang.CharSequence */
+  String
+  /* #else */
+  // gnu.lists.FString
+  /* #endif */
+  readString (Lexer in, int ch, int count)
     throws java.io.IOException, SyntaxException
   {
     int startPos = in.tokenBufferLength;
@@ -30,8 +36,6 @@ public class ReaderString extends ReadTableEntry
       {
 	for (;;)
 	  {
-	    int next;
-
 	    prev = c;
 
 	    // Read next char - inline the common case.
@@ -52,7 +56,16 @@ public class ReaderString extends ReadTableEntry
 	    switch (c)
 	      {
 	      case '\r':
-		in.tokenBufferAppend('\n');
+                int t;
+                if (port.getConvertCR())
+                  t = '\n';
+                else
+                  {
+                    t = '\r';
+                    // To suppress possible "\r\n"-conversion.
+                    c = ' ';
+                  }
+		in.tokenBufferAppend(t);
 		continue;
 	      case '\\':
 		if (in instanceof LispReader)
