@@ -1,18 +1,18 @@
 package gnu.kawa.reflect;
 import gnu.bytecode.*;
 import gnu.mapping.*;
-import gnu.expr.*;
 import java.io.*;
 
 public class ArrayLength 
   extends Procedure1
-  implements Inlineable, Externalizable
+  implements Externalizable
 {
   Type element_type;
 
   public ArrayLength (Type element_type)
   {
     this.element_type = element_type;
+    Procedure.compilerKey.set(this, "*gnu.kawa.reflect.CompileArrays:getForArrayLength");
   }
 
   public Object apply1 (Object array)
@@ -20,17 +20,9 @@ public class ArrayLength
     return gnu.math.IntNum.make(java.lang.reflect.Array.getLength(array));
   }
 
-  public void compile (ApplyExp exp, Compilation comp, Target target)
+  public boolean isSideEffectFree ()
   {
-    exp.getArgs()[0].compile(comp, ArrayType.make(element_type));
-    CodeAttr code = comp.getCode();
-    code.emitArrayLength();
-    target.compileFromStack(comp, gnu.kawa.lispexpr.LangPrimType.intType);
-  }
-
-  public gnu.bytecode.Type getReturnType (Expression[] args)
-  {
-    return gnu.kawa.lispexpr.LangPrimType.intType;
+    return true;
   }
 
   public void writeExternal(ObjectOutput out) throws IOException
