@@ -98,7 +98,11 @@ public class SlotGet extends Procedure2
     if (fname == "length" && clas.isArray())
       {
 	int length = java.lang.reflect.Array.getLength(obj);
-	return language.coerceToObject(length);
+        /* #ifdef JAVA5 */
+        return Integer.valueOf(length);
+        /* #else */
+        // return new Integer(length);
+        /* #endif */
       }
     if (fname == "class")
       return clas;
@@ -414,10 +418,8 @@ public class SlotGet extends Procedure2
                             : Target.pushValue(ctype));
             if (isStaticMethod)
               code.emitInvokeStatic(method);
-            else if (ctype.isInterface())
-              code.emitInvokeInterface(method);
             else
-              code.emitInvokeVirtual(method);
+              code.emitInvoke(method);
 	    target.compileFromStack(comp, method.getReturnType());
             return;
           }
