@@ -1,9 +1,11 @@
 package gnu.kawa.util;
+import java.util.*;
 
 public class WeakHashNode<K,V>
-/* #ifdef JAVA2 */
-extends java.lang.ref.WeakReference<K>
-/* #endif */
+  /* #ifdef JAVA2 */
+  extends java.lang.ref.WeakReference<K>
+  implements Map.Entry<K,V>
+  /* #endif */
 {
   public WeakHashNode<K,V> next;
   public int hash;
@@ -26,4 +28,21 @@ extends java.lang.ref.WeakReference<K>
   // K key;
   // public K get() { return key; }
   /* #endif */
+
+  public K getKey ()
+  {
+    return get();
+  }
+
+  public V getValue ()
+  {
+    return value;
+  }
+
+  public V setValue (V value)
+  {
+    V old = this.value;
+    this.value = value;
+    return old;
+  }
 }
