@@ -16,7 +16,9 @@ public class MakeElement extends NodeConstructor
 
   /** Optional tag.  If non-null, the element tag is this value,
    * rather than the first parameter. */
-  public Object tag;
+  public Symbol tag;
+
+  public String toString() { return "makeElement["+tag+"]"; }
 
   public int copyNamespacesMode = XMLFilter.COPY_NAMESPACES_PRESERVE;
 
