@@ -1,6 +1,5 @@
 package gnu.kawa.xml;
 import gnu.mapping.*;
-import gnu.xml.*;
 import gnu.expr.*;
 import gnu.lists.UnescapedData;
 
@@ -14,8 +13,10 @@ public class MakeUnescapedData extends Procedure1 implements CanInline
     return new UnescapedData(arg == null ? "" : arg.toString());
   }
 
-  public Expression inline (ApplyExp exp)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
+    exp.walkArgs(walker, argsInlined);
     Expression[] args = exp.getArgs();
     if (args.length == 1 && args[0] instanceof QuoteExp)
       return new QuoteExp(apply1(((QuoteExp) args[0]).getValue()));
