@@ -12,6 +12,14 @@ public abstract class Procedure extends PropertySet
 {
   private static final String sourceLocationKey = "source-location";
   private static final Symbol setterKey = Namespace.EmptyNamespace.getSymbol("setter");
+  /** @deprecated */
+  public static final Symbol inlinerKey = Namespace.EmptyNamespace.getSymbol("inliner");
+  // This should be a LazyPropertyKey<gnu.expr.Inlineable>, but we want
+  // to avoid any strict dependency on gnu.expr for run-time classes.
+  public static final LazyPropertyKey<?> compilerKey
+  = new LazyPropertyKey("compiler");
+  public static final LazyPropertyKey<?> inlineCallsKey
+  = new LazyPropertyKey("inliner");
 
   public void setSourceLocation (String file, int line)
   {
@@ -401,6 +409,15 @@ public abstract class Procedure extends PropertySet
     getSetter().applyN(args);
   }
 
+  /** True if this Procedure (definitely) has no side-effects.
+   * Note side-effect-free does not imply idempotent if this
+   * allocates an object with "identity".
+   */
+  public boolean isSideEffectFree ()
+  {
+    return false;
+  }
+
   public String toString ()
   {
     StringBuffer sbuf = new StringBuffer();
