@@ -2,16 +2,16 @@
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.math;
-import gnu.expr.*;
-import gnu.bytecode.Method;
-import gnu.bytecode.ClassType;
-import gnu.bytecode.Access;
-import gnu.bytecode.Type;
+import java.io.*;
 
-public class DFloNum extends RealNum implements Compilable
+public class DFloNum extends RealNum implements Externalizable
 {
   double value;
 
+  public DFloNum ()
+  {
+  }
+
   public DFloNum (double value)
   {
     this.value = value;
@@ -43,6 +43,15 @@ public class DFloNum extends RealNum implements Compilable
     return new DFloNum (value);
   }
 
+  public static DFloNum asDFloNumOrNull (Object value)
+  {
+    if (value instanceof DFloNum)
+      return (DFloNum) value;
+    if (value instanceof RealNum || value instanceof Number)
+      return new DFloNum(((Number) value).doubleValue());
+    return null;
+  }
+
   public final double doubleValue ()
   {
     return value;
@@ -185,17 +194,11 @@ public class DFloNum extends RealNum implements Compilable
 	int i = compare(y_rat.numerator(), y_rat.denominator(), value);
 	return i < -1 ? i : -i;
       }
-    if (obj instanceof RealNum)
-      return compare (value, ((RealNum)obj).doubleValue ());
-    if (! (obj instanceof RealNum))
-      throw new IllegalArgumentException ();
-    return ((RealNum)obj).compareReversed(this);
+    return compare (value, ((RealNum)obj).doubleValue ());
   }
 
   public int compareReversed (Numeric x)
   {
-    if (!(x instanceof RealNum))
-      throw new IllegalArgumentException ();
     if (x instanceof RatNum)
       {
 	RatNum x_rat = (RatNum) x;
@@ -238,9 +241,9 @@ public class DFloNum extends RealNum implements Compilable
 
    public String toString ()
    {
-    return (value == 1.0/0.0 ? "#i1/0"
-	    : value == -1.0/0.0 ? "#i-1/0"
-	    : Double.isNaN (value) ? "#i0/0"
+    return (value == 1.0/0.0 ? "+inf.0"
+	    : value == -1.0/0.0 ? "-inf.0"
+	    : Double.isNaN (value) ? "+nan.0"
 	    : Double.toString (value));
    }
 
@@ -251,32 +254,18 @@ public class DFloNum extends RealNum implements Compilable
     return "#d" + toString ();
    }
 
-  static ClassType thisType;
-  public static Method makeMethod;
-
-  public static void initMakeMethods ()
+  /**
+   * @serialData Writes the number as a double (using writeDouble).
+   */
+  public void writeExternal(ObjectOutput out) throws IOException
   {
-    if (thisType == null)
-      {
-	thisType = ClassType.make("gnu.math.DFloNum");
-	Type[] args = new Type[1];
-	args[0] = Type.double_type;
-	makeMethod = thisType.addMethod ("make", args, thisType,
-					     Access.PUBLIC|Access.STATIC);
-      }
+    out.writeDouble(value);
   }
 
-  public Literal makeLiteral (Compilation comp)
+  public void readExternal(ObjectInput in)
+    throws IOException, ClassNotFoundException
   {
-    initMakeMethods ();
-    return new Literal (this, thisType, comp);
-  }
-
-  public void emit (Literal literal, Compilation comp)
-  {
-    gnu.bytecode.CodeAttr code = comp.getCode();
-    code.emitPushDouble(value);
-    code.emitInvokeStatic(makeMethod);
+    value = in.readDouble();
   }
 
   /*
