@@ -10,7 +10,7 @@ import gnu.mapping.*;
 
 public abstract class Path
 /* #ifdef JAVA6 */
-// implements javax.tools.FileObject
+implements javax.tools.FileObject
 /* #endif */
 {
   /** This is equivalent to the System {@code "user.dir} property.
@@ -22,7 +22,7 @@ public abstract class Path
   public static Path defaultPath = userDirPath;
 
   /* #ifdef JAVA2 */
-  private static ThreadLocal pathLocation = new ThreadLocal();
+  private static ThreadLocal<Path> pathLocation = new ThreadLocal<Path>();
   /* #endif */
 
   protected Path ()
@@ -32,9 +32,9 @@ public abstract class Path
   public static Path currentPath ()
   {
     /* #ifdef JAVA2 */
-    Object path = pathLocation.get();
+    Path path = pathLocation.get();
     if (path != null)
-      return (Path) path;
+      return path;
     /* #endif */
     return defaultPath;
   }
