@@ -10,7 +10,7 @@ import gnu.bytecode.*;
 import gnu.kawa.util.HashNode;
 import gnu.kawa.reflect.StaticFieldLocation;
 import gnu.kawa.reflect.SingletonType;
-import gnu.kawa.functions.GetNamedPart;
+import gnu.kawa.functions.CompileNamedPart;
 import gnu.xquery.util.NamedCollator;
 import gnu.xquery.util.QNameUtils;
 import kawa.standard.Scheme;
@@ -290,17 +290,20 @@ public class XQResolveNames extends ResolveNames
           {
             // kludge - use xxx_BUILTIN mechanism?  FIXME
             String name = sym.getLocalName();
-            Expression f;
+            String mname;
             if ("request".equals(name))
-              f = XQParser.makeFunctionExp("gnu.kawa.servlet.GetRequest", 
-                                           "getRequest");
+              mname = "getCurrentRequest";
             else if ("response".equals(name))
-              f = XQParser.makeFunctionExp("gnu.kawa.servlet.GetResponse",
-                                           "getResponse");
+              mname = "getCurrentResponse";
             else
-              f = null;
-            if (f != null)
-              return new ApplyExp(f, Expression.noExpressions);
+              mname = null;
+            if (mname != null)
+              {
+                Method meth =
+                  ClassType.make("gnu.kawa.servlet.ServletRequestContext")
+                  .getDeclaredMethod(mname, 0);
+                return new ApplyExp(meth, Expression.noExpressions);
+              }
           }
         else if (symbol instanceof Symbol)
           {
@@ -353,7 +356,7 @@ public class XQResolveNames extends ResolveNames
                             uri.startsWith("class:"))
                           {
                             ClassType ctype = ClassType.make(uri.substring(6));
-                            return GetNamedPart.makeExp(ctype, sym.getName());
+                            return CompileNamedPart.makeExp(ctype, sym.getName());
                           }
                         decl = flookup(sym);
                       }
