@@ -178,6 +178,10 @@ public class RunXQTS extends FilterConsumer
             runner.summaryReport();
             xqlog.close();
 	  }
+        catch (OutOfMemoryError ex)
+          {
+            throw ex;
+          }
 	catch (Throwable ex)
 	  {
             ex.printStackTrace();
@@ -236,23 +240,23 @@ public class RunXQTS extends FilterConsumer
                    +"fn-idref-dtd-20|fn-idref-dtd-21|fn-idref-dtd-23|",
                    "fn:idref doesn't do much yet");
     /* #ifndef use:java.text.Normalizer */
-    expectFailures("fn-normalize-unicode1args-1|"
-                   +"fn-normalize-unicode1args-2|"
-                   +"fn-normalize-unicode1args-3|"
-                   +"fn-normalize-unicode1args-4|"
-                   +"fn-normalize-unicode1args-5|"
-                   +"fn-normalize-unicode1args-6|"
-                   +"fn-normalize-unicode2args-1|"
-                   +"fn-normalize-unicode2args-2|"
-                   +"fn-normalize-unicode2args-3|"
-                   +"fn-normalize-unicode-1|"
-                   +"fn-normalize-unicode-3|fn-normalize-unicode-4|"
-                   +"fn-normalize-unicode-5|fn-normalize-unicode-6|"
-                   +"fn-normalize-unicode-7|K-NormalizeUnicodeFunc-4|"
-                   +"K-NormalizeUnicodeFunc-5|K-NormalizeUnicodeFunc-6|"
-                   +"K-NormalizeUnicodeFunc-7|K-NormalizeUnicodeFunc-8|"
-                   +"K-NormalizeUnicodeFunc-11|K-NormalizeUnicodeFunc-12",
-                   "fn:normalize-unicode not unimplemented yet");
+    // expectFailures("fn-normalize-unicode1args-1|"
+    //                +"fn-normalize-unicode1args-2|"
+    //                +"fn-normalize-unicode1args-3|"
+    //                +"fn-normalize-unicode1args-4|"
+    //                +"fn-normalize-unicode1args-5|"
+    //                +"fn-normalize-unicode1args-6|"
+    //                +"fn-normalize-unicode2args-1|"
+    //                +"fn-normalize-unicode2args-2|"
+    //                +"fn-normalize-unicode2args-3|"
+    //                +"fn-normalize-unicode-1|"
+    //                +"fn-normalize-unicode-3|fn-normalize-unicode-4|"
+    //                +"fn-normalize-unicode-5|fn-normalize-unicode-6|"
+    //                +"fn-normalize-unicode-7|K-NormalizeUnicodeFunc-4|"
+    //                +"K-NormalizeUnicodeFunc-5|K-NormalizeUnicodeFunc-6|"
+    //                +"K-NormalizeUnicodeFunc-7|K-NormalizeUnicodeFunc-8|"
+    //                +"K-NormalizeUnicodeFunc-11|K-NormalizeUnicodeFunc-12",
+    //                "fn:normalize-unicode not unimplemented yet");
     /* #endif */
     // RunXQTS failures rather than Qexo errors:
     // Some work under gcj but not JDK 1.4.x or 1.5.0_05:
@@ -392,6 +396,7 @@ public class RunXQTS extends FilterConsumer
         outputCompareAlts.clear();
         expectedErrorsBuf.setLength(1);
         manager.clear();
+        ModuleContext.getContext().clear();
         Document.clearLocalCache();
       }
     else if (tagMatches("query"))
@@ -520,12 +525,6 @@ public class RunXQTS extends FilterConsumer
         if (messages.seenErrors())
           throw new SyntaxException(messages);
       }
-    catch (OutOfMemoryError ex)
-      {
-	System.err.println("caught "+ex);
-	System.exit(-1);
-        return;
-      }
     catch (SyntaxException ex)
       {
         in.close();
@@ -599,8 +598,7 @@ public class RunXQTS extends FilterConsumer
       }
     catch (OutOfMemoryError ex)
       {
-	System.err.println("caught "+ex);
-	System.exit(-1);
+        throw ex;
       }
     catch (Throwable ex)
       {
@@ -797,6 +795,10 @@ public class RunXQTS extends FilterConsumer
                   c2 = (c2 - 0xD800) * 0x400
                     + (arg2.charAt(++i2) - 0xDC00) + 0x10000;
               }
+            catch (OutOfMemoryError ex)
+              {
+                throw ex;
+              }
             catch (Throwable ex)
               {
                 return false;
@@ -927,6 +929,10 @@ public class RunXQTS extends FilterConsumer
                 if (Float.floatToIntBits(f1) != Float.floatToIntBits(f2))
                   return false;
               }
+            catch (OutOfMemoryError ex)
+              {
+                throw ex;
+              }
             catch (Throwable ex)
               {
                 return false;
@@ -1003,6 +1009,10 @@ public class RunXQTS extends FilterConsumer
                 // Other attributes and <test-case> body written by evalTest.
                 evalTest(testName);
               }
+            catch (OutOfMemoryError ex)
+              {
+                throw ex;
+              }
             catch (Throwable ex)
               {
                 System.err.println("test-case name:"+testName);
@@ -1043,6 +1053,10 @@ public class RunXQTS extends FilterConsumer
                 current.define(symbol, null, value);
                 externalVariablesSet.push(symbol);
               }
+            catch (OutOfMemoryError ex)
+              {
+                throw ex;
+              }
             catch (Throwable ex)
               {
                 System.err.println("input-query for "+testName+": cannot open "+filename);
@@ -1081,6 +1095,10 @@ public class RunXQTS extends FilterConsumer
             else
               contextItem = value;
           }
+        catch (OutOfMemoryError ex)
+          {
+            throw ex;
+          }
         catch (Throwable ex)
           {
             System.err.println("caught "+ex);
@@ -1104,7 +1122,6 @@ public class RunXQTS extends FilterConsumer
     else if (tagMatches("defaultCollection"))
       {
         String inputFile = getElementValue();
-        Object inputValue = sources.get(inputFile);
         Object val = NodeUtils.getSavedCollection("collection:"+inputFile);
         NodeUtils.setSavedCollection("#default", val);
       }
@@ -1126,7 +1143,11 @@ public class RunXQTS extends FilterConsumer
             KDocument value = (KDocument) Document.parseCached(path);
             collectionDocuments.writeObject(value);
           }
-         catch (Throwable ex)
+        catch (OutOfMemoryError ex)
+          {
+            throw ex;
+          }
+        catch (Throwable ex)
           {
             System.err.println("caught "+ex);
             System.err.println("reading data file "+path);
@@ -1153,7 +1174,7 @@ public class RunXQTS extends FilterConsumer
         String mclass = Compilation.mangleURI(uri)
           + '.' + XQuery.makeClassName(mpath);
         ModuleInfo minfo = manager.findWithSourcePath(mpath);
-        minfo.className = mclass;
+        minfo.setClassName(mclass);
         minfo.setNamespaceUri(uri);
       }
     /*
