@@ -28,19 +28,18 @@ public class CastableAs extends InstanceOf
   static final Method castableMethod
     = CastAs.typeXDataType.getDeclaredMethod("castable", 1);
 
-  public Expression inline (ApplyExp exp, ExpWalker walker)
+  public Expression inline (ApplyExp exp, InlineCalls walker,
+                            boolean argsInlined)
   {
-    exp = Invoke.inlineClassName(exp, 1, (InlineCalls) walker);
+    exp.walkArgs(walker, argsInlined);
+    exp = Invoke.inlineClassName(exp, 1, walker);
     Expression[] args = exp.getArgs();
     if (args.length != 2 || ! (args[1] instanceof QuoteExp))
       return exp;
     Object type = ((QuoteExp) args[1]).getValue();
     if (type instanceof XDataType)
-      {
-        XDataType xtype = (XDataType) type;
-        return new ApplyExp(castableMethod,
-                            new Expression[] { args[1], args[0] });
-      }
+      return new ApplyExp(castableMethod,
+                          new Expression[] { args[1], args[0] });
     return exp;
   }
 
