@@ -87,7 +87,7 @@ public class Lambda extends Syntax
 	  {
 	    SyntaxForm sf = (SyntaxForm) bindings;
 	    // FIXME
-	    bindings = sf.form;
+	    bindings = sf.getDatum();
 	  }
 	if (! (bindings instanceof Pair))
 	  break;
@@ -95,7 +95,7 @@ public class Lambda extends Syntax
         // An initial pass to count the parameters.
 	Object pair_car = pair.getCar();
 	if (pair_car instanceof SyntaxForm)
-	  pair_car = ((SyntaxForm) pair_car).form;
+	  pair_car = ((SyntaxForm) pair_car).getDatum();
 	if (pair_car == optionalKeyword)
 	  {
 	    if (opt_args >= 0)
@@ -192,10 +192,10 @@ public class Lambda extends Syntax
 	if (bindings instanceof SyntaxForm)
 	  {
 	    SyntaxForm sf = (SyntaxForm) bindings;
-	    bindings = sf.form;
+	    bindings = sf.getDatum();
 	    // The SyntaxForm "surrounds" both the current binding (the car),
 	    // as well as the cdr - i.e. the remaining bindings.
-	    templateScopeRest = sf.scope;
+	    templateScopeRest = sf.getScope();
 	  }
 	TemplateScope templateScope = templateScopeRest;
 	if (! (bindings instanceof Pair))
@@ -205,8 +205,8 @@ public class Lambda extends Syntax
 	if (pair_car instanceof SyntaxForm)
 	  {
 	    SyntaxForm sf = (SyntaxForm) pair_car;
-	    pair_car = sf.form;
-	    templateScope = sf.scope;
+	    pair_car = sf.getDatum();
+	    templateScope = sf.getScope();
 	  }
 	if (pair_car == optionalKeyword
 	    || pair_car == restKeyword || pair_car == keyKeyword)
@@ -249,8 +249,8 @@ public class Lambda extends Syntax
 	    if (pair_car instanceof SyntaxForm)
 	      {
 		SyntaxForm sf = (SyntaxForm) pair_car;
-		pair_car = sf.form;
-		templateScope = sf.scope;
+		pair_car = sf.getDatum();
+		templateScope = sf.getScope();
 	      }
             pair_car = tr.namespaceResolve(pair_car);
 	    if (pair_car instanceof Symbol
@@ -326,7 +326,7 @@ public class Lambda extends Syntax
 	Translator.setLine(decl, bindings);
 	if (typeSpecPair != null)
 	  {
-	    decl.setType(tr.exp2Type(typeSpecPair));
+            decl.setTypeExp(new LangExp(typeSpecPair));
 	    decl.setFlag(Declaration.TYPE_SPECIFIED);
 	  }
 	else if (mode == restKeyword)
@@ -338,8 +338,8 @@ public class Lambda extends Syntax
     if (bindings instanceof SyntaxForm)
       {
 	SyntaxForm sf = (SyntaxForm) bindings;
-	bindings = sf.form;
-	templateScopeRest = sf.scope;
+	bindings = sf.getDatum();
+	templateScopeRest = sf.getScope();
       }
     if (bindings instanceof Symbol)
       {
@@ -366,48 +366,46 @@ public class Lambda extends Syntax
     String allocationFlagName = null;
     int accessFlag = 0;
     int allocationFlag = 0;
-    SyntaxForm syntax = null;
+    SyntaxForm syntax0 = null;
     for (;;)
       {
 	while (body instanceof SyntaxForm)
 	  {
-	    syntax = (SyntaxForm) body;
-	    body = syntax.form;
+	    syntax0 = (SyntaxForm) body;
+	    body = syntax0.getDatum();
 	  }
 	if (! (body instanceof Pair))
 	  break;
 	Pair pair1 = (Pair) body;
 	Object attrName = Translator.stripSyntax(pair1.getCar());
+	if (tr.matches(attrName, "::"))
+	  attrName = null;
+	else if (! (attrName instanceof Keyword))
+	  break;
+
+        SyntaxForm syntax1 = syntax0;
 	Object pair1_cdr = pair1.getCdr();
 	while (pair1_cdr instanceof SyntaxForm)
 	  {
-	    syntax = (SyntaxForm) pair1_cdr;
-	    pair1_cdr = syntax.form;
+	    syntax1 = (SyntaxForm) pair1_cdr;
+	    pair1_cdr = syntax1.getDatum();
 	  }
 	if (! (pair1_cdr instanceof Pair))
 	  break;
 	Pair pair2 = (Pair) pair1_cdr;
 
 	Object attrValue;
-	if (tr.matches(attrName, "::"))
-	  attrName = null;
-	else if (! (attrName instanceof Keyword))
-	  break;
 	if (attrName == null)
 	  {
-            Expression attrExpr = tr.rewrite_car(pair2, syntax);
             if (lexp.isClassMethod() && "*init*".equals(lexp.getName()))
               tr.error('e', "explicit return type for '*init*' method");
             else
-              {
-                gnu.bytecode.Type rtype = tr.getLanguage().getTypeFor(attrExpr);
-                if (rtype != null)
-                  lexp.setReturnType(rtype);
-              }
+              // Defer rewrite until rewriteBody.
+              lexp.body = new LangExp(new Object[] { pair2, syntax1 });
 	  }
 	else if (attrName == kawa.standard.object.accessKeyword)
 	  {
-	    Expression attrExpr = tr.rewrite_car(pair2, syntax);
+	    Expression attrExpr = tr.rewrite_car(pair2, syntax1);
 	    if (! (attrExpr instanceof QuoteExp)
 		|| ! ((attrValue = ((QuoteExp) attrExpr).getValue()) instanceof SimpleSymbol
                       /* #ifdef use:java.lang.CharSequence */
@@ -444,7 +442,7 @@ public class Lambda extends Syntax
 	  }
 	else if (attrName == kawa.standard.object.allocationKeyword)
 	  {
-	    Expression attrExpr = tr.rewrite_car(pair2, syntax);
+	    Expression attrExpr = tr.rewrite_car(pair2, syntax1);
 	    if (! (attrExpr instanceof QuoteExp)
 		|| ! ((attrValue = ((QuoteExp) attrExpr).getValue()) instanceof SimpleSymbol
                       /* #ifdef use:java.lang.CharSequence */
@@ -484,16 +482,16 @@ public class Lambda extends Syntax
 	    else
 	      {
 		ReferenceExp[] exps = new ReferenceExp[count];
-		SyntaxForm syntaxLocal = syntax;
+		SyntaxForm syntax2 = syntax1;
 		for (int i = 0;  i < count; i++)
 		  {
 		    while (attrValue instanceof SyntaxForm)
 		      {
-			syntaxLocal = (SyntaxForm) attrValue;
-			attrValue = syntaxLocal.form;
+			syntax2 = (SyntaxForm) attrValue;
+			attrValue = syntax2.getDatum();
 		      }
 		    Pair pair3 = (Pair) attrValue;
-		    Expression throwsExpr = tr.rewrite_car(pair3, syntaxLocal);
+		    Expression throwsExpr = tr.rewrite_car(pair3, syntax2);
 		    if (throwsExpr instanceof ReferenceExp)
 		      {
 			exps[i] = (ReferenceExp) throwsExpr;
@@ -511,7 +509,7 @@ public class Lambda extends Syntax
 	  }
         else if (attrName == nameKeyword)
           {
-            Expression attrExpr = tr.rewrite_car(pair2, syntax);
+            Expression attrExpr = tr.rewrite_car(pair2, syntax1);
             if (attrExpr instanceof QuoteExp)
               lexp.setName(((QuoteExp) attrExpr).getValue().toString());
           }
@@ -524,8 +522,8 @@ public class Lambda extends Syntax
     accessFlag |= allocationFlag;
     if (accessFlag != 0)
       lexp.nameDecl.setFlag(accessFlag);
-    if (syntax != null)
-      body = syntax.fromDatumIfNeeded(body);
+    if (syntax0 != null)
+      body = SyntaxForms.fromDatumIfNeeded(body, syntax0);
     return body;
   }
 
@@ -555,6 +553,7 @@ public class Lambda extends Syntax
         && lexp.nameDecl != null
         && tr.getModule().getFlag(ModuleExp.SUPERTYPE_SPECIFIED))
       tr.curMethodLambda = lexp;
+    ScopeExp curs = tr.currentScope();
     tr.pushScope(lexp);
     Declaration prev = null;
     int key_args = lexp.keywords == null ? 0 : lexp.keywords.length;
@@ -573,6 +572,12 @@ public class Lambda extends Syntax
 	    numRenamedAlias++;
 	    cur = param;
 	  }
+        Expression texp = cur.getTypeExp();
+        if (texp instanceof LangExp)
+          {
+            Pair typeSpecPair = (Pair) ((LangExp) texp).getLangValue(); 
+            cur.setType(tr.exp2Type(typeSpecPair));
+          }
 	prev = cur;
 
         if (arg_i >= lexp.min_args
@@ -597,6 +602,14 @@ public class Lambda extends Syntax
 
     LambdaExp saveLambda = tr.curLambda;
     tr.curLambda = lexp;
+    Type rtype = lexp.returnType;
+    if (lexp.body instanceof LangExp)
+      {
+        Object[] tform = (Object[]) ((LangExp) lexp.body).getLangValue();
+        Expression texp = tr.rewrite_car((Pair) tform[0],
+                                         (SyntaxForm) tform[1]);
+        rtype = tr.getLanguage().getTypeFor(texp);
+      }
     lexp.body = tr.rewrite_body (body);
     tr.curLambda = saveLambda;
     Expression[] exps;
@@ -619,10 +632,10 @@ public class Lambda extends Syntax
             System.arraycopy(exps, 1, new_body, 0, len);
             lexp.body = BeginExp.canonicalize(new_body);
           }
-        Convert.setCoercedReturnValue(lexp, rexp, tr.getLanguage());
+        lexp.setCoercedReturnValue(rexp, tr.getLanguage());
       }
     else
-      lexp.setCoercedReturnType(lexp.returnType);
+      lexp.setCoercedReturnType(rtype);
     tr.pop(lexp);
     lexp.countDecls();
     tr.popRenamedAlias(numRenamedAlias);
