@@ -11,7 +11,11 @@ import gnu.lists.*;
 import gnu.kawa.lispexpr.*;
 import java.util.*;
 import gnu.kawa.functions.GetNamedPart;
+import gnu.kawa.functions.CompileNamedPart;
 import gnu.text.SourceLocator;
+/* #ifdef enable:XML */
+import gnu.xml.NamespaceBinding;
+/* #endif */
 
 /** Used to translate from source to Expression.
  * The result has macros expanded, lexical names bound, etc, and is
@@ -49,18 +53,18 @@ public class Translator extends Compilation
 
   public LambdaExp curMethodLambda;
 
+  /* #ifdef enable:XML */
+  public NamespaceBinding xmlElementNamespaces = NamespaceBinding.predefinedXML;
+  /* #endif */
+
   public static final Declaration getNamedPartDecl;
-  public static final StaticFieldLocation getNamedPartLocation;
   static {
     // Declare the special symbol $lookup$ (from the reader)
     // and bind it to getNamedPartDecl.
     String cname = "gnu.kawa.functions.GetNamedPart";
     String fname = "getNamedPart";
     getNamedPartDecl = Declaration.getDeclarationFromStatic(cname, fname);
-    StaticFieldLocation loc = new StaticFieldLocation(cname, fname);
-    loc.setProcedure();
-    loc.setDeclaration(getNamedPartDecl);
-    getNamedPartLocation = loc;
+    LispLanguage.getNamedPartLocation.setDeclaration(getNamedPartDecl);
   }
 
   /** Return true if decl is lexical and not fluid. */
@@ -101,13 +105,13 @@ public class Translator extends Compilation
 
   public final Expression rewrite_car (Pair pair, SyntaxForm syntax)
   {
-    if (syntax == null || syntax.scope == current_scope
+    if (syntax == null || syntax.getScope() == current_scope
 	|| pair.getCar() instanceof SyntaxForm)
       return rewrite_car(pair, false);
     ScopeExp save_scope = current_scope;
     try
       {
-	setCurrentScope(syntax.scope);
+	setCurrentScope(syntax.getScope());
 	return rewrite_car(pair, false);
       }
     finally
@@ -190,7 +194,7 @@ public class Translator extends Compilation
     if (form instanceof SyntaxForm)
       {
 	// FIXME
-	form = ((SyntaxForm) form).form;
+	form = ((SyntaxForm) form).getDatum();
       }
     if (form instanceof SimpleSymbol && ! selfEvaluatingSymbol(form))
       {
@@ -211,7 +215,7 @@ public class Translator extends Compilation
     if (form instanceof SyntaxForm)
       {
 	// FIXME
-	form = ((SyntaxForm) form).form;
+	form = ((SyntaxForm) form).getDatum();
       }
     if (form instanceof SimpleSymbol && ! selfEvaluatingSymbol(form))
       {
@@ -222,6 +226,15 @@ public class Translator extends Compilation
     return form == literal;
   }
 
+  public Object matchQuoted (Pair pair)
+  {
+    if (matches(pair.getCar(), LispLanguage.quote_sym)
+        && pair.getCdr() instanceof Pair
+        && (pair = (Pair) pair.getCdr()).getCdr() == LList.Empty)
+      return pair.getCar();
+    return null;
+  }
+
   public Declaration lookup(Object name, int namespace)
   {
     Declaration decl = lexical.lookup(name, namespace);
@@ -292,12 +305,14 @@ public class Translator extends Compilation
 
   public Expression rewrite_pair (Pair p, boolean function)
   {
-    if (p.getCar() instanceof Syntax)
-      return apply_rewrite((Syntax) p.getCar(), p);
-    Object cdr = p.getCdr();
-
     Expression func = rewrite_car (p, true);
     Object proc = null;
+    if (func instanceof QuoteExp)
+      {
+        proc = func.valueIfConstant();
+        if (proc instanceof Syntax)
+          return apply_rewrite((Syntax) proc, p);
+      }
     ReferenceExp ref = null;
     if (func instanceof ReferenceExp)
       {
@@ -354,6 +369,7 @@ public class Translator extends Compilation
 	  func.setFlag(ReferenceExp.PREFER_BINDING2);
       }
 
+    Object cdr = p.getCdr();
     int cdr_length = listLength(cdr);
 
     if (cdr_length == -1)
@@ -370,8 +386,8 @@ public class Translator extends Compilation
 	if (cdr instanceof SyntaxForm)
 	  {
 	    SyntaxForm sf = (SyntaxForm) cdr;
-	    cdr = sf.form;
-	    setCurrentScope(sf.scope);
+	    cdr = sf.getDatum();
+	    setCurrentScope(sf.getScope());
 	  }
 	Pair cdr_pair = (Pair) cdr;
 	Expression arg = rewrite_car (cdr_pair, false);
@@ -416,7 +432,7 @@ public class Translator extends Compilation
         if (sym != null)
           return rewrite(sym, function);
         // FIXME don't copy the args array in makeExp ...
-        return GetNamedPart.makeExp(part1, part2);
+        return CompileNamedPart.makeExp(part1, part2);
       }
     return ((LispLanguage) getLanguage()).makeApply(func, args);
   }
@@ -471,14 +487,14 @@ public class Translator extends Compilation
   public static Object stripSyntax (Object obj)
   {
     while (obj instanceof SyntaxForm)
-      obj = ((SyntaxForm) obj).form;
+      obj = ((SyntaxForm) obj).getDatum();
     return obj;
   }
 
   public static Object safeCar (Object obj)
   {
     while (obj instanceof SyntaxForm)
-      obj = ((SyntaxForm) obj).form;
+      obj = ((SyntaxForm) obj).getDatum();
     if (! (obj instanceof Pair))
       return null;
     return stripSyntax(((Pair) obj).getCar());
@@ -487,7 +503,7 @@ public class Translator extends Compilation
   public static Object safeCdr (Object obj)
   {
     while (obj instanceof SyntaxForm)
-      obj = ((SyntaxForm) obj).form;
+      obj = ((SyntaxForm) obj).getDatum();
     if (! (obj instanceof Pair))
       return null;
     return stripSyntax(((Pair) obj).getCdr());
@@ -509,9 +525,9 @@ public class Translator extends Compilation
       {
 	// 'n' is number of previous Pairs before 'fast' cursor.
 	while (fast instanceof SyntaxForm)
-	  fast = ((SyntaxForm) fast).form;
+	  fast = ((SyntaxForm) fast).getDatum();
 	while (slow instanceof SyntaxForm)
-	  slow = ((SyntaxForm) slow).form;
+	  slow = ((SyntaxForm) slow).getDatum();
 	if (fast == LList.Empty)
 	  return n;
 	if (! (fast instanceof Pair))
@@ -519,7 +535,7 @@ public class Translator extends Compilation
 	n++;
 	Object next = ((Pair) fast).getCdr();
 	while (next instanceof SyntaxForm)
-	  next = ((SyntaxForm) next).form;
+	  next = ((SyntaxForm) next).getDatum();
 	if (next == LList.Empty)
 	  return n;
 	if (! (next instanceof Pair))
@@ -540,8 +556,8 @@ public class Translator extends Compilation
 	ScopeExp save_scope = current_scope;
 	try
 	  {
-	    setCurrentScope(sf.scope);
-	    rewriteInBody(sf.form);
+	    setCurrentScope(sf.getScope());
+	    rewriteInBody(sf.getDatum());
 	  }
 	finally
 	  {
@@ -581,7 +597,7 @@ public class Translator extends Compilation
             Symbol sym = namespaceResolve(part1, part2);
             if (sym != null)
               return sym;
-            String combinedName = GetNamedPart.combineName(part1, part2);
+            String combinedName = CompileNamedPart.combineName(part1, part2);
             if (combinedName != null)
               return Namespace.EmptyNamespace.getSymbol(combinedName);
           }
@@ -600,8 +616,8 @@ public class Translator extends Compilation
 	ScopeExp save_scope = current_scope;
 	try
 	  {
-	    setCurrentScope(sf.scope);
-	    Expression s = rewrite(sf.form, function);
+	    setCurrentScope(sf.getScope());
+	    Expression s = rewrite(sf.getDatum(), function);
 	    return s;
 	  }
 	finally
@@ -662,7 +678,7 @@ public class Translator extends Compilation
                   part1 = new ReferenceExp(((ClassExp) caller.outer).nameDecl);
                 else
                   part1 = new ThisExp(caller.firstDecl());
-                return GetNamedPart.makeExp(part1,
+                return CompileNamedPart.makeExp(part1,
                                             QuoteExp.getInstance(dname));
               }
           }
@@ -728,7 +744,11 @@ public class Translator extends Compilation
                     if (! inlineOk(null)
                         // A kludge - we get a bunch of testsuite failures
                         // if we don't inline $lookup$.  FIXME.
-                        && decl != getNamedPartDecl)
+                        && (decl != getNamedPartDecl
+                            && ! ("objectSyntax".equals(floc.getMemberName())
+                                  // Another kludge to support "object" as a
+                                  // type specifier.
+                                  && "kawa.standard.object".equals(floc. getDeclaringClass().getName()))))
                       decl = null;
                     else if (immediate)
                       {
@@ -781,6 +801,13 @@ public class Translator extends Compilation
           }
 	if (decl != null)
           {
+            // A special kludge to deal with the overloading between the
+            // object macro and object as being equivalent to java.lang.Object.
+            // A cleaner solution would be to use an identifier macro.
+            if (! function
+                && decl.getConstantValue() instanceof kawa.standard.object)
+              return QuoteExp.getInstance(Object.class);
+
             // In Scheme/Lisp a variable cannot donate multiple values.
             decl.setFlag(Declaration.IS_SINGLE_VALUE);
             if (decl.getContext() instanceof PatternScope)
@@ -824,7 +851,7 @@ public class Translator extends Compilation
   public Object pushPositionOf(Object pair)
   {
     if (pair instanceof SyntaxForm)
-      pair = ((SyntaxForm) pair).form;
+      pair = ((SyntaxForm) pair).getDatum();
     if (! (pair instanceof PairWithPosition))
       return null;
     PairWithPosition ppair = (PairWithPosition) pair;
@@ -875,8 +902,22 @@ public class Translator extends Compilation
         texp = new InlineCalls(this).walk(texp);
 	if (texp instanceof ErrorExp)
 	  return null;
-	Type type = getLanguage().getTypeFor(texp);
-	 if (type == null)
+        Type type = getLanguage().getTypeFor(texp);
+        if (type == null)
+          {
+            try
+              {
+                Object t = texp.eval(env);
+                if (t instanceof Class)
+                  type = Type.make((Class) t);
+                else if (t instanceof Type)
+                  type = (Type) t;
+              }
+            catch (Throwable ex)
+              {
+              }
+          }
+        if (type == null)
 	   {
 	     if (texp instanceof ReferenceExp)
 	       error('e', "unknown type name '"
@@ -919,7 +960,7 @@ public class Translator extends Compilation
     if (syntax == null || form instanceof Expression)
       return form;
     else
-      return syntax.fromDatumIfNeeded(form);
+      return SyntaxForms.fromDatumIfNeeded(form, syntax);
   }
 
   public Object popForms (int first)
@@ -949,9 +990,9 @@ public class Translator extends Compilation
 	ScopeExp save_scope = currentScope();
 	try
 	  {
-	    setCurrentScope(sf.scope);
+	    setCurrentScope(sf.getScope());
 	    int first = formStack.size();
-	    scanForm(sf.form, defs);
+	    scanForm(sf.getDatum(), defs);
 	    formStack.add(wrapSyntax(popForms(first), sf));
 	    return;
 	  }
@@ -979,14 +1020,16 @@ public class Translator extends Compilation
         Syntax syntax = null;
         ScopeExp savedScope = current_scope;
         Object savedPosition = pushPositionOf(st);
+        if (st instanceof SourceLocator && defs.getLineNumber() < 0)
+          defs.setLocation((SourceLocator) st);
         try
           {
             Object obj = st_pair.getCar();
             if (obj instanceof SyntaxForm)
               {
                 SyntaxForm sf = (SyntaxForm) st_pair.getCar();
-                setCurrentScope(sf.scope);
-                obj = sf.form;
+                setCurrentScope(sf.getScope());
+                obj = sf.getDatum();
               }
             Pair p;
             if (obj instanceof Pair
@@ -1085,9 +1128,9 @@ public class Translator extends Compilation
 	    ScopeExp save_scope = current_scope;
 	    try
 	      {
-		setCurrentScope(sf.scope);
+		setCurrentScope(sf.getScope());
 		int first = formStack.size();
-		Object f = scanBody(sf.form, defs, makeList);
+		Object f = scanBody(sf.getDatum(), defs, makeList);
                 if (makeList)
                   {
                     f = wrapSyntax(f, sf);
@@ -1300,7 +1343,8 @@ public class Translator extends Compilation
 		 : "' declared but never defined");
 	    error('e', decl, msg1, msg2);
 	  }
-	if (mexp.getFlag(ModuleExp.EXPORT_SPECIFIED))
+	if (mexp.getFlag(ModuleExp.EXPORT_SPECIFIED)
+            || (generateMain && ! immediate))
 	  {
 	    if (decl.getFlag(Declaration.EXPORT_SPECIFIED))
 	      {
@@ -1468,13 +1512,13 @@ public class Translator extends Compilation
 
   public Declaration define (Object name, SyntaxForm nameSyntax, ScopeExp defs)
   {
-    boolean aliasNeeded = nameSyntax != null && nameSyntax.scope != currentScope();
+    boolean aliasNeeded = nameSyntax != null && nameSyntax.getScope() != currentScope();
     Object declName = aliasNeeded ? new String(name.toString()) : name;
     Declaration decl = defs.getDefine(declName, 'w', this);
     if (aliasNeeded)
       {
-	Declaration alias = makeRenamedAlias(name, decl, nameSyntax.scope);
-	nameSyntax.scope.addDeclaration(alias);
+	Declaration alias = makeRenamedAlias(name, decl, nameSyntax.getScope());
+        nameSyntax.getScope().addDeclaration(alias);
       }
     push(decl);
     return decl;
