@@ -1,8 +1,9 @@
 (require <kawa.lib.prim_syntax>)
 (require <kawa.lib.std_syntax>)
 (require <kawa.lib.syntax>)
+(require <kawa.lib.misc>)
 
-(define (number? x) :: <boolean> (instance? x <number>))
+(define (number? x) :: <boolean> (java.lang.Number? x))
 (define (quantity? x) :: <boolean>  (instance? x <quantity>))
 (define (complex? x) :: <boolean>  (instance? x <complex>))
 (define (real? x) :: <boolean> (instance? x <real>))
@@ -20,19 +21,28 @@
 	       (instance? x <java.lang.Short>)
 	       (instance? x <java.math.BigInteger>)))))
 
-(define (exact? x) :: <boolean> 
-  (and (instance? x <number>) (invoke (as <number> x) 'isExact)))
+(define (exact? x) :: boolean 
+  (and (java.lang.Number? x)
+       (gnu.kawa.functions.Arithmetic:isExact (as java.lang.Number x))))
 
-(define (inexact? x) :: <boolean>
-  (and (instance? x <number>) (not (invoke (as <number> x) 'isExact))))
+(define (inexact? x) :: boolean
+  (and (java.lang.Number? x)
+       (not (gnu.kawa.functions.Arithmetic:isExact (as java.lang.Number x)))))
 
-(define (zero? (x :: <number>)) :: <boolean> 
-  (invoke x 'isZero))
+(define (zero? (x :: java.lang.Number)) :: boolean
+  (cond ((gnu.math.Numeric? x)
+	 ((as gnu.math.Numeric x):isZero))
+	((java.math.BigInteger? x)
+	 (= 0 (as java.math.BigInteger x):signum))
+	((java.math.BigDecimal? x)
+	 (= 0 (as java.math.BigDecimal x):signum))
+	(else
+	 (= 0.0 (x:doubleValue)))))
 
 (define (positive? (x :: <real>)) :: <boolean>
   (> (invoke x 'sign) 0))
 
-(define (negative? (x :: <real>)) :: <boolean> 
+(define (negative? (x :: real)) :: <boolean> 
   (invoke x 'isNegative))
 
 (define (odd? (x :: <integer>)) :: <boolean> 
@@ -58,35 +68,23 @@
       (set! result
 	    (*:min result (args i))))))
 
-(define (abs (x :: <number>)) :: <number>
-  (invoke x 'abs))
+(define (abs (x :: java.lang.Number)) :: java.lang.Number
+  (cond ((gnu.math.Numeric? x)
+	 ((as gnu.math.Numeric x):abs))
+	((>= x 0)
+	 x)
+	(else
+	 (- x))))
 
-(define-procedure quotient
-  ;; (x :: <real>) (y :: <real>)) :: <real>
-  (lambda ((x :: <integer>) (y :: <integer>)) :: <integer>
-	  (invoke-static <integer> 'quotient x y))
-  (lambda ((x :: <real>) (y :: <real>)) :: <real>
-	  (invoke (/ x y) 'toInt (static-field <number> 'TRUNCATE))))
+(define (div-and-mod (x :: real) (y :: real))
+  (let* ((q (div x y))
+	 (r (- x (* q y))))
+    (values q r)))
 
-(define-procedure remainder
-  ;; (x :: <real>) (y :: <real>)) :: <real> 
-  (lambda ((x :: <integer>) (y :: <integer>)) :: <integer>
-	  (invoke-static <integer> 'remainder x y))
-  (lambda ((x :: <real>) (y :: <real>)) :: <real>
-	  (if (zero? y)
-	      (if (exact? y) x (exact->inexact x))
-	      (- x (* (invoke (/ x y) 'toInt (static-field <number> 'TRUNCATE))
-		      y)))))
-
-(define-procedure modulo
-  ;; (x :: <real>) (y :: <real>)) :: <real>
-  (lambda ((x :: <integer>) (y :: <integer>)) :: <integer>
-	  (invoke-static <integer> 'modulo x y))
-  (lambda ((x :: <real>) (y :: <real>)) :: <real>
-	  (if (zero? y)
-	      (if (exact? y) x (exact->inexact x))
-	      (- x (* (invoke (/ x y) 'toInt (static-field <number> 'FLOOR))
-		      y)))))
+(define (div0-and-mod0 (x :: real) (y :: real))
+  (let* ((q (div0 x y))
+	 (r (- x (* q y))))
+    (values q r)))
 
 (define (gcd #!rest (args :: <Object[]>)) :: <integer>
   (let ((n :: <int> args:length))
@@ -107,22 +105,22 @@
 	    (set! result (gnu.math.IntNum:lcm result (<integer>:@ (args i)))))))))
 
 (define (numerator (x :: <rational>)) :: <integer>
-  (invoke x 'numerator))
+  (x:numerator))
 
 (define (denominator (x :: <rational>)) :: <integer>
-  (invoke x 'denominator))
+  (x:denominator))
 
-(define (floor (x :: <real>)) :: <real>
-  (invoke x 'toInt (static-field <number> 'FLOOR)))
+(define (floor (x :: real)) :: real
+  (x:toInt gnu.math.Numeric:FLOOR))
 
-(define (ceiling (x :: <real>)) :: <real>
-  (invoke x 'toInt (static-field <number> 'CEILING)))
+(define (ceiling (x :: real)) :: real
+  (x:toInt gnu.math.Numeric:CEILING))
 
-(define (truncate (x :: <real>)) :: <real>
-  (invoke x 'toInt (static-field <number> 'TRUNCATE)))
+(define (truncate (x :: real)) :: real
+  (x:toInt gnu.math.Numeric:TRUNCATE))
 
-(define (round (x :: <real>)) :: <real>
-  (invoke x 'toInt (static-field <number> 'ROUND)))
+(define (round (x :: real)) :: real
+  (x:toInt gnu.math.Numeric:ROUND))
 
 (define (rationalize (x :: <real>) (y :: <real>)) :: <real>
   (gnu.math.RatNum:rationalize
@@ -177,35 +175,23 @@
 (define (imag-part (x :: <complex>)) :: <real>
   (invoke x 'im))
 
-(define (magnitude (x :: <number>)) :: <number>
-  (invoke x 'abs))
+(define (magnitude (x :: java.lang.Number)) :: java.lang.Number
+  (abs x))
 
 (define (angle (x :: <complex>)) :: <real>
   (invoke x 'angle))
 
-(define (exact->inexact (num :: <number>)) :: <number>
-  (if (invoke num 'isExact)
-      (make <gnu.math.DFloNum> (invoke (as <real> num) 'doubleValue))
-      num))
+(define (inexact (num :: java.lang.Number)) :: java.lang.Number
+  (gnu.kawa.functions.Arithmetic:toInexact num))
 
-(define (inexact->exact (num :: <number>)) :: <number>
-  (if (instance? num <real>)
-      (invoke (as <real> num) 'toExact)
-      num))
+(define (exact (num :: java.lang.Number)) :: java.lang.Number
+  (gnu.kawa.functions.Arithmetic:toExact num))
 
-(define (bitwise-arithmetic-shift (value :: <integer>) (amount :: <int>)) :: <integer>
-  (gnu.math.IntNum:shift value amount))
+(define (exact->inexact (num :: java.lang.Number)) :: java.lang.Number
+  (gnu.kawa.functions.Arithmetic:toInexact num))
 
-(define (bitwise-arithmetic-shift-left (value :: <integer>) (amount :: <int>)) :: <integer>
-  (if (< amount 0) (throw 'misc-error "shift amount must be non-negative"))
-  (gnu.math.IntNum:shift value amount))
-
-(define (bitwise-arithmetic-shift-right (value :: <integer>) (amount :: <int>)) :: <integer>
-  (if (< amount 0) (throw 'misc-error "shift amount must be non-negative"))
-  (gnu.math.IntNum:shift value (- amount)))
-
-(define (bitwise-not (i :: <integer>)) :: <integer>
-  (gnu.math.BitOps:not i))
+(define (inexact->exact (num :: java.lang.Number)) :: java.lang.Number
+  (gnu.kawa.functions.Arithmetic:toExact num))
 
 (define (logop (op :: <int>) (i :: <integer>) (j :: <integer>)) :: <integer>
   (invoke-static <gnu.math.BitOps> 'bitOp op i j))
@@ -218,48 +204,20 @@
   (gnu.math.BitOps:setBitValue i bitno new-value))
 
 (define (bitwise-copy-bit-field (to :: integer) (start :: int) (end :: int) (from :: integer)) ::  integer
-  (let* ((mask1 (bitwise-arithmetic-shift-left -1 start))
-	 (mask2 (bitwise-not (bitwise-arithmetic-shift-left -1 end)))
-	 (mask (bitwise-and mask1 mask2)))
+  (let* ((mask1 (gnu.math.IntNum:shift -1 start))
+	 (mask2 (gnu.math.BitOps:not (gnu.math.IntNum:shift -1 end)))
+	 (mask (gnu.math.BitOps:and mask1 mask2)))
     (bitwise-if mask
-		(bitwise-arithmetic-shift-left from start)
+		(gnu.math.IntNum:shift from start)
 		to)))
 
 (define (bitwise-bit-field (i :: <integer>) (start :: <int>) (end :: <int>))
   :: <integer>
   (invoke-static <gnu.math.BitOps> 'extract i start end))
 
-(define (bitwise-and #!rest (args :: <Object[]>)) :: <integer>
-  (let ((n :: <int> args:length))
-    (if (zero? n)
-	-1
-	(let ((result :: <integer> (args 0)))
-	  (do ((i :: <int> 1 (+ i 1)))
-	      ((>= i n) result)
-	    (let ((arg-i :: <integer> (args i)))
-	      (set! result (gnu.math.BitOps:and result arg-i))))))))
-
-(define (bitwise-ior #!rest (args :: <Object[]>)) :: <integer>
-  (let ((n :: <int> args:length))
-    (if (zero? n)
-	0
-	(let ((result :: <integer> (args 0)))
-	  (do ((i :: <int> 1 (+ i 1)))
-	      ((>= i n) result)
-	    (set! result (gnu.math.BitOps:ior result (args i))))))))
-
-(define (bitwise-xor #!rest (args :: <Object[]>)) :: <integer>
-  (let ((n :: <int> args:length))
-    (if (zero? n)
-	0
-	(let ((result :: <integer> (args 0)))
-	  (do ((i :: <int> 1 (+ i 1)))
-	      ((>= i n) result)
-	    (set! result (gnu.math.BitOps:xor result (args i))))))))
-
 (define (bitwise-if (e1 :: integer) (e2 :: integer) (e3  integer)) :: integer
-  (bitwise-ior (bitwise-and e1 e2)
-	       (bitwise-and (bitwise-not e1) e3)))
+  (gnu.math.BitOps:ior (gnu.math.BitOps:and e1 e2)
+		       (gnu.math.BitOps:and (gnu.math.BitOps:not e1) e3)))
 
 (define (logtest (i :: <integer>) (j :: <integer>))
   (invoke-static <gnu.math.BitOps> 'test i j))
@@ -281,14 +239,14 @@
 
 (define (bitwise-rotate-bit-field (n :: integer) (start :: int) (end :: int) (count :: int)) :: integer
   (let ((width (- end start)))
-    (if (positive? width)
-	(let* ((count (modulo count width))
+    (if (> width 0)
+	(let* (;; Optimization of modulo.
+	       (r (remainder count width))
+	       (count (if (< r 0) (+ r width) r))
 	       (field0 (bitwise-bit-field n start end))
-	       (field1 (bitwise-arithmetic-shift-left field0 count))
-	       (field2 (bitwise-arithmetic-shift-right
-			field0
-			(- width count)))
-	       (field (bitwise-ior field1 field2)))
+	       (field1 (gnu.math.IntNum:shift field0 count))
+	       (field2 (gnu.math.IntNum:shift field0 (- count width)))
+	       (field (gnu.math.BitOps:ior field1 field2)))
 	  (bitwise-copy-bit-field n start end field))
 	n)))
 
