@@ -22,9 +22,6 @@
         (lambda (form)
       (syntax-case form literals . parts))))))
 
-(define (gentemp) :: <symbol>
-  (invoke-static <gnu.expr.Symbols> 'gentemp))
-
 (define-syntax when (syntax-rules ()
 				  ((when cond exp ...)
 				   (if cond (begin exp ...)))))
@@ -33,11 +30,6 @@
 				  ((when cond exp ...)
 				   (if (not cond) (begin exp ...)))))
 
-(define (catch key (thunk :: <procedure>) (handler :: <procedure>))
-  (try-catch (thunk)
-	     (ex <kawa.lang.NamedException>
-		 (invoke ex 'applyHandler key handler))))
-
 (define-syntax (try-finally x)
   (syntax-case x ()
 	       ((_ try-part finally-part)
@@ -164,19 +156,16 @@
 (define-syntax (case-lambda form)
   (syntax-case form ()
     ((_ . cl)
-     (make <pair> ; The cons function isn't visible yet.
-       (syntax make-procedure)
-       (let loop ((clauses (syntax cl)))
-	 (syntax-case clauses ()
-	   (((formals . body) . rest)
-	    (make <pair>
-	      (syntax (lambda formals . body))
-	      (loop (syntax rest))))
-	   (()
-	    '())
-	   (rest
-	    (list (syntax-error (syntax rest)
-				"invalid case-lambda clause")))))))))
+     #`(gnu.expr.GenericProc:makeWithoutSorting
+	. ,(let loop ((clauses #'cl))
+	     (syntax-case clauses ()
+	       (((formals . body) . rest)
+		#`((lambda formals . body) . ,(loop #'rest)))
+	       (()
+		'())
+	       (rest
+		(list (syntax-error (syntax rest)
+				    "invalid case-lambda clause")))))))))
 
 ;; COND-EXPAND implementation from http://srfi.schemers.org/srfi-0/srfi-0.html
 ;; Copyright (C) Marc Feeley (1999). All Rights Reserved.
@@ -263,3 +252,20 @@
     ((receive formals expression body ...)
      (call-with-values (lambda () expression)
                        (lambda formals body ...)))))
+
+(define-syntax define-alias-parameter
+  (syntax-rules ()
+    ((define-alias-parameter name type location)
+     (begin
+       (define-constant name :: <gnu.mapping.LocationProc>
+	 (gnu.mapping.LocationProc:makeNamed 'name location))
+       (gnu.mapping.LocationProc:pushConverter
+	name
+	(lambda (arg)
+	  (try-catch
+	   (as type arg)
+	   (ex <java.lang.ClassCastException>
+	       (let ((wt (gnu.mapping.WrongType:make ex name
+						     (as <int> 1) arg)))
+		 (set! (field wt 'expectedType) type)
+		 (primitive-throw wt))))))))))
