@@ -8,6 +8,7 @@ import gnu.text.SyntaxException;
 import gnu.lists.*;
 import java.util.Vector;
 import gnu.bytecode.ClassType;
+import gnu.kawa.servlet.HttpRequestContext;
 
 /** Start a "Read-Eval-Print-Loop" for the Kawa Scheme evaluator. */
 
@@ -18,6 +19,8 @@ public class repl extends Procedure0or1
   Language language;
   static Language previousLanguage;
 
+  static int defaultParseOptions = Language.PARSE_PROLOG|Language.PARSE_EXPLICIT;
+
   public repl(Language language)
   {
     this.language = language;
@@ -100,7 +103,7 @@ public class repl extends Procedure0or1
     printOption(out, "--main", "Generate an application, with a main method");
     printOption(out, "--applet", "Generate an applet");
     printOption(out, "--servlet", "Generate a servlet");
-    printOption(out, "--module-static", "Top-leval definitions are by default static");
+    printOption(out, "--module-static", "Top-level definitions are by default static");
 
     Vector keys = Compilation.options.keys();
     for (int i = 0; i < keys.size(); ++i)
@@ -113,39 +116,6 @@ public class repl extends Procedure0or1
     out.println("For more information go to:  http://www.gnu.org/software/kawa/");
   }
 
-  /** Number of times exitDecrement calls before we exit. */
-  private static int exitCounter;
-  /** See exitDecrement. */
-  public static synchronized void exitIncrement()
-  {
-    if (exitCounter == 0)
-      exitCounter++;
-    exitCounter++;
-  }
-
-  /** Work around an AWT bug, where AWT threads are non-daemon.
-   * Thus if you start up AWT, the JVM will wait for the AWT to finish,
-   * even if there are no other non-daemon threads.
-   * So call exitIncrement() each time a Freme is created,
-   * and call exitDecrement() a Frame is closed. */
-  public static synchronized void exitDecrement()
-  {
-    int counter = exitCounter;
-    if (counter > 0)
-      {
-	counter--;
-	if (counter == 0)
-	  {
-	    System.exit(0);
-	  }
-	else
-	  exitCounter = counter;
-      }
-  }
-
-  public static String[] commandLineArgArray;
-  public static FVector commandLineArguments;
-
   public static String homeDirectory;
 
   static void checkInitFile ()
@@ -169,30 +139,14 @@ public class repl extends Procedure0or1
 	  scmHomeDirectory = Boolean.FALSE;
 	Environment.getCurrent().put("home-directory", scmHomeDirectory);
 	if (initFile != null && initFile.exists())
-	  if (! Shell.runFile(initFile.getPath(), 0))
+	  if (! Shell.runFileOrClass(initFile.getPath(), true, 0))
             System.exit(-1);
       }
   }
 
   public static void setArgs (String[] args, int arg_start)
   {
-    int nargs = args.length - arg_start;
-    Object[] array = new Object[nargs];
-    if (arg_start == 0)
-      commandLineArgArray = args;
-    else
-      {
-	String[] strings = new String[nargs];
-	for (int i = nargs;  --i >= 0; )
-	  strings[i] = args[i+arg_start];
-	commandLineArgArray = strings;
-      }
-    for (int i = nargs;  --i >= 0; )
-      array[i] = new FString (args[i + arg_start]);
-    commandLineArguments = new FVector (array);  // FIXME scsh has list
-    // FIXME scsh also has command-line proc
-    Environment.getCurrent().put("command-line-arguments",
-				 commandLineArguments);
+    ApplicationMainSupport.setArgs(args, arg_start);
   }
 
   public static void getLanguageFromFilenameExtension(String name)
@@ -258,7 +212,7 @@ public class repl extends Procedure0or1
 	    getLanguageFromFilenameExtension(filename);
 	    setArgs (args, iArg+1);
 	    checkInitFile();
-	    if (! Shell.runFile (filename, 0))
+	    if (! Shell.runFileOrClass(filename, true, 0))
               System.exit(-1);
 	    something_done = true;
 	  }
@@ -284,7 +238,7 @@ public class repl extends Procedure0or1
 	    getLanguageFromFilenameExtension(filename);
 	    setArgs (args, iArg+1);
 	    checkInitFile();
-	    if (! Shell.runFile(filename, skipLines))
+	    if (! Shell.runFileOrClass(filename, true, skipLines))
               System.exit(-1);
             return -1;
 	  }
@@ -543,17 +497,77 @@ public class repl extends Procedure0or1
 		throw new Error(ex.toString());
 	      }
 	  }
-	else if (arg.equals("--main"))
+        else if (arg.equals("--http-auto-handler"))
+          {
+            iArg += 2;
+	    if (iArg >= maxArg)
+	      bad_option (arg);
+            /* #ifdef use:com.sun.net.httpserver */
+            try
+              {
+                gnu.kawa.servlet.KawaHttpHandler.addAutoHandler(args[iArg-1], args[iArg]);
+              }
+            catch (java.io.IOException ex)
+	      {
+		throw new RuntimeException(ex);
+	      }
+            catch (NoClassDefFoundError ex)
+              {
+                System.err.println("kawa: HttpServer classes not found");
+                System.exit(-1);
+              }
+            /* #else */
+            // System.err.println("kawa: HttpServer classes not found");
+            // System.exit(-1);
+            /* #endif */
+          }
+        else if (arg.equals("--http-start"))
+          {
+            iArg += 1;
+	    if (iArg >= maxArg)
+	      bad_option("missing httpd port argument");
+            /* #ifdef use:com.sun.net.httpserver */
+            int port;
+            try
+              {
+                port = Integer.parseInt(args[iArg]);
+              }
+            catch (NumberFormatException ex)
+              {
+                bad_option("malformed server port#");
+                port = -1; // never seen.
+              }
+            try
+              {
+                gnu.kawa.servlet.KawaHttpHandler.startServer(port);
+              }
+            catch (NoClassDefFoundError ex)
+              {
+                System.err.println("kawa: HttpServer classes not found");
+                System.exit(-1);
+              }
+            catch (IOException ex)
+              {
+                throw new RuntimeException(ex);
+              }
+	    something_done = true;
+            /* #else */
+            // System.err.println("kawa: HttpServer classes not found");
+            // System.exit(-1);
+            /* #endif */
+          }
+        else if (arg.equals("--main"))
 	  {
 	    Compilation.generateMainDefault = true;
 	  }
 	else if (arg.equals("--applet"))
 	  {
-	    Compilation.generateAppletDefault = true;
+	    defaultParseOptions |= Language.PARSE_FOR_APPLET;
 	  }
 	else if (arg.equals("--servlet"))
 	  {
-	    Compilation.generateServletDefault = true;
+	    defaultParseOptions |= Language.PARSE_FOR_SERVLET;
+            HttpRequestContext.importServletDefinitions = 2;
 	  }
 	else if (arg.equals("--debug-dump-zip"))
 	  {
@@ -575,28 +589,33 @@ public class repl extends Procedure0or1
 	  {
             SourceMessages.debugStackTraceOnWarning = true;
 	  }
+	else if (arg.equals("--module-nonstatic")
+                 || arg.equals("--no-module-static"))
+	  {
+	    Compilation.moduleStatic = Compilation.MODULE_NONSTATIC;
+	  }
 	else if (arg.equals("--module-static"))
 	  {
-	    gnu.expr.Compilation.moduleStatic = 1;
+	    Compilation.moduleStatic = Compilation.MODULE_STATIC;
 	  }
         else if (arg.equals("--module-static-run"))
           {
-            gnu.expr.Compilation.moduleStatic = 2;
+            Compilation.moduleStatic = Compilation.MODULE_STATIC_RUN;
           }
 	else if (arg.equals("--fewer-classes"))
 	  {
-	    gnu.expr.Compilation.fewerClasses = true;
+	    Compilation.fewerClasses = true;
 	  }
 	else if (arg.equals("--no-inline")
 		 || arg.equals("--inline=none"))
 	  {
-	    gnu.expr.Compilation.inlineOk = false;
+	    Compilation.inlineOk = false;
 	  }
         else if (arg.equals("--no-console"))
           noConsole = true;
 	else if (arg.equals("--inline"))
 	  {
-	    gnu.expr.Compilation.inlineOk = true;
+	    Compilation.inlineOk = true;
 	  }
 	else if (arg.equals("--cps"))
 	  {
@@ -692,44 +711,8 @@ public class repl extends Procedure0or1
 		  }
 	      }
 	  }
-	else
-          {
-            int ci = arg.indexOf('=');
-            if (ci <= 0)
-              break;
-            String key = arg.substring(0, ci);
-            String value = arg.substring(ci+1);
-            for (int i = 0; ; i++)
-              {
-                String[] propertyField = propertyFields[i];
-                if (propertyField == null)
-                  break;
-                if (key.equals(propertyField[0]))
-                  {
-                    String cname = propertyField[1];
-                    String fname = propertyField[2];
-                    try
-                      {
-                        Class clas = Class.forName(cname);
-                        ThreadLocation loc = (ThreadLocation)
-                          clas.getDeclaredField(fname).get(null);
-                        loc.setGlobal(value);
-                        break;
-                      }
-                    catch (Throwable ex)
-                      {
-                        System.err.println("error setting property " + key
-                                           +" field "+cname+'.'+fname+": "+ex);
-                        System.exit(-1);
-                      }
-                  }
-              }
-            Symbol symbol = Symbol.parse(key);
-            // Run Language's static initializer.
-            Language.getDefaultLanguage();
-            Environment current = Environment.getCurrent();
-            current.define(symbol, null, value);
-          }
+	else if (! ApplicationMainSupport.processSetProperty(arg))
+          break;
       }
     return something_done ? -1 : iArg;
   }
@@ -761,7 +744,8 @@ public class repl extends Procedure0or1
               }
             
             comp
-              = language.parse(fstream, messages, Language.PARSE_PROLOG);
+              = language.parse(fstream, messages,
+                               defaultParseOptions);
 
             if (compilationTopname != null)
               {
@@ -840,25 +824,6 @@ public class repl extends Procedure0or1
     System.exit(-1);
   }
 
-  /** A list of standard command-line fluid names to map to static fields.
-   * For each entry:
-   * element 0 is a property name (before the '=' in the comamnd-line);
-   * element 1 is the name of a class;
-   * element 2 is the name of a static ThreadLocation field. */
-  static String[][] propertyFields =
-    {
-      { "out:doctype-system", "gnu.xml.XMLPrinter", "doctypeSystem" },
-      { "out:doctype-public", "gnu.xml.XMLPrinter", "doctypePublic" },
-      { "out:base", "gnu.kawa.functions.DisplayFormat", "outBase" },
-      { "out:radix", "gnu.kawa.functions.DisplayFormat", "outRadix" },
-      { "out:line-length", "gnu.text.PrettyWriter", "lineLengthLoc" },
-      { "out:right-margin", "gnu.text.PrettyWriter", "lineLengthLoc" },
-      { "out:miser-width", "gnu.text.PrettyWriter", "miserWidthLoc" },
-      { "out:xml-indent", "gnu.xml.XMLPrinter", "indentLoc" },
-      { "display:toolkit", "gnu.kawa.models.Display", "myDisplay" },
-      null
-    };
-
   public static void main(String args[])
   {
     try
@@ -873,7 +838,7 @@ public class repl extends Procedure0or1
 	    getLanguageFromFilenameExtension(filename);
 	    setArgs (args, iArg+1);
 	    checkInitFile();
-	    ok = Shell.runFile(filename, 0);
+	    ok = Shell.runFileOrClass(filename, false, 0);
 	  }
 	else
 	  {
@@ -898,7 +863,7 @@ public class repl extends Procedure0or1
 	    // Redundant if registerShutdownHook succeeded (e.g on JDK 1.3).
 	    gnu.mapping.OutPort.runCleanups();
 	  }
-	exitDecrement();
+	ModuleBody.exitDecrement();
       }
   }
 
