@@ -10,7 +10,7 @@ public class IfFeature
     if (form instanceof SyntaxForm)
       {
 	SyntaxForm sf = (SyntaxForm) form;
-	form = sf.form;
+	form = sf.getDatum();
       }
     if (form instanceof String || form instanceof SimpleSymbol)
       return hasFeature(form.toString());
@@ -50,6 +50,32 @@ public class IfFeature
       return true;
     if (name == "srfi-39") // Parameter objects
       return true;
+    /* #ifdef use:java.text.Normalizer */
+    if (name == "string-normalize-unicode")
+      {
+        /* #ifdef JAVA6COMPAT5 */
+        // try
+        //   {
+        //     Class.forName("java.text.Normalizer");
+        //     return true;
+        //   }
+        // catch (ClassNotFoundException ex)
+        //   {
+        //     return false;
+        //   }
+        /* #else */
+        return true;
+        /* #endif */
+      }
+    /* #endif */
+    if (name == "in-http-server" || name == "in-servlet")
+      {
+        int mflags = ModuleContext.getContext().getFlags();
+        if (name == "in-http-server")
+          return (mflags & ModuleContext.IN_HTTP_SERVER) != 0;
+        if (name == "in-servlet")
+          return (mflags & ModuleContext.IN_SERVLET) != 0;
+      }
 
     String provide_name = ("%provide%"+name).intern();
     Compilation comp = Compilation.getCurrent();
