@@ -12,17 +12,13 @@ import gnu.text.Lexer;
 import gnu.text.SourceMessages;
 import gnu.kawa.lispexpr.*;
 import gnu.lists.AbstractFormat;
-import gnu.kawa.functions.Apply;
-import gnu.kawa.functions.ApplyToArgs;
-import gnu.kawa.functions.DisplayFormat;
-import gnu.kawa.functions.NumberCompare;
-import gnu.kawa.functions.GetNamedPart;
-import gnu.kawa.functions.MultiplyOp;
+import gnu.kawa.functions.*;
 import gnu.kawa.reflect.ClassMethods;
 import gnu.kawa.reflect.StaticFieldLocation;
 import gnu.math.DFloNum;
 import gnu.kawa.xml.XmlNamespace;
 import gnu.math.Unit;
+import gnu.kawa.servlet.HttpRequestContext;
 
 public class Scheme extends LispLanguage
 {
@@ -35,13 +31,12 @@ public class Scheme extends LispLanguage
   public static final Scheme instance;
 
   public static final gnu.kawa.reflect.InstanceOf instanceOf;
-  public static final not not;
+  public static final Not not;
   public static final kawa.standard.map map;
   public static final kawa.standard.map forEach;
   public static final gnu.kawa.functions.IsEq isEq;
   public static final gnu.kawa.functions.IsEqv isEqv;
   public static final gnu.kawa.functions.IsEqual isEqual;
-  public static final kawa.repl repl;
 
   public static final NumberCompare numEqu;
   public static final NumberCompare numGrt;
@@ -62,7 +57,7 @@ public class Scheme extends LispLanguage
 
     instance = new Scheme(kawaEnvironment);
     instanceOf = new gnu.kawa.reflect.InstanceOf(instance, "instance?");
-    not = new not(instance, "not");
+    not = new Not(instance, "not");
     applyToArgs = new ApplyToArgs("apply-to-args", instance);
     applyFieldDecl
       = Declaration.getDeclarationFromStatic("kawa.standard.Scheme",
@@ -84,12 +79,20 @@ public class Scheme extends LispLanguage
     numLEq = NumberCompare.make(instance, "<=",
                                 NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU);
 
-    // Declare the special symbol $lookup$ (from the reader)
-    // and bind it to getNamedPartDecl.
-    instance.environ.addLocation(LispLanguage.lookup_sym, null, Translator.getNamedPartLocation);
-
-    repl = new kawa.repl(instance);
     instance.initScheme();
+
+    int withServlets = HttpRequestContext.importServletDefinitions;
+    if (withServlets > 0)
+      {
+        try
+          {
+            instance.loadClass(withServlets > 1 ? "gnu.kawa.servlet.servlets"
+                               : "gnu.kawa.servlet.HTTP");
+          }
+        catch (Throwable ex)
+          {
+          }
+      }
   }
 
   public static Scheme getInstance()
@@ -102,14 +105,13 @@ public class Scheme extends LispLanguage
     return kawaEnvironment;
   }
 
-  public static final Lambda lambda = new kawa.lang.Lambda();
-  static { lambda.setKeywords(Special.optional, Special.rest, Special.key); }
-
   private void initScheme ()
   {
       environ = nullEnvironment;
 
-      defSntxStFld("lambda", "kawa.standard.Scheme", "lambda");
+      environ.addLocation(LispLanguage.lookup_sym, null, getNamedPartLocation);
+
+      defSntxStFld("lambda", "kawa.standard.SchemeCompilation", "lambda");
 
       //-- Section 4.1  -- complete
       defSntxStFld(LispLanguage.quote_sym, "kawa.lang.Quote", "plainQuote");
@@ -214,6 +216,13 @@ public class Scheme extends LispLanguage
       defProcStFld("symbol?", "kawa.lib.misc");
       defProcStFld("symbol->string", "kawa.lib.misc");
       defProcStFld("string->symbol", "kawa.lib.misc");
+      defProcStFld("symbol=?", "kawa.lib.misc");
+      defProcStFld("symbol-local-name", "kawa.lib.misc");
+      defProcStFld("symbol-namespace", "kawa.lib.misc");
+      defProcStFld("symbol-namespace-uri", "kawa.lib.misc");
+      defProcStFld("symbol-prefix", "kawa.lib.misc");
+      defProcStFld("namespace-uri", "kawa.lib.misc");
+      defProcStFld("namespace-prefix", "kawa.lib.misc");
 
       //-- Section 6.5
       defProcStFld("number?", "kawa.lib.numbers");
@@ -234,18 +243,24 @@ public class Scheme extends LispLanguage
       defProcStFld("negative?", "kawa.lib.numbers");
       defProcStFld("odd?", "kawa.lib.numbers");
       defProcStFld("even?", "kawa.lib.numbers");
-      defProcStFld ("max", "kawa.lib.numbers");
-      defProcStFld ("min", "kawa.lib.numbers");
+      defProcStFld("max", "kawa.lib.numbers");
+      defProcStFld("min", "kawa.lib.numbers");
       defProcStFld("+", "gnu.kawa.functions.AddOp", "$Pl");
       defProcStFld("-", "gnu.kawa.functions.AddOp", "$Mn");
       defProcStFld("*", "gnu.kawa.functions.MultiplyOp", "$St");
       defProcStFld("/", "gnu.kawa.functions.DivideOp", "$Sl");
       defProcStFld("abs", "kawa.lib.numbers");
-      defProcStFld("quotient", "kawa.lib.numbers");
-      defProcStFld("remainder", "kawa.lib.numbers");
-      defProcStFld("modulo", "kawa.lib.numbers");
-      defProcStFld ("gcd", "kawa.lib.numbers");
-      defProcStFld ("lcm", "kawa.lib.numbers");
+      defProcStFld("quotient", "gnu.kawa.functions.DivideOp", "quotient");
+      defProcStFld("remainder", "gnu.kawa.functions.DivideOp", "remainder");
+      defProcStFld("modulo", "gnu.kawa.functions.DivideOp", "modulo");
+      defProcStFld("div", "gnu.kawa.functions.DivideOp", "div");
+      defProcStFld("mod", "gnu.kawa.functions.DivideOp", "mod");
+      defProcStFld("div0", "gnu.kawa.functions.DivideOp", "div0");
+      defProcStFld("mod0", "gnu.kawa.functions.DivideOp", "mod0");
+      defProcStFld("div-and-mod", "kawa.lib.numbers");
+      defProcStFld("div0-and-mod0", "kawa.lib.numbers");
+      defProcStFld("gcd", "kawa.lib.numbers");
+      defProcStFld("lcm", "kawa.lib.numbers");
       defProcStFld("numerator", "kawa.lib.numbers");
       defProcStFld("denominator", "kawa.lib.numbers");
       defProcStFld("floor", "kawa.lib.numbers");
@@ -269,6 +284,8 @@ public class Scheme extends LispLanguage
       defProcStFld("imag-part", "kawa.lib.numbers");
       defProcStFld("magnitude", "kawa.lib.numbers");
       defProcStFld("angle", "kawa.lib.numbers");
+      defProcStFld("inexact", "kawa.lib.numbers");
+      defProcStFld("exact", "kawa.lib.numbers");
       defProcStFld("exact->inexact", "kawa.lib.numbers");
       defProcStFld("inexact->exact", "kawa.lib.numbers");
       defProcStFld("number->string", "kawa.lib.numbers");
@@ -418,7 +435,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("case-lambda", "kawa.lib.syntax");
       defSntxStFld("receive", "kawa.lib.syntax");
       defProcStFld("eval", "kawa.lang.Eval");
-      defProcStFld("repl", "kawa.standard.Scheme", "repl");
+      defProcStFld("repl", "kawa.standard.SchemeCompilation", "repl");
       defProcStFld("scheme-report-environment", "kawa.lib.misc");
       defProcStFld("null-environment", "kawa.lib.misc");
       defProcStFld("interaction-environment", "kawa.lib.misc");
@@ -436,28 +453,28 @@ public class Scheme extends LispLanguage
                    "kawa.standard.define_autoload",
                    "define_autoloads_from_file");
 
-      defProcStFld("exit", "kawa.lib.thread");
+      defProcStFld("exit", "kawa.lib.rnrs.programs");
+      defProcStFld("command-line", "kawa.lib.rnrs.programs");
 
-      defProcStFld("bitwise-arithmetic-shift", "kawa.lib.numbers");
-      defProcStFld("arithmetic-shift", "kawa.lib.numbers",
-                   Compilation.mangleNameIfNeeded("bitwise-arithmetic-shift"));
-      defProcStFld("ash", "kawa.lib.numbers",
-                   Compilation.mangleNameIfNeeded("bitwise-arithmetic-shift"));
-      defProcStFld("bitwise-arithmetic-shift-left", "kawa.lib.numbers");
-      defProcStFld("bitwise-arithmetic-shift-right", "kawa.lib.numbers");
-      defProcStFld("bitwise-and", "kawa.lib.numbers");
-      defProcStFld("logand", "kawa.lib.numbers",
-                   Compilation.mangleNameIfNeeded("bitwise-and"));
-      defProcStFld("bitwise-ior", "kawa.lib.numbers");
-      defProcStFld("logior", "kawa.lib.numbers",
-                   Compilation.mangleNameIfNeeded("bitwise-ior"));
-      defProcStFld("bitwise-xor", "kawa.lib.numbers");
-      defProcStFld("logxor", "kawa.lib.numbers",
-                   Compilation.mangleNameIfNeeded("bitwise-xor"));
+      defProcStFld("bitwise-arithmetic-shift",
+                   "gnu.kawa.functions.BitwiseOp", "ashift");
+      defProcStFld("arithmetic-shift",
+                   "gnu.kawa.functions.BitwiseOp", "ashift");
+      defProcStFld("ash",
+                   "gnu.kawa.functions.BitwiseOp", "ashift");
+      defProcStFld("bitwise-arithmetic-shift-left",
+                   "gnu.kawa.functions.BitwiseOp", "ashiftl");
+      defProcStFld("bitwise-arithmetic-shift-right",
+                   "gnu.kawa.functions.BitwiseOp", "ashiftr");
+      defProcStFld("bitwise-and", "gnu.kawa.functions.BitwiseOp", "and");
+      defProcStFld("logand", "gnu.kawa.functions.BitwiseOp", "and");
+      defProcStFld("bitwise-ior", "gnu.kawa.functions.BitwiseOp", "ior");
+      defProcStFld("logior", "gnu.kawa.functions.BitwiseOp", "ior");
+      defProcStFld("bitwise-xor", "gnu.kawa.functions.BitwiseOp", "xor");
+      defProcStFld("logxor", "gnu.kawa.functions.BitwiseOp", "xor");
       defProcStFld("bitwise-if", "kawa.lib.numbers");
-      defProcStFld("bitwise-not", "kawa.lib.numbers");
-      defProcStFld("lognot", "kawa.lib.numbers",
-                   Compilation.mangleNameIfNeeded("bitwise-not"));
+      defProcStFld("bitwise-not", "gnu.kawa.functions.BitwiseOp", "not");
+      defProcStFld("lognot", "gnu.kawa.functions.BitwiseOp", "not");
       defProcStFld("logop", "kawa.lib.numbers");
       defProcStFld("bitwise-bit-set?", "kawa.lib.numbers");
       defProcStFld("logbit?", "kawa.lib.numbers",
@@ -507,7 +524,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("try-finally", "kawa.lib.syntax");
       defSntxStFld("try-catch", "kawa.lib.prim_syntax");
       defProcStFld("throw", "kawa.standard.throw_name", "throwName");
-      defProcStFld("catch", "kawa.lib.syntax");
+      defProcStFld("catch", "kawa.lib.system");
       defProcStFld("error", "kawa.lib.misc");
       defProcStFld("as", "gnu.kawa.functions.Convert", "as");
       defProcStFld("instance?", "kawa.standard.Scheme", "instanceOf");
@@ -551,9 +568,10 @@ public class Scheme extends LispLanguage
       defProcStFld("syntax-source", "kawa.lib.std_syntax");
       defProcStFld("syntax-line", "kawa.lib.std_syntax");
       defProcStFld("syntax-column", "kawa.lib.std_syntax");
-      defSntxStFld("define-for-syntax", "kawa.lib.prim_syntax");
-      defSntxStFld("include", "kawa.lib.files");
-      defSntxStFld("include-relative", "kawa.lib.files");
+      defSntxStFld("begin-for-syntax", "kawa.lib.std_syntax");
+      defSntxStFld("define-for-syntax", "kawa.lib.std_syntax");
+      defSntxStFld("include", "kawa.lib.misc_syntax");
+      defSntxStFld("include-relative", "kawa.lib.misc_syntax");
 
       defProcStFld("file-exists?", "kawa.lib.files");
       defProcStFld("file-directory?", "kawa.lib.files");
@@ -574,6 +592,7 @@ public class Scheme extends LispLanguage
       defProcStFld("tokenize-string-to-string-array", "kawa.lib.system");
       defProcStFld("tokenize-string-using-shell", "kawa.lib.system");
       defProcStFld("command-parse", "kawa.lib.system");
+      defProcStFld("process-command-line-assignments", "kawa.lib.system");
       
       defProcStFld("record-accessor", "kawa.lib.reflection");
       defProcStFld("record-modifier", "kawa.lib.reflection");
@@ -599,14 +618,13 @@ public class Scheme extends LispLanguage
       defProcStFld("scheme-implementation-version", "kawa.lib.misc");
       defProcStFld("scheme-window", "kawa.lib.windows");
       defSntxStFld("define-procedure", "kawa.lib.std_syntax");
-      defProcStFld("add-procedure-properties", "kawa.lib.std_syntax");
+      defProcStFld("add-procedure-properties", "kawa.lib.misc");
       defProcStFld("make-procedure",
                    "gnu.kawa.functions.MakeProcedure", "makeProcedure");
       defProcStFld("procedure-property", "kawa.lib.misc");
       defProcStFld("set-procedure-property!", "kawa.lib.misc");
-      defSntxStFld("provide", "kawa.lib.misc");
-      defSntxStFld("test-begin", "kawa.lib.misc");
-      defProcStFld("namespace", "kawa.lib.misc");
+      defSntxStFld("provide", "kawa.lib.misc_syntax");
+      defSntxStFld("test-begin", "kawa.lib.misc_syntax");
 
       defProcStFld("quantity->number", "kawa.lib.numbers");
       defProcStFld("quantity->unit", "kawa.lib.numbers");
@@ -622,13 +640,12 @@ public class Scheme extends LispLanguage
                    "define_base_unit");
       defProcStFld("duration", "kawa.lib.numbers");
 
-      defProcStFld("gentemp", "kawa.lib.syntax");
+      defProcStFld("gentemp", "kawa.lib.misc");
       defSntxStFld("defmacro", "kawa.lib.syntax");
       defProcStFld("setter", "gnu.kawa.functions.Setter", "setter");
-      defSntxStFld("resource-url", "kawa.lib.files");
 
-      defProcStFld("resolve-uri", "kawa.lib.files");
-      defSntxStFld("module-uri", "kawa.lib.files");
+      defSntxStFld("resource-url", "kawa.lib.misc_syntax");
+      defSntxStFld("module-uri", "kawa.lib.misc_syntax");
 
       defSntxStFld("future", "kawa.lib.thread");
       defProcStFld("sleep", "kawa.lib.thread");
@@ -636,6 +653,7 @@ public class Scheme extends LispLanguage
 
       defSntxStFld("trace", "kawa.lib.trace");
       defSntxStFld("untrace", "kawa.lib.trace");
+      defSntxStFld("disassemble", "kawa.lib.trace");
 
       defProcStFld("format", "gnu.kawa.functions.Format");
       defProcStFld("parse-format", "gnu.kawa.functions.ParseFormat", "parseFormat");
@@ -792,6 +810,8 @@ public class Scheme extends LispLanguage
       defAliasStFld("path", "gnu.kawa.lispexpr.LangObjType", "pathType");
       defAliasStFld("filepath", "gnu.kawa.lispexpr.LangObjType", "filepathType");
       defAliasStFld("URI", "gnu.kawa.lispexpr.LangObjType", "URIType");
+      defProcStFld("resolve-uri", "kawa.lib.files");
+
       defAliasStFld("vector", "gnu.kawa.lispexpr.LangObjType", "vectorType");
       defAliasStFld("string", "gnu.kawa.lispexpr.LangObjType", "stringType");
       defAliasStFld("list", "gnu.kawa.lispexpr.LangObjType", "listType");
@@ -949,13 +969,14 @@ public class Scheme extends LispLanguage
 	types.put ("String", Type.toStringType);
 
 	types.put ("object", Type.objectType);
-	types.put ("number", ClassType.make("gnu.math.Numeric"));
+	types.put ("number", LangObjType.numericType);
 	types.put ("quantity", ClassType.make("gnu.math.Quantity"));
 	types.put ("complex", ClassType.make("gnu.math.Complex"));
-	types.put ("real", ClassType.make("gnu.math.RealNum"));
-	types.put ("rational", ClassType.make("gnu.math.RatNum"));
+	types.put ("real", LangObjType.realType);
+	types.put ("rational", LangObjType.rationalType);
 	types.put ("integer", LangObjType.integerType);
 	types.put ("symbol", ClassType.make("gnu.mapping.Symbol"));
+	types.put ("namespace", ClassType.make("gnu.mapping.Namespace"));
 	types.put ("keyword", ClassType.make("gnu.expr.Keyword"));
 	types.put ("pair", ClassType.make("gnu.lists.Pair"));
 	types.put ("pair-with-position",
@@ -993,7 +1014,7 @@ public class Scheme extends LispLanguage
         types.put ("u64vector", ClassType.make("gnu.lists.U64Vector"));
         types.put ("f32vector", ClassType.make("gnu.lists.F32Vector"));
         types.put ("f64vector", ClassType.make("gnu.lists.F64Vector"));
-        types.put ("document", ClassType.make("gnu.lists.TreeList"));
+        types.put ("document", ClassType.make("gnu.kawa.xml.KDocument"));
         types.put ("readtable", ClassType.make("gnu.kawa.lispexpr.ReadTable"));
       }
     Type type = (Type) types.get(name);
@@ -1024,6 +1045,14 @@ public class Scheme extends LispLanguage
       return Type.toStringType;
     if ("gnu.math.IntNum".equals(name))
       return LangObjType.integerType;
+    if ("gnu.math.DFloNum".equals(name))
+      return LangObjType.dflonumType;
+    if ("gnu.math.RatNum".equals(name))
+      return LangObjType.rationalType;
+    if ("gnu.math.RealNum".equals(name))
+      return LangObjType.realType;
+    if ("gnu.math.Numeric".equals(name))
+      return LangObjType.numericType;
     if ("gnu.lists.FVector".equals(name))
       return LangObjType.vectorType;
     if ("gnu.lists.LList".equals(name))
@@ -1076,7 +1105,7 @@ public class Scheme extends LispLanguage
   }
 
   public static final Namespace unitNamespace =
-    Namespace.make("http://kawa.gnu.org/unit", "unit");
+    Namespace.valueOf("http://kawa.gnu.org/unit", "unit");
 
   /** If a symbol is lexically unbound, look for a default binding.
    * If the symbol is the name of an existing Java type/class,
@@ -1130,6 +1159,8 @@ public class Scheme extends LispLanguage
           }
       }
     char ch0 = name.charAt(0);
+
+    // Look for quantities.
     if (ch0 == '-' || ch0 == '+' || Character.digit(ch0, 10) >= 0)
       {
         // 1: initial + or -1 seen.
@@ -1276,11 +1307,16 @@ public class Scheme extends LispLanguage
               }
           }
       }
+
+    boolean sawAngle;
     if (len > 2 && ch0 == '<' && name.charAt(len-1) == '>')
       {
         name = name.substring(1, len-1);
         len -= 2;
+        sawAngle = true;
       }
+    else
+      sawAngle = false;
     int rank = 0;
     while (len > 2 && name.charAt(len-2) == '[' && name.charAt(len-1) == ']')
       {
@@ -1295,6 +1331,14 @@ public class Scheme extends LispLanguage
       { 
         Class clas;
         Type type = getNamedType(cname);
+        if (rank > 0 && (! sawAngle || type == null))
+          {
+            Symbol tsymbol = namespace.getSymbol(cname.intern());
+            Expression texp = tr.rewrite(tsymbol, false);
+            texp = new InlineCalls(tr).walk(texp);
+            if (! (texp instanceof ErrorExp))
+              type = tr.getLanguage().getTypeFor(texp);
+          }
         if (type != null)
           {
             // Somewhat inconsistent: Types named by getNamedType are Type,
@@ -1364,7 +1408,8 @@ public class Scheme extends LispLanguage
     tab.putReaderCtorFld("path", "gnu.kawa.lispexpr.LangObjType", "pathType");
     tab.putReaderCtorFld("filepath", "gnu.kawa.lispexpr.LangObjType", "filepathType");
     tab.putReaderCtorFld("URI", "gnu.kawa.lispexpr.LangObjType", "URIType");
-    tab.putReaderCtorFld("namespace", "kawa.lib.misc", "namespace");
+    tab.putReaderCtor("symbol", ClassType.make("gnu.mapping.Symbol"));
+    tab.putReaderCtor("namespace", ClassType.make("gnu.mapping.Namespace"));
     tab.putReaderCtorFld("duration", "kawa.lib.numbers", "duration");
     tab.setFinalColonIsKeyword(true);
     return tab;
