@@ -11,7 +11,7 @@ import gnu.kawa.functions.Convert;
 public class object extends Syntax
 {
   public static final object objectSyntax
-    = new kawa.standard.object(Scheme.lambda);
+    = new kawa.standard.object(SchemeCompilation.lambda);
   static { objectSyntax.setName("object"); }
 
   Lambda lambda;
@@ -64,7 +64,7 @@ public class object extends Syntax
     Object classNamePair = null;
     LambdaExp method_list = null;
     LambdaExp last_method = null;
-    int classAccessFlag = 0;
+    long classAccessFlag = 0;
     // First pass (get Declarations).
     Vector inits = new Vector(20);
     for (Object obj = components;  obj != LList.Empty;  )
@@ -72,7 +72,7 @@ public class object extends Syntax
 	// The SyntaxForm scopes aren't used in scanClassDef, but they are
 	// used in rewriteClassDef, and might as well make the code the same.
 	while (obj instanceof SyntaxForm)
-	  obj = ((SyntaxForm) obj).form;
+	  obj = ((SyntaxForm) obj).getDatum();
 	if (! (obj instanceof Pair))
 	  {
 	    tr.error('e', "object member not a list");
@@ -81,13 +81,13 @@ public class object extends Syntax
 	pair = (Pair) obj;
 	Object pair_car = pair.getCar();
 	while (pair_car instanceof SyntaxForm)
-	  pair_car = ((SyntaxForm) pair_car).form;
+	  pair_car = ((SyntaxForm) pair_car).getDatum();
 	obj = pair.getCdr(); // Next member.
 	Object savedPos1 = tr.pushPositionOf(pair);
         if (pair_car instanceof Keyword)
           {
             while (obj instanceof SyntaxForm)
-              obj = ((SyntaxForm) obj).form;
+              obj = ((SyntaxForm) obj).getDatum();
             if (obj instanceof Pair)
               {
                 if (pair_car == interfaceKeyword)
@@ -136,7 +136,7 @@ public class object extends Syntax
 	pair = (Pair) pair_car;
 	pair_car = pair.getCar();
 	while (pair_car instanceof SyntaxForm)
-	  pair_car = ((SyntaxForm) pair_car).form;
+	  pair_car = ((SyntaxForm) pair_car).getDatum();
 	if (pair_car instanceof String || pair_car instanceof Symbol
 	    || pair_car instanceof Keyword)
 	  { // Field declaration.
@@ -145,7 +145,7 @@ public class object extends Syntax
 	    Object args;
 	    Declaration decl;
 	    int allocationFlag = 0;
-	    int accessFlag = 0;
+	    long accessFlag = 0;
 	    if (sname instanceof Keyword)
 	      {
 		decl = null;
@@ -165,12 +165,12 @@ public class object extends Syntax
 	    while (args != LList.Empty)
 	      {
                 while (args instanceof SyntaxForm)
-                  args = ((SyntaxForm) args).form;
+                  args = ((SyntaxForm) args).getDatum();
 		pair = (Pair) args;
 		Pair keyPair = pair;
 		Object key = pair.getCar();
                 while (key instanceof SyntaxForm)
-                  key = ((SyntaxForm) key).form;
+                  key = ((SyntaxForm) key).getDatum();
 		Object savedPos2 = tr.pushPositionOf(pair);
 		args = pair.getCdr();
 		if ((key == coloncolon || key instanceof Keyword)
@@ -218,13 +218,11 @@ public class object extends Syntax
 		      }
 		    else if (key == accessKeyword)
 		      {
-                        int newAccessFlag = matchAccess(value, tr);
+                        long newAccessFlag = matchAccess(value, tr);
                         if (newAccessFlag == 0)
                           tr.error('e', "unknown access specifier");
                         else if (accessFlag != 0)
-                          tr.error('e', "duplicate access specifiers - "
-                                   + accessString(accessFlag) + " and "
-                                   + accessString(newAccessFlag));
+                          tr.error('e', "duplicate access specifiers");
 			accessFlag = newAccessFlag;
 		      }
 		    else
@@ -355,10 +353,17 @@ public class object extends Syntax
 	while (superlist instanceof SyntaxForm)
 	  {
 	    // FIXME - need to pass syntax.
-	    superlist = ((SyntaxForm) superlist).form;
+	    superlist = ((SyntaxForm) superlist).getDatum();
 	  }
 	Pair superpair = (Pair) superlist;
 	supers[i] = tr.rewrite_car(superpair, false);
+        if (supers[i] instanceof ReferenceExp)
+          {
+            Declaration decl = Declaration.followAliases(((ReferenceExp) supers[i]).getBinding());
+            Expression svalue;
+            if (decl != null && (svalue = decl.getValue()) instanceof ClassExp)
+              ((ClassExp) svalue).setFlag(ClassExp.HAS_SUBCLASS);
+          }
 	superlist = superpair.getCdr();
       }
 
@@ -367,11 +372,21 @@ public class object extends Syntax
         Expression classNameExp = tr.rewrite_car((Pair) classNamePair, false);
         Object classNameVal = classNameExp.valueIfConstant();
         String classNameSpecifier;
-        if (classNameVal instanceof String
-            && (classNameSpecifier = (String) classNameVal).length() > 0)
+        boolean isString;
+        /* #ifdef use:java.lang.CharSequence */
+        isString = classNameVal instanceof CharSequence;
+        /* #else */
+        // isString = classNameVal instanceof CharSeq || classNameVal instanceof String;
+        /* #endif */
+        if (isString
+            && (classNameSpecifier = classNameVal.toString()).length() > 0)
           oexp.classNameSpecifier = classNameSpecifier;
         else
-          tr.error('e', "class-name specifier must be a non-empty string literal");
+          {
+            Object savedPos = tr.pushPositionOf(classNamePair);
+            tr.error('e', "class-name specifier must be a non-empty string literal");
+            tr.popPositionOf(savedPos);
+          }
       }
     oexp.supers = supers;
 
@@ -398,7 +413,7 @@ public class object extends Syntax
 	while (obj instanceof SyntaxForm)
 	  {
 	    componentsSyntax = (SyntaxForm) obj;
-	    obj = componentsSyntax.form;
+	    obj = componentsSyntax.getDatum();
 	  }
 	Pair pair = (Pair) obj;
 	Object savedPos1 = tr.pushPositionOf(pair);
@@ -407,7 +422,7 @@ public class object extends Syntax
 	while (pair_car instanceof SyntaxForm)
 	  {
 	    memberSyntax = (SyntaxForm) pair_car;
-	    pair_car = memberSyntax.form;
+	    pair_car = memberSyntax.getDatum();
 	  }
 	try
 	  {
@@ -425,7 +440,7 @@ public class object extends Syntax
 	    while (pair_car instanceof SyntaxForm)
 	      {
 		memberCarSyntax = (SyntaxForm) pair_car;
-		pair_car = memberCarSyntax.form;
+		pair_car = memberCarSyntax.getDatum();
 	      }
 	    if (pair_car instanceof String || pair_car instanceof Symbol
 		|| pair_car instanceof Keyword)
@@ -440,12 +455,12 @@ public class object extends Syntax
                     while (args instanceof SyntaxForm)
                       {
                         memberSyntax = (SyntaxForm) args;
-                        args = memberSyntax.form;
+                        args = memberSyntax.getDatum();
                       }
 		    pair = (Pair) args;
 		    Object key = pair.getCar();
                     while (key instanceof SyntaxForm)
-                      key = ((SyntaxForm) key).form;
+                      key = ((SyntaxForm) key).getDatum();
 		    Object savedPos2 = tr.pushPositionOf(pair);
 		    args = pair.getCdr();
 		    if ((key == coloncolon || key instanceof Keyword)
@@ -511,21 +526,17 @@ public class object extends Syntax
 		// renamed alias.  A TemplateScope that covers the formals
 		// *and* the body we handle using setCurrentScope.
 		if (memberSyntax != null)
-		  tr.setCurrentScope(memberSyntax.scope);
+		  tr.setCurrentScope(memberSyntax.getScope());
                 if ("*init*".equals(meth.getName()))
-                  {
-                    meth.setReturnType(Type.voidType);
-                    if (! oexp.isSimple())
-                      tr.error('e', "'*init*' methods only supported for simple classes");
-                  }
+                  meth.setReturnType(Type.voidType);
                 Translator.setLine(meth, pair);
                 LambdaExp saveLambda = tr.curMethodLambda;
                 tr.curMethodLambda = meth;
 		lambda.rewrite(meth, ((Pair) pair_car).getCdr(), pair.getCdr(), tr,
 			       memberCarSyntax != null
 			       && (memberSyntax == null
-				   || memberCarSyntax.scope != memberSyntax.scope)
-			       ? memberCarSyntax.scope
+				   || memberCarSyntax.getScope() != memberSyntax.getScope())
+			       ? memberCarSyntax.getScope()
 			       : null);
                 tr.curMethodLambda = saveLambda;
 		if (memberSyntax != null)
@@ -591,7 +602,7 @@ public class object extends Syntax
         initValue = sexp;
       }
     else
-      initValue = Convert.makeCoercion(initValue, new QuoteExp(Type.voidType));
+      initValue = Compilation.makeCoercion(initValue, new QuoteExp(Type.voidType));
     ((BeginExp) initMethod.body).add(initValue);
     tr.curMethodLambda = saveLambda;
     tr.pop(initMethod);
@@ -603,46 +614,67 @@ public class object extends Syntax
   static boolean matches (Object exp, String tag, Translator tr)
   {
     String value;
+    Object qvalue;
     Pair pair;
     if (exp instanceof Keyword)
       value = ((Keyword) exp).getName();
     else if (exp instanceof FString)
       value = ((FString) exp).toString();
     else if (exp instanceof Pair
-	     && tr.matches((pair = (Pair) exp).getCar(), Scheme.quote_sym)
-	     && pair.getCdr() instanceof Pair
-	     && (pair = (Pair) pair.getCdr()).getCdr() == LList.Empty
-	     && pair.getCar() instanceof gnu.mapping.SimpleSymbol)
-      value = pair.getCar().toString();
+             && (qvalue = tr.matchQuoted((Pair) exp)) instanceof gnu.mapping.SimpleSymbol)
+      value = qvalue.toString();
     else
       return false;
     return tag == null || tag.equals(value);
   }
 
-  static int matchAccess (Object value, Translator tr)
+  static long matchAccess (Object value, Translator tr)
   {
-    if (matches(value, "private", tr))
-      return Declaration.PRIVATE_ACCESS;
-    else if (matches(value, "protected", tr))
-      return Declaration.PROTECTED_ACCESS;
-    else if (matches(value, "public", tr))
-      return Declaration.PUBLIC_ACCESS;
-    else if (matches(value, "package", tr))
-      return Declaration.PACKAGE_ACCESS;
-    else
-      return 0;
+    if (value instanceof Pair)
+      {
+        Pair p = (Pair) value;
+        value = tr.matchQuoted((Pair) value);
+        if (value instanceof Pair)
+          return matchAccess2((Pair) value, tr);
+      }
+    return matchAccess1(value, tr);
   }
 
-  static String accessString (int accessFlag)
+  private static long matchAccess2 (Pair pair, Translator tr)
   {
-    if (accessFlag == Declaration.PRIVATE_ACCESS)
-      return "private";
-    if (accessFlag == Declaration.PROTECTED_ACCESS)
-      return "protected";
-    if (accessFlag == Declaration.PUBLIC_ACCESS)
-      return "public";
-    if (accessFlag == Declaration.PACKAGE_ACCESS)
-      return "package";
-    return "<internal error>";
+    long icar = matchAccess1(pair.getCar(), tr);
+    Object cdr = pair.getCdr();
+    if (cdr == LList.Empty || icar == 0)
+      return icar;
+    else if (cdr instanceof Pair)
+      {
+        long icdr = matchAccess2((Pair) cdr, tr);
+        if (icdr != 0)
+          return icar | icdr;
+      }
+    return 0;
+  }
+
+  private static long matchAccess1 (Object value, Translator tr)
+  {
+    if (value instanceof Keyword)
+      value = ((Keyword) value).getName();
+    else if (value instanceof FString)
+      value = ((FString) value).toString();
+    else if (value instanceof gnu.mapping.SimpleSymbol)
+      value = value.toString();
+    if ("private".equals(value))
+      return Declaration.PRIVATE_ACCESS;
+    if ("protected".equals(value))
+      return Declaration.PROTECTED_ACCESS;
+    if ("public".equals(value))
+      return Declaration.PUBLIC_ACCESS;
+    if ("package".equals(value))
+      return Declaration.PACKAGE_ACCESS;
+    if ("volatile".equals(value))
+      return Declaration.VOLATILE_ACCESS;
+    if ("transient".equals(value))
+      return Declaration.TRANSIENT_ACCESS;
+    return 0;
   }
 }
