@@ -36,7 +36,7 @@ public class require extends Syntax
 
      One complication is knowing whether a (require <B>) refers to a
      source file to be compiled.  It is not enough to check if a class
-     B exists, since if we're compiljng B we want to use the current
+     B exists, since if we're compiling B we want to use the current
      source B.scm, not an older B.class.  This is complicated by the
      (module-name B) declaration: We don't know whether source file
      B.scm provides the B class until we've parsed B.scm.  A solution
@@ -75,8 +75,13 @@ public class require extends Syntax
     map("readtable", SLIB_PREFIX + "readtable");
     map("srfi-10", SLIB_PREFIX + "readtable");
     map("http", "gnu.kawa.servlet.HTTP");
+    map("servlets", "gnu.kawa.servlet.servlets");
     map("srfi-1", SLIB_PREFIX + "srfi1");
     map("list-lib", SLIB_PREFIX + "srfi1");
+    map("srfi-2", SLIB_PREFIX + "srfi2");
+    map("and-let*", SLIB_PREFIX + "srfi2");
+    map("srfi-13", SLIB_PREFIX + "srfi13");
+    map("string-lib", SLIB_PREFIX + "srfi13");
     map("srfi-34", SLIB_PREFIX + "srfi34");
     map("srfi-35", SLIB_PREFIX + "conditions");
     map("condition", SLIB_PREFIX + "conditions");
@@ -223,6 +228,7 @@ public class require extends Syntax
     ModuleManager manager = ModuleManager.getInstance();
     long now;
     if ((info.getState() & 1) == 0
+        && info.getCompilation() == null
         && ! info.checkCurrent(manager, (now = System.currentTimeMillis())))
       {
         SourceMessages messages = tr.getMessages();
@@ -232,7 +238,7 @@ public class require extends Syntax
           {
             InPort fstream = InPort.openFile(info.getSourceAbsPath());
             info.clearClass();
-            info.className = className;
+            info.setClassName(className);
             int options = Language.PARSE_PROLOG;
             if (tr.immediate)
               options |= Language.PARSE_IMMEDIATE;
@@ -255,9 +261,7 @@ public class require extends Syntax
             // otherwise ignore it - it's already been recorded in messages.
             return false;
           }
-        ModuleExp mexp = comp.getModule();
-        ClassType ctype = mexp.classFor(comp);
-        info.className = ctype.getName();
+        info.setClassName(comp.getModule().classFor(comp).getName());
       }
 
     if (tr.minfo != null && tr.getState() < Compilation.BODY_PARSED)
@@ -275,7 +279,7 @@ public class require extends Syntax
       }
 
     ClassType type = info.getClassType();
-    String tname = info.className;
+    String tname = type.getName();
     boolean immediate = tr.immediate && defs instanceof ModuleExp;
     boolean isRunnable = (info.getState() < Compilation.RESOLVED
                           ? info.getCompilation().makeRunnable()
@@ -405,7 +409,6 @@ public class require extends Syntax
         if (! isImportedInstance)
           {
             fref.setDontDereference(true);
-            fref.setFlag(ReferenceExp.CREATE_FIELD_REFERENCE);
             if (! immediate)
               adecl.setPrivate(true);
           } 
@@ -484,7 +487,7 @@ public class require extends Syntax
         dofind.setLine(tr);
         forms.addElement(dofind);
       }
-    tr.mustCompileHere();
+
     return true;
   }
 
