@@ -3,7 +3,6 @@ import kawa.lang.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.lists.*;
-import gnu.kawa.functions.Setter;
 
 /**
  * The Syntax transformer that re-writes the Scheme "set!" primitive.
@@ -22,7 +21,7 @@ public class set_b extends Syntax
     while (o1 instanceof SyntaxForm)
       {
 	syntax = (SyntaxForm) o1;
-	o1 = syntax.form;
+	o1 = syntax.getDatum();
       }
     if (! (o1 instanceof Pair))
       return tr.syntaxError ("missing name");
@@ -32,7 +31,7 @@ public class set_b extends Syntax
     while (o2 instanceof SyntaxForm)
       {
 	syntax = (SyntaxForm) o2;
-	o2 = syntax.form;
+	o2 = syntax.getDatum();
       }
     Pair p2;
     if (! (o2 instanceof Pair)
@@ -60,7 +59,8 @@ public class set_b extends Syntax
 	Expression[] xargs = new Expression[nargs+1];
 	System.arraycopy(args, skip, xargs, 0, nargs);
 	xargs[nargs] = value;
-	return new ApplyExp(new ApplyExp(new ReferenceExp(Setter.setterDecl),
+        Declaration setter = gnu.kawa.functions.CompilationHelpers.setterDecl;
+	return new ApplyExp(new ApplyExp(new ReferenceExp(setter),
                                          setterArgs), xargs);
       }
     else if (! (name instanceof ReferenceExp))
@@ -78,7 +78,12 @@ public class set_b extends Syntax
 	if (decl != null)
 	  decl.noteValue (value);
 	if (decl.getFlag(Declaration.IS_CONSTANT))
-	  return tr.syntaxError ("constant variable is set!");
+	  return tr.syntaxError ("constant variable "+decl.getName()+" is set!");
+        else if (decl.context != tr.mainLambda
+                 && decl.context instanceof ModuleExp
+                 && ! decl.context.getFlag(ModuleExp.IMMEDIATE))
+          // Should be an error, but for now make it a warning
+	  tr.error('w', decl, "imported variable ", " is set!");
       }
     return sexp;
   }
