@@ -1,4 +1,4 @@
-(test-init "Miscellaneous" 176)
+(test-init "Miscellaneous" 198)
 
 ;;; DSSSL spec example 11
 (test '(3 4 5 6) (lambda x x) 3 4 5 6)
@@ -45,12 +45,25 @@
 	(cons x y))
       100)
 
-(test #t keyword? foo:)
 (test #t keyword? 'foo:)
 (test #f keyword? 'foo\:)
+(test #t keyword? 'foo:)
+(test #t keyword? foo:)
+(test #f keyword? 'foo)
+(test #f keyword? ':)
+(test #t keyword? ||:)
+(test #t keyword? (car '(a: b:)))
+(test #f keyword? "bar")
 
 ;;; DSSSL spec example 44
 (test "Argentina" keyword->string \Argentina:)
+(test "foo" keyword->string foo:)
+(test "" keyword->string ||:)
+(test "a b c" keyword->string (string->keyword "a b c"))
+(test foo: string->keyword "foo")
+(test ||: string->keyword "")
+
+(test "Hello" symbol->string 'H\x65;llo)
 
 ;;; DSSSL spec example 45
 (test foobar: string->keyword "foobar")
@@ -125,6 +138,7 @@
 (call-with-input-string
  cr-test-string
  (lambda (iport)
+   (iport:setConvertCR #t)
    (test 1 input-port-column-number iport)
    (test 1 input-port-line-number iport)
    (test 'a read iport)
@@ -273,9 +287,9 @@
 (define f8-10 ((caddr f7-20) 10))
 (test '(100 20 10 2) 'closure-test3 ((cadddr f8-10) 2))
 
-(define (f6 a)
+(define (f60 a)
   (define (x6 b) a)
-  (define (f7 b)
+  (define (f70 b)
     (define (x7 c) b)
     (define (f8 c)
       (define (x8 d) c)
@@ -283,11 +297,11 @@
         (list a b c d))
       (list a b c f9))
     (list a b f8))
-  (list a f7))
-(define f6-100 (f6 100)) 
-(define f7-20 ((cadr f6-100) 20)) 
-(define f8-10 ((caddr f7-20) 10)) 
-(test '(100 20 10 2) 'closure-test4 ((cadddr f8-10) 2)) 
+  (list a f70))
+(define f60-100 (f60 100)) 
+(define f70-20 ((cadr f60-100) 20)) 
+(define f80-10 ((caddr f70-20) 10)) 
+(test '(100 20 10 2) 'closure-test4 ((cadddr f80-10) 2))
 
 ;; A bug reported by Edward Mandac <ed@texar.com>.
 (test "Done" 'do-future (do   ((test 'empty))
@@ -713,7 +727,22 @@
 (test "<b xmlns=\"http://www.w3.org/1999/xhtml\"><code>Foo</code></b>" 'html-contructor-4
       (as-xml (html:b (html:code "Foo"))))
 
-(test "<list><b xmlns=\"http://www.w3.org/1999/xhtml\">bold 1</b> <b xmlns=\"http://www.w3.org/1999/xhtml\">bold2</b></list>" 'html-contructor-4 (as-xml (map html:b '("bold 1" "bold2"))))
+(test "<code xmlns=\"http://www.w3.org/1999/xhtml\">Foo</code>" 'html-contructor-1lit
+      (as-xml #<html:code>Foo</html:code>))
+(test "<a xmlns=\"http://www.w3.org/1999/xhtml\" href=\"foo.html\">Foo</a>" 'html-contructor-2lit
+      (as-xml #<html:a {'href}="&{"foo"}.&(string-append "ht" "ml")">Foo</>))
+(define-xml-namespace h "HTML")
+(test "<h:code xmlns:h=\"HTML\">Foo</h:code>" 'html-contructor-3lit
+      (as-xml #<h:code>Foo</>))
+(test "<b xmlns=\"http://www.w3.org/1999/xhtml\"><code>Foo</code></b>" 'html-contructor-4lit
+      (as-xml #<html:b><html:code>Foo</></>))
+(test "<b xmlns=\"http://www.w3.org/1999/xhtml\"><code>FooBar</code></b>" 'html-contructor-4enc
+      (let ((body1 "Foo")
+	    (body2 "Bar")
+	    (code 'html:code))
+	(as-xml #<{(quote html:b)}><{code}>&{body1}&(car (list body2))</></>)))
+
+(test "<list><b xmlns=\"http://www.w3.org/1999/xhtml\">bold 1</b> <b xmlns=\"http://www.w3.org/1999/xhtml\">bold2</b></list>" 'html-contructor-5 (as-xml (map html:b '("bold 1" "bold2"))))
 
 ;; Based on Savannah bug#18736, "intenal compile error -- svn rev 5816".
 ;; From Thomas Kirk <tk@research.att.com>
@@ -889,3 +918,27 @@
 
 (require "test-cycle12.scm")
 (test '(8 12) c1x-c2x)
+
+;; Savannah bug #27257 "non-int dim. spec. in emitNewArray"
+(define (alloc-array count val)
+  (object[] length: (+ 1 count) 1: val 2: (+ 1 val)))
+(test "[#!null 10 11 #!null #!null]" 'test-savannah-27257
+      (format #f "~s" (alloc-array 4 10)))
+
+(define shared-1 '(#2=(3 4) 9 #2# #2#))
+(test '((3 4) 9 (3 4) (3 4)) 'shared-1 shared-1)
+
+(test 25 'multiple-do-with-type-specs
+      (let ((ll '(1 3 5 7 9)))
+	(do ((x :: list ll (cdr x))
+	     (sum :: int 0 (+ sum (car x))))
+	    ((null? x) sum))))
+
+;; Savannah bug #28957: exception in inliner, from Helmut Eller
+(test "Type java.lang.CharSequence[]" 'test-savannah-28957
+      (((string[]):getClass):toString))
+
+;; Savannah bug #28926: EOL conversion in READ
+(test "(#\\X #\\return #\\Y #\\return #\\newline #\\Z)" 'test-savannah-28926
+      (format "~w" (let ((in (string #\" #\X #\return #\Y #\return #\newline #\Z #\")))
+		     (string->list (call-with-input-string in read)))))
