@@ -1,6 +1,6 @@
 (module-static #f)
 (module-export my-factorial list-length-2 deldup call-to-first
-	       date mod1-v5
+	       date mod1-v5 <simpleAux> 
 	       test1-import0 mod0-v1 mod0-v2 mod0-v3 mod0-f1 mod0-m1
 	       namespace-syntax-test make-array make-array-fun mA)
 (require <module0>)
